
/***********************************************************************************
CM - Software para programao e manipulao de matrizes

Direitos Autorais Reservados (c) 2004 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os 
termos da Licena Pblica Geral GNU conforme publicada pela Free Software 
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso 
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA 
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes. 

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este 
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59 
Temple Street, Suite 330, Boston, MA 02111-1307 USA. 

***********************************************************************************

Parte integrante do SOC - SOftware Cientfico 

Descrio: 	 
	
            Descricao grafica das principais estruturas de dados           
**	      ====================================================
**                                                                          
**  +----------+
**  | variavel | (tipo: Simbolo_t *)
**  +----------+
**       |
**       |
**       |           (Simbolo_t)
**       +--------->+-----------+
**                  |  nome     |  (tipo: char *)
**                  +-----------+
**                  |  tipo     |  (tipo: int)
**                  +-----------+ 
**      (uniao "u") | mat / fcn |\ (tipos: M_t * & S_t *) (Matriz_t)
**                  +-----------+ +---------------------->+------+
**      lista     / | proximo   |  (tipo: Simbolo_t *)    | nlin | (int)
**      bi-ligada<  +-----------+                         +------+
**                \ | anterior  |  (tipo: Simbolo_t *)    | ncol | (int)
**                  +-----------+                         +------+
**                                                        | mat  |  (double *)
**                                                        +------+
**                                                           |
**                                 +-------------------------+
**                                 |
**                                 +->+---+---+---+---+---+---+---
**                                    |1,1|1,2|1,3|...|2,1|2,2| ...
**                                    +---+---+---+---+---+---+---
**
 
Desenvolvido por: 

	Moacir Pedroso Junior 
	Silvio Evangelista
		
	Informaes do CVS:
	$Source:
	$Revision:
	$Date:

***********************************************************************************/





/* 
**  estrutura que define uma matriz
*/

#ifdef EDIX
#define void	int
#endif

struct Matriz{             /* Matriz */
       int nlin;           /* numero real de linhas */
       int ncol;           /* numero real de colunas */
       double *mat;        /* a matriz propriamente */
};

typedef struct Matriz Matriz_t;





/*
**  estrutura que define um no da tabela de simbolos
*/

struct Simbolo{            /* no da tabela de simbolos */
       char *nome;         /* nome do simbolo */
       int tipo;           /* tipo: ID, INTRINS, INDEF, TEMP */
       union {             /* ponteiro para o elemento */
               Matriz_t *val;               /* se ID, TEMP */
               struct Simbolo *(*fcn)();    /* se INTRINS */
	       int    (*defn)();	    /* Function ou Procedure */
       } u;
                                 /* duplamente ligada para facil remocao */
       struct Simbolo *proximo;  /* proximo da lista */
       struct Simbolo *previo;   /* previo da lista */
};

typedef struct Simbolo Simbolo_t;

typedef char *Datum;

typedef int (*Inst) ();	   /* Instrucao de Maquina */

extern Inst prog[];

/*
**  para acessar componentes a partir de um ponteiro para simbolo
*/

#define STOP 		(Inst) 0
#define NOME(s)         ((s)->nome)
#define TIPO(s)         ((s)->tipo)
#define MATRIZ(s)       ((s)->u.val)
#define NLINS(s)        (((s)->u.val)->nlin)
#define NCOLS(s)        (((s)->u.val)->ncol)
#define DADOS(s)        (((s)->u.val)->mat)
#define PROXIMO(s)      ((s)->proximo)
#define PREVIO(s)       ((s)->previo)

/*
**  para acessar um elemento de matriz . Primeiro elemento tem
**  indices (0,0)  ( 0<=i<=nlin-1, 0<=j<=ncol-1) 
*/

#define DADOij(s,i,j)   (DADOS(s)[(i)*NCOLS(s)+(j)])


/*
**  para acessar componentes a partir de um ponteiro para matriz
*/
#define MATNL(v)        ((v)->nlin)
#define MATNC(v)        ((v)->ncol)
#define ELEMS(v)        ((v)->mat)
#define ELEMij(v,i,j)   (ELEMS(v)[(i)*MATNC(v)+(j)])

/* 
** Definicao do numero maximo de caracteres para nome do arquivo
*/

#define MAXDIR TAMSTR


/*
**  codigos de erros ( para uso de explerro() )
*/

#define       MATINDEFER          1         /* matriz indefinida */
#define       EOFEMCOMER          2         /* eof em comentario */
#define       NOMEGDEER           3         /* nome muito grande */
#define       SINTAXEER           4         /* erro de sintaxe (yacc) */
#define       MEMORIAER           5         /* acabou espaco de memoria */
#define       OPERPFER            6         /* erro ponto flutuante */
#define       MATSOMAER           7         /* soma de matrizes */
#define       MATDIFER            8         /* subtracao de matrizes */
#define       MATEMULER           9         /* multiplicacao elementos */
#define       MATEDIVER           10        /* divisao de elementos */
#define       MATMULTER           11        /* multiplicacao de matrizes */
#define       MATDIVEER           12        /* divisao esquerda */
#define       MATDIVDER           13        /* divisao direita */
#define       MATEPOTER           14        /* exponenciacao elementos */
#define       MATPOTER            15        /* potencia de matrizes */
#define       VCONCER             16        /* concatenacao vertical */
#define       HCONCER             17        /* concatenacao horizontal */
#define       INTER               18        /* interrupcao pelo usuario */


/*
**   variaveis globais
*/

#ifndef EXTVAR
#define EXTVAR extern
#endif

EXTVAR int numerro;               /* numero do erro mais recente */
EXTVAR int debugflag;             /* =1 ==> debug em acao */
EXTVAR int trexec;                /* traco de execucao */
EXTVAR int interativo;            /* =1 ==> modo interativo */

#define dprintf         if (debugflag) printf
#define IFTREX(a,b,c,d) if (trexec) traco(a,b,c,d) 
#define simbprint(rot,simb)  if (debugflag) printf(">>>>> %s: NOME(sp)=%s, nl=%d, nc=%d\n", rot, NOME(simb), NLINS(simb), NCOLS(simb))



/*
**   todas as funcoes
*/

#ifdef MSDOS

/*
** funcoes definidas em  "cm_cm01.y" (ytab.c)
*/

extern char *emalloc(unsigned int);
extern void execerro(char *, char *);
extern void fpecatch(int);
extern void aviso(char *, char *);
extern void traco(char *, Simbolo_t *, Simbolo_t *, Simbolo_t *);
extern void info(void);
extern void conferro(Simbolo_t *, Simbolo_t *, char *);
extern void fim(void);
extern void pprompt(void);
extern void eofarq();
extern void execarq(char *);
extern void efree(char *);

/*
** funcoes definidas em "ts_cm01.c"
*/

extern Simbolo_t *procure(char *);
extern Simbolo_t *instale(char *, int, Matriz_t *);
extern void libere(char *);
extern void libsim(Simbolo_t *);
extern void libtemp(void);
extern void liste(void);
extern void limpe(void);
extern void limpe_f(void);

/*
** funcoes definidas em "in_cm01.c"
*/

extern void init(void);

/*
** funcoes definidas em "ac_cm01.c"
*/

extern void matlista(Simbolo_t *,double,double,char *, int, int);
extern Simbolo_t *atribua(Simbolo_t *, Simbolo_t *);
extern Simbolo_t *nummat(double);
extern Simbolo_t *matsoma(Simbolo_t *, Simbolo_t *);  
extern Simbolo_t *matdif(Simbolo_t *, Simbolo_t *);  
extern Simbolo_t *matemul(Simbolo_t *, Simbolo_t *);  
extern Simbolo_t *matediv(Simbolo_t *, Simbolo_t *);  
extern Simbolo_t *matmult(Simbolo_t *, Simbolo_t *);  
extern Simbolo_t *matdive(Simbolo_t *, Simbolo_t *);  
extern Simbolo_t *matdivd(Simbolo_t *, Simbolo_t *);  
extern Simbolo_t *matepot(Simbolo_t *, Simbolo_t *);  
extern Simbolo_t *matpot(Simbolo_t *, Simbolo_t *);  
extern Simbolo_t *mattran(Simbolo_t *);  
extern Simbolo_t *matneg(Simbolo_t *);  
extern Simbolo_t *vconcat(Simbolo_t *, Simbolo_t *);  
extern Simbolo_t *hconcat(Simbolo_t *, Simbolo_t *);  
extern Simbolo_t *matkron(Simbolo_t *, Simbolo_t *);  
extern void setsubs(Simbolo_t *, Simbolo_t *, Simbolo_t *, Simbolo_t *);
extern Simbolo_t *matsubs(Simbolo_t *, Simbolo_t *, Simbolo_t *);
extern Simbolo_t *genindex(Simbolo_t *, Simbolo_t *);
extern int verdim(Simbolo_t *, Simbolo_t *);  
extern Matriz_t *fazmat(int, int);  
extern void copmat(Matriz_t *, Matriz_t *);  
extern Simbolo_t *faztemp(int, int);  
extern void gennom(char *);  

/*
** funcoes definidas em "ma_cm01.c"
*/

extern Simbolo_t *mSin(Simbolo_t *);
extern Simbolo_t *mCos(Simbolo_t *);
extern Simbolo_t *mAtan(Simbolo_t *);
extern Simbolo_t *mInt(Simbolo_t *);
extern Simbolo_t *mInv(Simbolo_t *);
extern Simbolo_t *mAbs(Simbolo_t *);
extern Simbolo_t *mLog(Simbolo_t *);
extern Simbolo_t *mLog10(Simbolo_t *);
extern Simbolo_t *mExp(Simbolo_t *);
extern Simbolo_t *mSqrt(Simbolo_t *);
extern Simbolo_t *mNlin(Simbolo_t *);
extern Simbolo_t *mNcol(Simbolo_t *);
extern Simbolo_t *mIdent(Simbolo_t *);
extern Simbolo_t *mDiag(Simbolo_t *);
extern Simbolo_t *mVdiag(Simbolo_t *);
extern Simbolo_t *mDesign(Simbolo_t *);
extern Simbolo_t *mSum(Simbolo_t *);
extern Simbolo_t *mLinsum(Simbolo_t *);
extern Simbolo_t *mColsum(Simbolo_t *);
extern Simbolo_t *mSsq(Simbolo_t *);
extern Simbolo_t *mLinssq(Simbolo_t *);
extern Simbolo_t *mColssq(Simbolo_t *);
extern Simbolo_t *mMax(Simbolo_t *);
extern Simbolo_t *mMin(Simbolo_t *);
extern Simbolo_t *mTraco(Simbolo_t *);
extern void 	  meigen(Simbolo_t *, Simbolo_t *, Simbolo_t *);
extern double 	  vererro(double, char *);
extern void 	  mtri(Simbolo_t *, Simbolo_t *, Simbolo_t *, Simbolo_t *);
extern void       mgs(Simbolo_t *, Simbolo_t *, Simbolo_t *);
extern void       mgs1(Simbolo_t *, Simbolo_t *, Simbolo_t *);
extern void	  mdvs(Simbolo_t *,Simbolo_t *,Simbolo_t *,Simbolo_t *);
extern Simbolo_t *mdet(Simbolo_t *);
extern Simbolo_t *mposto(Simbolo_t *);
extern Simbolo_t *mcho(Simbolo_t *);
extern Simbolo_t *minvp(Simbolo_t *);
extern Simbolo_t *mfprob(Simbolo_t *,Simbolo_t *,Simbolo_t *);
extern Simbolo_t *xprob2(Simbolo_t *,Simbolo_t *);
extern Simbolo_t *ordem(Simbolo_t *);

extern int       mcata(Simbolo_t *, Simbolo_t *, Simbolo_t *, Simbolo_t *, int, char *);

/*
** funcoes defindas em "pc_cm01.c"
*/

extern void grave(Simbolo_t *, char *, char *, int);
extern void prossiga(void);

/*
** Funcoes Definidas No initcode.c
*/

extern void initcode(void);
extern void push(Datum);
extern Datum pop(void);
extern Datum libpop(void);
extern void tentaLibSim(Simbolo_t *);
extern Inst *code(Inst);
extern void execute(Inst *);
extern void varpshn(void);
extern void varpshs(void);
extern void varpsint(void);
extern void varnulo(void);
extern void varpush(void);
extern void atribu1(void);
extern void toler(void);
extern void setcol(void);
extern void setdef(void);
extern void setimp(void);
extern void setdir(void);
extern void setsai(void);
extern void setsem(void);
extern void constpush(void);
extern void imprimr(void);
extern void imprim1(void);
extern void imprim2(void);
extern void imprim3(void);
extern void imprim4(void);
extern void imprim5(void);
extern void imprim6(void);
extern void imprim7(void);
extern void matsub1(void);
extern void matsub2(void);
extern void matsub3(void);
extern void setsub1(void);
extern void setsub2(void);
extern void setsub3(void);
extern void hconca1(void);
extern void vconca1(void);
extern void add(void);
extern void sub(void);
extern void mult(void);
extern void divv(void);
extern void divi(void);
extern void multp(void);
extern void multc(void);
extern void dive(void);
extern void divd(void);
extern void kron(void);
extern void epot(void);
extern void pot(void);
extern void tran(void);
extern void multran(void);
extern void matneg1(void);
extern void bltin(void);
extern void mfprob1(void);
extern void mfinv1(void);
extern void mxinv1(void);
extern void mxprob1(void);
extern void mtinv1(void);
extern void mtprob1(void);
extern Simbolo_t *ninv1(Simbolo_t *);
extern Simbolo_t *nprob1(Simbolo_t *);
extern void grave1(void);
extern void grave2(void);
extern void grave3(void);
extern void grave4(void);
extern void leia1(void);
extern void leia2(void);
extern void leia3(void);
extern void genind1(void);
extern void execar1(void);
extern void anote1(void);
extern void anote2(void);
extern void whilecod(void);
extern void ifcode(void);
extern void define(Simbolo_t *);
extern void call(void);
extern void ret(void);
extern void funcret(void);
extern void procret(void);
extern void arg(void);
extern void argsub1(void);
extern void argsub2(void);
extern void argsub3(void);
extern void argassign(void);
extern Simbolo_t * igual(Datum, Datum);
extern Simbolo_t * diff(Datum, Datum);
extern Simbolo_t * maior(Datum, Datum);
extern Simbolo_t * maioreq(Datum, Datum);
extern Simbolo_t * menor(Datum, Datum);
extern Simbolo_t * menoreq(Datum, Datum);
extern void gt(void);
extern void lt(void);
extern void ge(void);
extern void le(void);
extern void eq(void);
extern void ne(void);
extern void and(void);
extern void or(void);
extern void not(void);
extern void existe(void);
extern void meigen1(void);
extern void gs1(void);
extern void mdvs1(void);
extern void mcata1(void);
extern void criamat(void);
extern void contagem(void);
extern void criarnd(void);
extern void mbdiag(void);
extern int fuzz(double,double);
extern void libtmp2(void);
extern void set_parm(void);
extern void reset_parm(void);

/*
** Funcoes para uso do yacc e lex.
*/

extern int yyparse(void);
extern void yyerror(char *);
extern int yylex(void);
extern void defnonly(char *);

/*
** Outras.
*/

extern int rg_(int, int, double *, double *, double *, int *, double *,
		int *, double *, int *);
extern void settrex(void);
extern void efree(char *);
extern void libtmp1(void);
extern void eofarq(void);
extern char *word_k(char *, int, char *);
extern void trim(char *, char *);
extern void info(void);
extern void infocom(void);
extern void infoex(void);
extern void infofun(void);
extern void infoop(void);
extern void infopro(void);
extern Simbolo_t *leia(char *, char *, int, char *, double);
extern pegtok(char *, char, char *, int *);
extern void liste_f(void);
extern void liste_o(void);
extern int pegalista(Simbolo_t *, double *, int, int);
extern void ordena(int, double *);
extern int ptroca(int, Simbolo_t *);
extern void swap(double *, double *);
extern void sswap(int, double *, int, double *, int);
extern int isamax(int, double *, int);
extern void sscal(int, double, double *, int);
extern void saxpy(int, double, double *, int, double *, int);
extern double sdot(int, double *, int, double *, int);
extern void intcatch(int);
extern void minha_fn(int);
extern Simbolo_t *mmulcol(Simbolo_t *, Simbolo_t *);
extern int follow(int, int, int);
extern Simbolo_t *mvetor(Simbolo_t *);
extern Simbolo_t *muvetor(Simbolo_t *);
extern int ver_val_var(char *, int, char *, double);

#else

/*
** funcoes definidas em  "cm_cm01.y" (ytab.c)
*/

extern char *emalloc();
extern void execerro();
extern void fpecatch();
extern void aviso();
extern void traco();
extern void info();
extern void conferro();
extern void fim();
extern void pprompt();
extern void eofarq();
extern void execarq();
extern void efree();

/*
** funcoes definidas em "ts_cm01.c"
*/

extern Simbolo_t *procure();
extern Simbolo_t *instale();
extern void libere();
extern void libsim();
extern void libtemp();
extern void liste();
extern void limpe();
extern void limpe_f();

/*
** funcoes definidas em "in_cm01.c"
*/

extern void init();

/*
** funcoes definidas em "ac_cm01.c"
*/

extern void matlista();
extern Simbolo_t *atribua();
extern Simbolo_t *nummat();
extern Simbolo_t *matsoma();  
extern Simbolo_t *matdif();  
extern Simbolo_t *matemul();  
extern Simbolo_t *matediv();  
extern Simbolo_t *matmult();  
extern Simbolo_t *matdive();  
extern Simbolo_t *matdivd();  
extern Simbolo_t *matepot();  
extern Simbolo_t *matpot();  
extern Simbolo_t *mattran();  
extern Simbolo_t *matneg();  
extern Simbolo_t *vconcat();  
extern Simbolo_t *hconcat();  
extern Simbolo_t *matkron();  
extern void setsubs();
extern Simbolo_t *matsubs();
extern Simbolo_t *genindex();
extern int verdim();  
extern Matriz_t *fazmat();  
extern void copmat();  
extern Simbolo_t *faztemp();  
extern void gennom();  

/*
** funcoes definidas em "ma_cm01.c"
*/

extern Simbolo_t *mSin();
extern Simbolo_t *mCos();
extern Simbolo_t *mAtan();
extern Simbolo_t *mInt();
extern Simbolo_t *mInv();
extern Simbolo_t *mAbs();
extern Simbolo_t *mLog();
extern Simbolo_t *mLog10();
extern Simbolo_t *mExp();
extern Simbolo_t *mSqrt();
extern Simbolo_t *mNlin();
extern Simbolo_t *mNcol();
extern Simbolo_t *mIdent();
extern Simbolo_t *mDiag();
extern Simbolo_t *mVdiag();
extern Simbolo_t *mDesign();
extern Simbolo_t *mSum();
extern Simbolo_t *mLinsum();
extern Simbolo_t *mColsum();
extern Simbolo_t *mSsq();
extern Simbolo_t *mLinssq();
extern Simbolo_t *mColssq();
extern Simbolo_t *mMax();
extern Simbolo_t *mMin();
extern Simbolo_t *mTraco();
extern void 	  meigen();
extern double 	  vererro();
extern void 	  mtri();
extern void       mgs();
extern void       mgs1();
extern void	  mdvs();
extern Simbolo_t *mdet();
extern Simbolo_t *mposto();
extern Simbolo_t *mcho();
extern Simbolo_t *minvp();
extern Simbolo_t *mfprob();
extern Simbolo_t *xprob2();
extern Simbolo_t *ordem();

extern int       mcata();

/*
** funcoes defindas em "pc_cm01.c"
*/

extern Simbolo_t *leia();
extern void grave();
extern void prossiga();

/*
** funcoes definidas no initcode.c
*/

extern void initcode();
extern void push();
extern Datum pop();
extern Datum libpop();
extern void tentaLibSim();
extern Inst *code();
extern void execute();
extern void varpshn();
extern void varpshs();
extern void varpsint();
extern void varnulo();
extern void varpush();
extern void atribu1();
extern void toler();
extern void setcol();
extern void setdef();
extern void setimp();
extern void setdir();
extern void setsai();
extern void setsem();
extern void constpush();
extern void imprimr();
extern void imprim1();
extern void imprim2();
extern void imprim3();
extern void imprim4();
extern void imprim5();
extern void imprim6();
extern void imprim7();
extern void matsub1();
extern void matsub2();
extern void matsub3();
extern void setsub1();
extern void setsub2();
extern void setsub3();
extern void hconca1();
extern void vconca1();
extern void add();
extern void sub();
extern void mult();
extern void divv();
extern void divi();
extern void multp();
extern void multc();
extern void dive();
extern void divd();
extern void kron();
extern void epot();
extern void pot();
extern void tran();
extern void multran();
extern void matneg1();
extern void bltin();
extern void mfprob1();
extern void mfinv1();
extern void mxinv1();
extern void mxprob1();
extern void mtinv1();
extern void mtprob1();
extern Simbolo_t *ninv1();
extern Simbolo_t *nprob1();
extern void grave1();
extern void grave2();
extern void grave3();
extern void grave4();
extern void leia1();
extern void leia2();
extern void leia3();
extern void genind1();
extern void execar1();
extern void anote1();
extern void anote2();
extern void whilecod();
extern void ifcode();
extern void define();
extern void call();
extern void ret();
extern void funcret();
extern void procret();
extern void arg();
extern void argsub1();
extern void argsub2();
extern void argsub3();
extern void argassign();
extern Simbolo_t * igual();
extern Simbolo_t * diff();
extern Simbolo_t * maior();
extern Simbolo_t * maioreq();
extern Simbolo_t * menor();
extern Simbolo_t * menoreq();
extern void gt();
extern void lt();
extern void ge();
extern void le();
extern void eq();
extern void ne();
extern void and();
extern void or();
extern void not();
extern void existe();
extern void meigen1();
extern void gs1();
extern void mdvs1();
extern void mcata1();
extern void criamat();
extern void contagem();
extern void criarnd();
extern void mbdiag();
extern int fuzz();
extern void libtmp2();
extern void set_parm();
extern void reset_parm();

/*
** Funcoes para uso do yacc e lex.
*/

extern int yyparse();
extern void yyerror();
extern int yylex();
extern void defnonly();

/*
** Outras.
*/

extern int rg_();
extern void settrex();
extern void efree();
extern void libtmp1();
extern void eofarq();
extern char *word_k();
extern void trim();
extern void info();
extern void infocom();
extern void infoex();
extern void infofun();
extern void infoop();
extern void infopro();
extern Simbolo_t *leia();
extern int pegtok();
extern void liste_f();
extern void liste_o();
extern int pegalista();
extern void ordena();
extern int ptroca();
extern void swap();
extern void sswap();
extern int isamax();
extern void sscal();
extern void saxpy();
extern double sdot();
extern void intcatch();
extern void minha_fn();
extern Simbolo_t *mmulcol();
extern int follow();
extern Simbolo_t *mvetor();
extern Simbolo_t *muvetor();
extern int ver_val_var();

#endif

/*
** funcoes definidas em  "matpack.c"
*/

#include "matpack.h"

