#include <stdio.h>
#include <math.h>
#include "cm_cm01.h"
#include "pcsgbd.h" /* Luciana */

#ifdef MSDOS
#define MAXNIVEL 89
#else
#define MAXNIVEL 150
#endif

/***********************************************************************************
CM - Software para programao e manipulao de matrizes

Direitos Autorais Reservados (c) 2004 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os 
termos da Licena Pblica Geral GNU conforme publicada pela Free Software 
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso 
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA 
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes. 

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este 
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59 
Temple Street, Suite 330, Boston, MA 02111-1307 USA. 

***********************************************************************************

Parte integrante do SOC - SOftware Cientfico 

Descrio: 	
 
Gera uma matriz de incidncia a partir de um vetor de nmeros inteiros positivos.
Se B  esta matriz de incidncia, a dimenso de B  (nxm) com m igual ao nmero de valores distintos dos elementos 
do argumento e n o nmero de elementos do argumento.
Exemplo:

B = DELIN(A2);

B    Col1   Col2   Col3
Lin1    1        0        0
Lin2    1        0        0
Lin3    0        1        0
Lin4    0        0        1
Lin5    0        1        0


Desenvolvido por: 

	Moacir Pedroso Junior 
	Silvio Evangelista
		
	Informaes do CVS:
	$Source:
	$Revision:
	$Date:

***********************************************************************************/

Simbolo_t *mDesign(x)
Simbolo_t *x;
{
	Simbolo_t *sr;
	int flag;
	int nl, nc, i, j;
	double *l_elem;
	double t;
#ifdef EDIX
	double *dp;
#endif

	if ( NCOLS(x) != 1 && NLINS(x) != 1)

/* *** Padroniz. msg erro - Luciana 23/12/94
		execerro("delin: ", "argumento nao e' vetor coluna"); 
*** */
		execerro(PegaMsg(3123), (char *) 0);

	if (NLINS(x) >= NCOLS(x))  {
		nl = NLINS(x); 
		flag = 1;
	}
	else {
		nl = NCOLS(x); 
		flag = 0;
	}
	l_elem = (double *) emalloc((unsigned int) (MAXNIVEL*sizeof(double)));

	nc = pegalista(x, l_elem, flag, nl);
	ordena(nc,l_elem);

	sr = faztemp(nl,nc);

	for (i = 0; i < nl; i++) {
		for (j = 0; j < nc; j++) {
#ifdef EDIX
			dp = &(DADOij(sr, i, j));
			*dp = 0.0;
#else
			DADOij(sr, i, j) = 0.0;
#endif
		}
	}

	for (i = 0; i < nl; i++) {
		if(flag) 
			t = DADOij(x,i,0);
		else
			t = DADOij(x,0,i);
		for(j=0; j < nc; j++) {
			if ((t == l_elem[j]))
				break;
		}
		
#ifdef EDIX
		dp = &(DADOij(sr, i, j ));
		*dp = 1.0;
#else
		DADOij(sr, i, j ) = 1.0;
#endif
	}
	efree((char *) l_elem);
	return sr;
}


/*--------------------------------------------------------------------- */
/*									*/
/* Funcao: Pegalista							*/
/*	   Retorna o numero de niveis diferentes de um vetor		*/
/*									*/
/*--------------------------------------------------------------------- */

int pegalista(x, l_elem,flag,nl)
Simbolo_t *x;
double *l_elem;
int flag;
int nl;
{
	int i;
	int j;
	int nc;
	double t;

	nc = 0;
	l_elem[nc++] = DADOij(x,0,0);

	for(i=1; i < nl; i++) {
		if (nc >= MAXNIVEL)  {

/* *** Padroniz. msg erro - Luciana 23/12/94
			execerro("delin: ","Numero maximo de niveis e' 89"); 
*** */
			execerro(PegaMsg(1104), (char *) 0);
		}
		else  {
			if (flag)
				t = DADOij(x,i,0);
			else
				t = DADOij(x,0,i);

			for(j=0; j < nc; j++) {
				if (l_elem[j] == t)
					break;
			}
			if (j >= nc)
				l_elem[nc++] = t;
		}
	}
	return(nc);
}

/* ------------------------------------------------------------------------
** ordena()
** 	Programado por:	Silvio Evangelista
**	Data:		Outubro 1986
**	Descricao:	ordena o vetor de numeros
**	Variaveis ent.	com_med  - valor da media.
**			com_nv   - numero de obs.
**	nesultado:	vetor ordenado      
** ------------------------------------------------------------------------- */

void ordena(nc , l_elem)
int nc;
double *l_elem;
{
	int inter, j;
	double temp1;

	for(inter=nc-1;inter>0;--inter) {
		for(j=0; j < inter ; ++j) {
			if (l_elem[j] > l_elem[inter]) {
				temp1  = l_elem[j];
				l_elem[j] = l_elem[inter];
				l_elem[inter]  = temp1;
			}
		}
	}
}
