#include <stdio.h>
#include <math.h>
#include "cm_cm01.h"
#include "pcsgbd.h" /* Luciana */

/***********************************************************************************
CM - Software para programao e manipulao de matrizes

Direitos Autorais Reservados (c) 2004 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os 
termos da Licena Pblica Geral GNU conforme publicada pela Free Software 
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso 
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA 
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes. 

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este 
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59 
Temple Street, Suite 330, Boston, MA 02111-1307 USA. 

***********************************************************************************

Parte integrante do SOC - SOftware Cientfico 

Descrio: 	
 
Calcula a decomposio em valor singular de uma matriz retangular de dimenso (mxn) com m >= n. Esta funo
 retorna trs variveis: o vetor dos valores singulares e duas matrizes associadas a estes valores.
Ela s pode ser usada com o comando de atribuio (=).  necessrio atribuir trs variveis de uma s vez, 
separando-as por vrgula (,). A primeira identifica o vetor dos valores singulares.
Se B, C e D representam estas variveis respectivamente,
  C' * arg * D = DI ,
onde 
DI  uma matriz diagonal formada pelos elementos de B (valores sigulares).
As dimenses de B, C e D so  (nx1), (mxn) e (nxn) respectivamente.



Desenvolvido por: 

	Moacir Pedroso Junior 
	Silvio Evangelista
		
	Informaes do CVS:
	$Source:
	$Revision:
	$Date:

***********************************************************************************/

extern double _toler_;

#define TOL _toler_
#ifdef MSDOS
# define	EPS	1.0e-15
#else
# define	EPS	1.0e-15
#endif

void mdvs(a,q,u,v)
Simbolo_t *a;
Simbolo_t *q;
Simbolo_t *u;
Simbolo_t *v;
{
	int i,j,k,l,l1,m,n;
	double c,f,g,h,s,x,y,z,eps;
	Simbolo_t *e = faztemp(NCOLS(a),1);
#ifdef EDIX
	double *dp;
#endif
	n = NCOLS(a);
	m = NLINS(a);
	eps = TOL;

	if (n > m)

/* *** Padroniz. msg erro - Luciana 23/12/94
		execerro("Num. de colunas maior que num. de linhas",(char *) 0);
*** */
		execerro(PegaMsg(3125),(char *) 0);

	for(i=0; i < m; ++i)
		for(j=0; j < n; ++j) {
#ifdef EDIX
			dp = &(DADOij(u,i,j));
			*dp = DADOij(a,i,j);
#else
			DADOij(u,i,j) = DADOij(a,i,j);
#endif
		}
	g = x = 0.;
	   
	for(i=0; i < n; ++i) {
#ifdef EDIX
		dp = &(DADOij(e,i,0));
		*dp = g;
#else
		DADOij(e,i,0) = g;
#endif
		s = 0.;
		l = i + 1;
		for(j=i; j < m; ++j)
			s += DADOij(u,j,i)* DADOij(u,j,i);
		if( s < EPS)
			g = 0.;
		else {
			f = DADOij(u,i,i);
			if (f < 0.)
				g = sqrt(s);
			else
				g = -sqrt(s);
			h = f*g-s;
#ifdef EDIX
			dp = &(DADOij(u,i,i));
			*dp = f - g;
#else
			DADOij(u,i,i) = f - g;
#endif
			if (l < n)  {
				for(j=l; j < n; ++j) {
					s = 0.;
					for(k=i; k < m; ++k)
						s += DADOij(u,k,i)* DADOij(u,k,j);
					f = s / h;
					for(k=i; k < m; ++k) {
#ifdef EDIX
						dp = &(DADOij(u,k,j));
						*dp += f* DADOij(u,k,i);
#else
						DADOij(u,k,j) += f* DADOij(u,k,i);
#endif
					}
				}
			}
		}

#ifdef EDIX
		dp = &(DADOij(q,i,0));
		*dp = g;
#else
		DADOij(q,i,0) = g;
#endif
		s = 0.;
		if (l < n) {
			for(j=l; j < n; ++j)
				s += DADOij(u,i,j)* DADOij(u,i,j);
			if (s < EPS)
				g = 0.;
			else {
				f = DADOij(u,i,i+1);
				if (f < 0.) 
					g = sqrt(s);
				else
					g = -sqrt(s);
				h = f*g-s; 
#ifdef EDIX
				dp = &(DADOij(u,i,i+1));
				*dp = f - g;
#else
				DADOij(u,i,i+1) = f - g;
#endif
				for(j=l; j < n; ++j) {
#ifdef EDIX
					dp = &(DADOij(e,j,0));
					*dp = DADOij(u,i,j)/h;
#else
					DADOij(e,j,0) = DADOij(u,i,j)/h;
#endif
				}
				for(j=l; j < m; ++j) {
					s = 0.;
					for(k=l; k < n; ++k)
						s += DADOij(u,j,k)* DADOij(u,i,k);
					for(k=l; k < n; ++k) {
#ifdef EDIX
						dp = &(DADOij(u,j,k));
						*dp += s* DADOij(e,k,0);
#else
						DADOij(u,j,k) += s* DADOij(e,k,0);
#endif
					}
				}
			}
		}
		y = fabs( DADOij(q,i,0)) + fabs( DADOij(e,i,0));
		if (y > x)
			x = y;
	}
	/* transformacao qr, v */
	for(i=n-1; i >= 0; --i) {
		if (i < (n-1)) {
			if (g != 0.) {
				h =  DADOij(u,i,i+1)*g;
				for(j=l; j < n; ++j) {
#ifdef EDIX
					dp = &(DADOij(v,j,i));
					*dp = DADOij(u,i,j) / h;
#else
					DADOij(v,j,i) = DADOij(u,i,j) / h;
#endif
				}
				for(j=l; j < n; ++j) {
					s = 0.;
					for(k=l; k < n; ++k)
						s += DADOij(u,i,k)* DADOij(v,k,j);
					for(k=l; k < n; ++k) {
#ifdef EDIX
						dp = &(DADOij(v,k,j));
						*dp += s* DADOij(v,k,i);
#else
						DADOij(v,k,j) += s* DADOij(v,k,i);
#endif
					}
				}
			}
			for(j=l; j < n; ++j) {
#ifdef EDIX
				dp = &(DADOij(v,j,i));
				*dp = 0.;
				dp = &(DADOij(v,i,j));
				*dp = 0.;
#else
				DADOij(v,i,j) = DADOij(v,j,i) = 0.;
#endif
			}
		}
#ifdef EDIX
		dp = &(DADOij(v,i,i));
		*dp = 1.;
#else
		DADOij(v,i,i) = 1.;
#endif
		g = DADOij(e,i,0);
		l = i;
	}

	/* transformacao lr, u */
	for(i=n-1; i >= 0; --i) {
		l = i + 1;
		g = DADOij(q,i,0);
		if (i < (n-1)) {
			for(j=l; j < n;  ++j) {
#ifdef EDIX
				dp = &(DADOij(u,i,j));
				*dp = 0.0;
#else
				DADOij(u,i,j) = 0.0;
#endif
			}
		}
		if(g != 0.) {
			h = DADOij(u,i,i)*g;
			if (i < (n-1)) {
				for(j=l; j < n; ++j) {
					s = 0.;
					for(k=l; k < m; ++k)
						s += DADOij(u,k,i)* DADOij(u,k,j);
					f = s/h;
					for(k=i; k < m; ++k) {
#ifdef EDIX
						dp = &(DADOij(u,k,j));
						*dp += f* DADOij(u,k,i);
#else
						DADOij(u,k,j) += f* DADOij(u,k,i);
#endif
					}
				}
			}
			for(j=i; j < m; ++j) {
#ifdef EDIX
				dp = &(DADOij(u,j,i));
				*dp = DADOij(u,j,i)/g;
#else
				DADOij(u,j,i) = DADOij(u,j,i)/g;
#endif
			}
		}
		else  
			for(j=i; j < m; ++j) {
#ifdef EDIX
				dp = &(DADOij(u,j,i));
				*dp = 0.;
#else
				DADOij(u,j,i) = 0.;
#endif
			}
#ifdef EDIX
		dp = &(DADOij(u,i,i));
		*dp = DADOij(u,i,i) + 1.;
#else
		DADOij(u,i,i) = DADOij(u,i,i) + 1.;
#endif
	}

	/* Diagonalizacao da forma bidiagonal */

	eps = eps*x;
	for(k=n-1; k >= 0; --k) {
		test_split:
		for(l=k; l >= 0; --l) {
			if (fabs( DADOij(e,l,0)) <= eps)
				goto test_conv;
			if (fabs( DADOij(q,l-1,0)) <= eps)
				goto cancela;
		}

		/* cancelamento de e[l], se l > 1 */

		cancela:
		c = 0.;
		s = 1.;
		l1 = l - 1;
		for(i=l; i <= k; ++i) {
			f = s* DADOij(e,i,0);
#ifdef EDIX
			dp = &(DADOij(e,i,0));
			*dp = c* DADOij(e,i,0);
#else
			DADOij(e,i,0) = c* DADOij(e,i,0);
#endif
			if (fabs(f) <= eps)
				goto test_conv;
			g = DADOij(q,i,0);
			h = DADOij(q,i,0) = sqrt(f*f+g*g);
			c = g / h;
			s = -f/h;
			   
			for(j=0; j < m; ++j) {
				y = DADOij(u,j,l1);
				z = DADOij(u,j,i);
#ifdef EDIX
				dp = &(DADOij(u,j,l1));
				*dp = y*c+z*s;
				dp = &(DADOij(u,j,i));
				*dp  = -y*s+z*c;
#else
				DADOij(u,j,l1) = y*c+z*s;
				DADOij(u,j,i)  = -y*s+z*c;
#endif
			}
		}
		test_conv:
		z = DADOij(q,k,0);
		if ( l == k)
			goto converge;
		x = DADOij(q,l,0);
		y = DADOij(q,k-1,0);
		g = DADOij(e,k-1,0);
		h = DADOij(e,k,0);
		f = ((y-z)*(y+z)+(g-h)*(g+h))/(2.0*h*y);
		g = sqrt(f*f+1.);
		f = ((x-z)*(x+z)+h*((y/((f < 0.)? (f-g):(f+g)))-h))/x; 
		
		c = s = 1.;
		for(i=l+1; i <= k; ++i) {
			g = DADOij(e,i,0);
			y = DADOij(q,i,0);
			h = s*g;
			g = c*g;
#ifdef EDIX
			dp = &(DADOij(e,i-1,0));
			*dp = z = sqrt(f*f+h*h);
#else
			DADOij(e,i-1,0) = z = sqrt(f*f+h*h);
#endif
			c = f/z;
			s = h/z;
			f = x*c+g*s;
			g = (-x)*s+g*c;
			h = y*s;
			y = y*c;
			for(j=0; j < n; ++j) {
				x = DADOij(v,j,i-1);
				z = DADOij(v,j,i);
#ifdef EDIX
				dp = &(DADOij(v,j,i-1));
				*dp = x*c+z*s;
				dp = &(DADOij(v,j,i));
				*dp = -x*s + z*c;
#else
				DADOij(v,j,i-1) = x*c+z*s;
				DADOij(v,j,i) = -x*s + z*c;
#endif
			}
#ifdef EDIX
			dp = &(DADOij(q,i-1,0));
			*dp = z = sqrt(f*f + h*h);
#else
			DADOij(q,i-1,0) = z = sqrt(f*f + h*h);
#endif
			if (fabs(z) > EPS) {
				c = f/z;
				s = h/z;
			}
			f = c*g + s*y;
			x = -s*g + c*y;
		
			for(j=0; j < m; ++j) {
				y = DADOij(u,j,i-1);
				z = DADOij(u,j,i);
#ifdef EDIX
				dp = &(DADOij(u,j,i-1));
				*dp = y*c + z*s;
				dp = &(DADOij(u,j,i));
				*dp = -y*s + z*c;
#else
				DADOij(u,j,i-1) = y*c + z*s;
				DADOij(u,j,i) = -y*s + z*c;
#endif
			}
		}
#ifdef EDIX
		dp = &(DADOij(e,l,0));
		*dp = 0.;
		dp = &(DADOij(e,k,0));
		*dp = f;
		dp = &(DADOij(q,k,0));
		*dp = x;
#else
		DADOij(e,l,0) = 0.;
		DADOij(e,k,0) = f;
		DADOij(q,k,0) = x;
#endif
		goto test_split;

		converge:
		if (z < 0.) {
#ifdef EDIX
			dp = &(DADOij(q,k,0));
			*dp = -z;
#else
			DADOij(q,k,0) = -z;
#endif
			for(j=0; j < n; ++j) {
#ifdef EDIX
				dp = &(DADOij(v,j,k));
				*dp = -DADOij(v,j,k);
#else
				DADOij(v,j,k) = -DADOij(v,j,k);
#endif
			}
		}
	}

	tentaLibSim(e);
}
