
/***********************************************************************************
 CORREL - Constri a matriz de correlao (Pearson) entre as varivies. 

 Direitos Autorais Reservados (c) 2004 Embrapa Informtica Agropecuria

 Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
 termos da Licena Pblica Geral GNU conforme publicada pela Free Software
 Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
 posterior.

 Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
 GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
 FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

 Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
 programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
 Temple Street, Suite 330, Boston, MA 02111-1307 USA.

 **********************************************************************************

 Parte integrante do SOC - SOftware Cientfico

 Descrio:

	Faz o parsing do correl.

 Desenvolvido por:

	Dalton Francisco Andrade
	Maria Fernanda Moura
	Slvio Roberto Medeiros Evangelista

 Informaes do CVS:
        $Source: /cvsroot/soc/soc/src/correl/fontes/lerpar.c,v $:
        $Revision: 1.1 $:
        $Date: 2005/03/22 18:13:11 $:

***********************************************************************************/

#include "correl.h"
#include <string.h>
#include "getoken.h"

static char *sccsid = "@(#)lerpar.c	3.5	(NTIA/correl)	97/02/13";


void token_tl()
{
    int i = 0;
    int c;

    while ((c = xgetc(stdin)) == ' ' || c == '\t' || c == '\n'
	   || c == '\r');
    while (c != ';' && i < 127) {
	titulo[i++] = (char) c;
	c = xgetc(stdin);
    }
    titulo[i] = '\0';
}

void token_var()
{
    int i;
    int c;
    double mmin;
    double mmax;
    char token[128];
    char tipo;
    long ret;

    ncom = nvar = 0;


    while ((c = getoken(token)) != PTVG && strcmpi(token, "com") != 0) {
	if (c != VARIAV)
	    erro(5, 0);		/* Esperava uma variavel */

	if ((i = veri_var(fd, token)) < 0)
	    erro(3, 0);		/* Variavel nao existe */

	if ((ret =
	     (long) pega_var(fd, i, vvar[nvar], &tipo, (MINVAL *) & mmin,
			     (MAXVAL *) & mmax)) < 0L) {
	    fprintf(stderr, PegaMsg(1062));
#ifdef MEM_DEBUG
	    if (Mem_Used() != 0) {
		printf("***Lista de memoria nao esta vazia ***\n");
		Mem_Store("correl");
		exit(0);
	    }
#endif
	    setOSstdin();
	    exit(2);
	}
	if (tipo != 'n')
	    erro(7, 0);

	var[nvar++] = i;
    }

    if (strcmpi(token, "com") == 0) {
	while ((c = getoken(token)) != PTVG) {
	    if (c != VARIAV)
		erro(5, 0);	/* Token invalido */

	    if ((i = veri_var(fd, token)) < 0)
		erro(3, 0);	/* Variavel nao existe */

	    if ((ret =
		 (long) pega_var(fd, i, vcom[ncom], &tipo,
				 (MINVAL *) & mmin,
				 (MAXVAL *) & mmax)) < 0L) {
		fprintf(stderr, PegaMsg(1062));
#ifdef MEM_DEBUG
		if (Mem_Used() != 0) {
		    printf("***Lista de memoria nao esta vazia ***\n");
		    Mem_Store("correl");
		    exit(0);
		}
#endif

		setOSstdin();
		exit(2);
	    }
	    if (tipo != 'n')
		erro(7, 0);
	    com[ncom++] = i;
	}
    }
}

void token_class()
{
    int i;
    int c;
    char token[128];
    double mmin, mmax;
    char tipo;
    long ret;

    nclass = 0;

    while ((c = getoken(token)) != PTVG && strcmpi(token, "ord") != 0) {
	if (nclass >= MAXCLASS)
	    erro(8, 0);
	if (c != VARIAV)
	    erro(5, 0);		/* Token invalido */

	if ((i = veri_var(fd, token)) < 0)
	    erro(3, 0);		/* Variavel nao existe */

	strcpy(vclass[nclass], token);
	if ((ret =
	     (long) pega_var(fd, i, vclass[nclass], &tipo,
			     (MINVAL *) & mmin, (MAXVAL *) & mmax)) < 0L) {
	    fprintf(stderr, PegaMsg(1062));
#ifdef MEM_DEBUG
	    if (Mem_Used() != 0) {
		printf("***Lista de memoria nao esta vazia ***\n");
		Mem_Store("correl");
		exit(0);
	    }
#endif

	    setOSstdin();
	    exit(2);
	}
	if (tipo == 'n')
	    tclass[nclass] = 1;	/* tipo numerica */
/* OSCAR: Ate' a marca. */
	else if (tipo == 'd')
	    tclass[nclass] = 2;	/* tipo data */
/* XXXXXXXXXXXXXXXXXXXX */
	else
	    tclass[nclass] = 0;	/* tipo alfa */
	class[nclass++] = i;
    }

    ordc = 0;
    if (c != PTVG) {		/* tem ord */
	if ((c = getoken(token)) != IGUAL)
	    erro(11, 0);
	c = getoken(token);
	if (strcmpi(token, "asc") == 0) {
	    ordc = 0;		/* ascendente */
	} else if (strcmpi(token, "desc") == 0) {
	    ordc = 1;		/* descendente */
	} else if (strcmpi(token, "nao") == 0) {
	    ordc = 2;		/* nao ordenado */
	} else {
	    erro(9, 0);
	}
	if ((c = getoken(token)) != PTVG) {
	    erro(10, 0);
	}
    }
}

void ler_par()
{
    int c;
    int i;
    char token[128];
    double mmin, mmax;
    int n;
    char tipo;
    long ret;

    while ((c = getoken(token)) != EOF && c != RB) {
	switch (c) {
	case VARIAV:
	    if (strcmpi(token, "tl") == 0)
		token_tl();
	    else if (strcmpi(token, "class") == 0)
		token_class();
	    else if (strcmpi(token, "var") == 0)
		token_var();
	    else
		erro(4, 0);
	    break;
	case PTVG:
	    break;
	default:
	    erro(4, 0);
	    break;
	}
    }
    if (nvar == 0) {
	n = fd->nvar;		/* o numero de variaveis */
	for (i = 0; i < n; ++i) {
	    if ((ret =
		 (long) pega_var(fd, i, token, &tipo, (MINVAL *) & mmin,
				 (MAXVAL *) & mmax)) < 0L) {
		fprintf(stderr, PegaMsg(1062));
#ifdef MEM_DEBUG
		if (Mem_Used() != 0) {
		    printf("***Lista de memoria nao esta vazia ***\n");
		    Mem_Store("correl");
		    exit(0);
		}
#endif

		setOSstdin();
		exit(2);
	    }
	    if (tipo == 'n') {
		if (nvar >= MAXMEM) {
		    erro(0, 0);
		} else {
		    strcpy(vvar[nvar], token);
		    var[nvar++] = i;
		}
	    }
	}
    }

    if (nvar == 0) {
	erro(6, 1);
    }
    if (ncom == 0) {
	temcom = 1;		/* nao tem 'com' */
	ncom = nvar;		/* coloca com == var */
	for (i = 0; i < ncom; ++i) {
	    com[i] = var[i];
	    strcpy(vcom[i], vvar[i]);
	}
    } else
	temcom = 0;		/* tem comando 'com' */
}
