/***********************************************************************************
 ESTAT - Clculo de Estatsticas Descritivas
 
 Direitos Autorais Reservados (c) 2004 Embrapa Informtica Agropecuria

 Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
 termos da Licena Pblica Geral GNU conforme publicada pela Free Software
 Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
 posterior.

 Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
 GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
 FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

 Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
 programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
 Temple Street, Suite 330, Boston, MA 02111-1307 USA.

 **********************************************************************************

 Parte integrante do SOC - SOftware Cientfico

 Descrio:

    Mdulo integrante  do software ESTAT, que  calcula estatsticas descritivas 
    de variveis  numricas. As  estatsticas  que  podem  ser requeridas so: nmero 
    de observaes, valor mnimo, valor mximo, total, mdia, varincia, desvio padro, 
    erro padro, coeficiente de variao, assimetria, curtose, mediana, estatstica D 
    de Kolmogorov, percentis e modas.
    Neste mdulo encontram-se as rotinas referentes aos cclculos com base na tabela de
    hashing, que so: moda, percentis e D de Kolmogorov. 
    Rotinas :
    	. calc_moda    : calcula a moda para uma tabela de hashing
	. calc_percentil  : calcula o percentil dado
	. calc_kolmogorov : calcula a estatistica de	kolmogorov

 Desenvolvido por:

	  Dalton Francisco Andrade
	  Maria Fernanda Moura
	  Maurcio Noronha Festa

 Informaes do CVS:
        $Source:
        $Revision:
        $Date:

***********************************************************************************/

#include "estat.h"

#define _toler_ 1.e-10

extern int v_hash;

Lista calc_moda(tam)
int tam;
{
    Lista lsnome;
    struct simb *p;
    int i, j, k;
    double freq, v_freq[5], a, a1;
    int v_nome[5], b, b1;
    int existe_moda = 0;

    lsnome = L_Create();

    for (j = 0; j < 5; j++) {
	v_nome[j] = -1;
	v_freq[j] = (double) -1;
    }
    for (i = 0; i < tam; i++) {
	p = pesquisa(nomes[0][i], tabhash[0]);
	freq = p->def;
	if (freq > 1.0) {
	    /* monta o vetor de modas ordenado por frequencia */
	    for (j = 0; j < 5; j++) {
		if (freq >= v_freq[j]) {
		    a = v_freq[j];
		    b = v_nome[j];
		    v_freq[j] = freq;
		    v_nome[j] = i;
		    for (k = j + 1; k < 5; k++) {
			a1 = v_freq[k];
			b1 = v_nome[k];
			v_freq[k] = a;
			v_nome[k] = b;
			a = a1;
			b = b1;
		    }
		    break;
		}
	    }
	}
    }
    /* lista de modas esperada pelo programa original */
    if (v_nome[0] > -1) {
	ins(lsnome, nomes[0][v_nome[0]]);
	for (j = 1; (j < 5) && (v_freq[j] >= v_freq[j - 1]); j++) {
	    ins(lsnome, nomes[0][v_nome[j]]);
	}
    }

    return (lsnome);
}

/* -------------- Calc_Percentil -----------------------------------*/

double calc_percentil(tam, p)
int tam;
double p;
{
    double valor, score, freq, f, x1, x2;
    int i;
    struct simb *q;

    score = wfreq[v_hash] * p;
    i = 0;
    freq = 0.0;
    do {
	q = pesquisa(nomes[0][i], tabhash[0]);
	freq = freq + q->def;
	++i;
    } while ((lfuzz(freq, score) == -1) && (i < tam));
    --i;
    f = fabs(freq - score);
    if (f < _toler_ && (i < (tam - 1))) {
	x1 = atof(nomes[0][i]);
	x2 = atof(nomes[0][i + 1]);
	valor = (x1 + x2) / 2.0;
    } else {
	if ((f > _toler_) && (lfuzz(f, 1.0) == -1)) {
	    if ((q->def > 1.0) || (i == 0)) {
		valor = atof(nomes[0][i]);
	    } else {
		x1 = atof(nomes[0][i - 1]);
		x2 = atof(nomes[0][i]);
		valor = x1 + (x2 - x1) * (1 - f);
	    }
	} else
	    valor = atof(nomes[0][i]);
    }
    return (valor);
}

/* -------------- Calc_kolmogorov -------------------------------*/

double calc_kolmogorov(tam)
int tam;
{
    int i;
    double maior = 0.0;
    double media, desvio, z, d, x;

    media = xmean2[v_hash];
    if (!(lfuzz(wfreq[v_hash], 1.0) == 1)
	|| (lfuzz(s22[v_hash], 0.0) == 0)) {
	maior = MISSING;
    } else {
	desvio = sqrt(s22[v_hash] / (wfreq[v_hash] - 1));
	for (i = 0; i < tam; i++) {
	    x = atof(nomes[0][i]);
	    z = (x - media) / desvio;
	    x = (double) (i + 1) / (double) tam;
	    d = fabs(x - nprob(z));
	    if (lfuzz(maior, d) == -1)
		maior = d;
	}
    }
    return (maior);
}
