
/***********************************************************************************
 FREQ - Gera tabelas para descrever a distribuio da frequencia dos valores

 Direitos Autorais Reservados (c) 2004 Embrapa Informtica Agropecuria

 Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
 termos da Licena Pblica Geral GNU conforme publicada pela Free Software
 Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
 posterior.

 Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
 GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
 FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

 Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
 programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
 Temple Street, Suite 330, Boston, MA 02111-1307 USA.

 **********************************************************************************

 Parte integrante do SOC - SOftware Cientfico

 Descrio:

	Funcoes referentes ao calculo das estatisticas do modulo

 Desenvolvido por:

	Dalton Francisco Andrade
	Sonia Ternes
	Slvio Roberto Medeiros Evangelista

 Informaes do CVS:
        $Source: /cvsroot/soc/soc/src/freq/fontes/estat.c,v $:
        $Revision: 1.1 $:
        $Date: 2005/03/22 19:07:20 $:

***********************************************************************************/


#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include "pcsgbd.h"		/* para uso de rounddec() */
#include "freq.h"

#ifdef MEM_DEBUG
#include "mshell.h"
#endif

static char *sccsid = "@(#)estat.c	3.4	(NTIA/freq)	95/07/26";

#define  PI 3.141592653589793
#define  TOL 1.e-8

/* Calculo dos totais marginais de linha e coluna */
/*  Comando analisado para impressao do tipo : E*D*C*A*B  */

long *som_lin(n_obs, mat, num)
int *n_obs;
long *mat;			/* ponteiro para 'matriz' de frequencia */
int num;
{

    int nproc;			/* no de linhas de tab ==> A */
    int nelem;			/* no. de obs da var corresp. a coluna de tab ==> B */
    int nsalt;			/* (no. de obs. var linha) * (var coluna) :
				   dim = 1 ==> B
				   dim = 2,3,4,5 ==> A*B */
    int nvez;			/* no. de vezes em que vamos somar :
				   dim = 2 ==> nvez = 1
				   dim = 3 ==> nvez = C
				   dim = 4 ==> nvez = D*C
				   dim = 5 ==> nvez = E*D*C */

    long *somalin;
    int ii, k, j, inc1, inc;
    long soma;

    nproc = n_obs[num - 2];
    nelem = n_obs[num - 1];

    nsalt = nproc * nelem;

    switch (num) {

    case 2:
	nvez = 1;
	break;

    case 3:
	nvez = n_obs[0];
	break;

    case 4:
	nvez = n_obs[0] * n_obs[1];
	break;

    case 5:
	nvez = n_obs[0] * n_obs[1] * n_obs[2];
	break;
    }

/*
	somalin = (long *) aloc(nproc,sizeof (long));
*/
    somalin = (long *) calloc(nproc, sizeof(long));

    for (ii = 0; ii < nproc; ++ii) {
	soma = 0L;
	inc1 = nelem * ii;
	for (k = 0; k < nvez; ++k) {
	    inc = k * nsalt + inc1;
	    for (j = 0; j < nelem; ++j) {
		soma += mat[j + inc];
	    }
	}
	somalin[ii] = soma;
    }
    return (somalin);
}

/* ================================================================= */

/*  Comando analisado para impressao do tipo : E*D*C*A*B  */

long *som_col(n_obs, mat, num)
int *n_obs;
long *mat;			/* ponteiro para 'matriz' de frequencia */
int num;
{


    int nelem;			/* no. de vezes em que vamos somar :
				   dim = 2 ==> nvez = A
				   dim = 3 ==> nvez = C*A
				   dim = 4 ==> nvez = D*C*A
				   dim = 5 ==> nvez = E*D*C*A */
    int nproc;			/* no de colunas de tab ==> B */

    int ii, j, inc;
    long *somacol, soma;


    nproc = n_obs[num - 1];

    switch (num) {

    case 2:
	nelem = n_obs[0];
	break;

    case 3:
	nelem = n_obs[0] * n_obs[1];
	break;

    case 4:
	nelem = n_obs[0] * n_obs[1] * n_obs[2];
	break;

    case 5:
	nelem = n_obs[0] * n_obs[1] * n_obs[2] * n_obs[3];
	break;
    }

/*
	somacol = (long *) aloc(nproc,sizeof(long));
*/
    somacol = (long *) calloc(nproc, sizeof(long));

    for (j = 0; j < nproc; ++j) {
	soma = 0L;
	for (ii = 0; ii < nelem; ++ii) {
	    inc = ii * nproc + j;
	    soma += mat[inc];
	}
	somacol[j] = soma;
    }
    return (somacol);
}


/* ================================================================= */

/* Impressao do qui-quadrado , gl e p-value */

void imp_x(qui, no, dim)	/* qui-quadrado da tabela */
double qui;
int *no;			/* vetor de dimensoes */
int dim;			/* dimensao de no */
{
    int gl;			/* grau de liberdade */
    double pv;			/* p-value */

    switch (dim) {
    case 2:
	gl = (no[0] - 1) * (no[1] - 1);
	if (gl > 0)
	    pv = xprob(qui, (double) gl);
	break;

    case 3:
	gl = (no[0] - 1) * (no[1] - 1) * (no[2] - 1);
	if (gl > 0)
	    pv = xprob(qui, (double) gl);
	break;

    case 4:
	gl = (no[0] - 1) * (no[1] - 1) * (no[2] - 1) * (no[3] - 1);
	if (gl > 0)
	    pv = xprob(qui, (double) gl);
	break;

    case 5:
	gl = (no[0] - 1) * (no[1] - 1) * (no[2] - 1) *
	    (no[3] - 1) * (no[4] - 1);
	if (gl > 0)
	    pv = xprob(qui, (double) gl);
	break;
    }


    if (gl > 0) {
	printf("\n");
	printf("\n      %12s %3s %4s %3s ", "qui-quadrado", " ", "gl",
	       " ");
	printf("%17s", "PR > qui-quadrado");
	printf("\n      %12s %3s %4s %3s %17s", "------------", " ", "--",
	       " ", "-----------------");
	printf("\n");
	/*
	   printf("\n      %12.3f %8d %21.3f\n",qui,gl,pv);
	 */
	printf("\n      %12s %8d %21.3f\n", rounddec(qui, 12, 3), gl, pv);
    }
}
