/***********************************************************************************
 Genese - Linguagem de programao para a gerao e manipulao de arquivo .dat

 Direitos Autorais Reservados (c) 2004 Embrapa Informtica Agropecuria

 Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
 termos da Licena Pblica Geral GNU conforme publicada pela Free Software
 Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
 posterior.

 Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
 GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
 FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

 Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
 programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
 Temple Street, Suite 330, Boston, MA 02111-1307 USA.

 ***********************************************************************************

 Parte integrante do SOC - SOftware Cientfico

 Descrio:

	Arquivo que contem a funo main do mdulo genese

 Desenvolvido por:

      Carlos Eduardo Assis Paniago
	Oscar Ivan Palma Pacheco
      Carla Geovana do N. Macario
	Maria Fernanda Moura
	Adriana Delfino dos Santos
	

 Informaes do CVS:
        $Source:
        $Revision:
        $Date:

***********************************************************************************/
#include <stdio.h>
#include <string.h>
#include <signal.h>

#include "socasm.h"
#include "pcsgbd.h"

#ifndef VAXVMS
# include <malloc.h>
#endif

#ifdef MSDOS
# include "y_tab.h"
#else
#ifdef VAXVMS
#include "ytab.h"
#else
# include "y.tab.h"
#endif
#endif

#ifdef MEM_DEBUG
# include "mshell.h"
#endif

#include "genese.h"

/* ----------------------------------------------------------------------------
**
** 
**
** Modulo para gerar e manipular dados. (Linguagem de programacao)
**
** Forma de usar:
**
**		atualiza [-p] nomebase
**
** Flags envolvidas:
**
**		      -p : imprime quadro atualizado de variaveis
**		nomebase : nome da base de dados a ser atualizada
**
** Valor retornado:
**
**			Retorna a base atualizada
**
** Programado por:
**
**		Oscar Ivan Palma Pacheco
**		Carlos Fernando Assis Paniago
**
** Documentado por:
**
**		Carla Geovana do N. Macario
**
**
** Data da ultima alteracao:
**
**		97/02/28 15:45:54
**
** ----------------------------------------------------------------------------
*/

#ifdef DEBUG
int	f_1;
int	f_2;
int	f_4;
int	f_8;
int	f_16;
#endif

/*
** Define as palavras-chave do genese
*/
struct keywords keywords[] = {
	"if",		IF,
	"se",		IF,
	"else",		ELSE,
	"cc",		ELSE,
	"while",	WHILE,
	"enquanto",	WHILE,
	"keep",		KEEP,
	"manter",	KEEP,
	"drop",		DROP,
	"excluir",	DROP,
	"continue",	CONTINUE,
	"output",	OUTPUT,
	"grave",	OUTPUT,
	"sread",	SREAD,
	"leias",	SREAD,
	"fread",	EREAD,
	"leiaf",	EREAD,
	"where",	WHERE,
	"onde",		WHERE,
	"sopen",	SOPEN,
	"abres",	SOPEN,
	"fopen",	EOPEN,
	"abref",	EOPEN,
	"float",	FLOAT,
	"num",		FLOAT,
	"date",		DATE,
	"data",		DATE,
	"string",	STRING,
	"alfa",		STRING,
	"print",	PRINT,
	"liste",	PRINT,
	"file",		MYFILE,
	"arquivo",	MYFILE,
	"debug",	RDEBUG,
	"ftoa",		FTOA,
	"atof",		ATOF,
	"dtof",		DTOF,
	"dton",		DTOF,
	"ftod",		FTOD,
	"ntod",		FTOD,
	"dtoa",		DTOA,
	"atod",		ATOD,
	"monet",	MNT,
	"stop",		STOP,
	"pare",		STOP,
	"break",	BREAK,
	"interrompa",	BREAK,
	"abort",	ABORT,
	"aborte",	ABORT,
	"conchor",	CONCH,
	"concver",	CONCV,
	"chave",	CHAVE,
	"ord",		ORD,
	"asc",		ASC,
	"desc",		DESC,
	"reinicie",	REWIND,
	"rewind",	REWIND,
	"pause",	PAUSE,
	"pausa",	PAUSE,
	0,	 	0
};

/*
** Define os nomes das funcoes matematicas e de manipulacao de datas numericas
*/
struct funcwords funcwords[] = {
    "sin",	OSIN,	 1,	'n',	 0,	 0,	0,
    "cos",	OCOS,	 1,	'n',	 0,	 0,	0,
    "tan",	OTAN,	 1,	'n',	 0,	 0,	0,
    "asin",	OASIN,	 1,	'n',	 0,	 0,	0,
    "acos",	OACOS,	 1,	'n',	 0,	 0,	0,
    "atan",	OATAN,	 1,	'n',	 0,	 0,	0,
    "sqrt",	OSQRT,	 1,	'n',	 0,	 0,	0,
    "exp",	OEXP,	 1,	'n',	 0,	 0,	0,
    "abs",	OABS,	 1,	'n',	 0,	 0,	0,
    "log",	OLOG,	 1,	'n',	 0,	 0,	0,
    "log10",	OLOG10,	 1,	'n',	 0,	 0,	0,
    "rand",	ORAND,	 1,	'n',	 0,	 0,	0,
    "int",	OINT,	 1,	'n',	 0,	 0,	0,
    "ceil",	OCEIL,	 1,	'n',	 0,	 0,	0,
    "floor",	OFLOOR,	 1,	'n',	 0,	 0,	0,
    "hora",	OTIME,	 0,	 0,	 0,	 0,	0,
    "time",	OTIME,	 0,	 0,	 0,	 0,	0,
    "verstr",	OSCAN,	 2,	'a',	'a',	 0,	0,
    "nday",	ONDAY,	 1,	'd',	 0,	 0,	0,
    "ndia",	ONDAY,	 1,	'd',	 0,	 0,	0,
    "ddia",	ODIA,	 1,	'd',	 0,	 0,	0,
    "dday",	ODIA,	 1,	'd',	 0,	 0,	0,
    "dmes",	OMES,	 1,	'd',	 0,	 0,	0,
    "dmonth",	OMES,	 1,	'd',	 0,	 0,	0,
    "dano",	OANO,	 1,	'd',	 0,	 0,	0,
    "dyear",	OANO,	 1,	'd',	 0,	 0,	0,
    "max",	OMAX,	 2,	'n',	'n',	 0,	0,
    "min",	OMIN,	 2,	'n',	'n',	 0,	0,
    "tamstr",	OSTRLEN, 1,	'a',	 0,	 0,	0,
    "fprob",	OFPROB,	 3,	'n',	'n',	'n',	0,
    "finv",	OFINV,	 3,	'n',	'n',	'n',	0,
    "nprob",	ONPROB,	 1,	'n',	 0,	 0,	0,
    "ninv",	ONINV,	 1,	'n',	 0,	 0,	0,
    "tprob",	OTPROB,	 2,	'n',	'n',	 0,	0,
    "tinv",	OTINV,	 2,	'n',	'n',	 0,	0,
    "xprob",	OXPROB,	 2,	'n',	'n',	 0,	0,
    "xinv",	OXINV,	 2,	'n',	'n',	 0,	0,
    "existarq",	OEXTARQ, 1,	'a',	 0,	 0,	0,
    0,	 	0,	 0,	0,	 0,	 0,	0
};

/*
** Define os nomes das funcoes que nao retornam valor.
*/

struct funcwords funcvwords[] = {
    "srand",	OSRAND,		1,	'n',	0,	0,	0,
    0,	 	0,		0,	 0,	0,	0,	0
};

/*
** Define os nomes das funcoes de manipulacao de datas 
*/

struct funcwords funcdwords[] = {
    "data",	ODATE,	0,	0,	0,	0,	0,
    "date",	ODATE,	0,	0,	0,	0,	0,
    0,	 	0,	0,	0,	0,	0,	0
};

/*
** Define os nomes das constantes
*/
struct keywords cnstwords [] = {
	"_eof",		OEOF,
	"_fda",		OEOF,
	"_pi",		OPI,
	"_e",		OE,
	"_euler",	OEULER,
	0,	 	0
};

/*
** Define os nomes das funcoes que manipulam strings
*/
struct funcwords funcswords [] = {
    "adata",	OADATE,	0,	 0,	 0,	 0,	0,
    "adate",	OADATE,	0,	 0,	 0,	 0,	0,
    "ahora",	OATIME,	0,	 0,	 0,	 0,	0,
    "atime",	OATIME,	0,	 0,	 0,	 0,	0,
    "substr",	OSTRMID,3,	'a',	'n',	'n',	0,
    "repstr",	OASTRRP,2,	'a',	'n',	 0,	0,
    "selstr",	OAWRDK,	3,	'a',	'n',	'a',	0,
    "rbstr",	OATRIM,	1,	'a',	 0,	 0,	0,
    "trcstr",	OASSTR,	3,	'a',	'a',	'a',	0,
    "esq",	OAESQ,	2,	'a',	'n',	 0,	0,
    "dir",	OADIR,	2,	'a',	'n',	 0,	0,
    "cent",	OACENT,	2,	'a',	'n',	 0,	0,
    "just",	OAJUST,	2,	'a',	'n',	 0,	0,
    "mastr",	OAUPPER,1,	'a',	 0,	 0,	0,
    "mistr",	OALOWER,1,	'a',	 0,	 0,	0,
    0,	 	0,	0,	 0,	 0,	 0,	0
};

/*
** Define os nomes dos modos dos arquivos
*/
struct keywords modos[] = {
	"l",		ENTRADA,
	"e",		ENTRADA,
	"i",		ENTRADA,
	"r",		ENTRADA,
	"g",		SAIDA,
	"s",		SAIDA,
	"o",		SAIDA,
	"w",		SAIDA,
	0,		0
};

/*
** Define a estrutura onde serao armazenadas as informacoes sobre os
** arquivos existentes no programa genese
*/
struct arquivos arquivos[MAXGENARQ];

/*
** Define a estrutura onde serao armazenadas as informacoes sobre as
** variaveis existentes no programa genese
*/
struct variavel variavel[MAXGENVAR];

/*
** Define as variaveis externas do modulo genese
*/
unsigned int	ins[MAXGENINS];	/* vetor de instrucoes a serem executadas  */
int	keepdrop[MAXVAR];	/* vetor que contem os indices das vars. a
				   serem mantidas ou excluidas  */
char	*formatos[MAXFORM];	/* formatacao de cada aqruivo externo 
				   formatado */
int	*listas[MAXFORM];	/* lista de vars. de cada arquivo externo */
int	numvar[MAXFORM];	/* no. de vars. de cara arquivo externo  */
char	*frdform = (char *) NULL;	/* buffer com o formato do arquivo 
					   externo formatado  */

int	nvarc = 0; 	/* no. de vars. correntes  */	
int	nvar;		/* no. de vars. existentes no programa  */
int	nomeflag = 0;	/*  */	
int	pc=0;		/* contador do programa genese em execucao  */
int	output = -1;	/* flag - tem arquivos de saida  */
int	pervar = 0;	/* contem o tipo da variavel  */
int	iskeep = 0;	/* flag - var. deve ser mantida  */	
int	isdrop = 0;	/* flag - var. deve ser excluida  */	
int	nkd = 0; 	/* total de vars. a serem mantidas ou excluidas  */
int	narqs = 1;	/* numero de arquivos existentes no programa  */
char	anome[9];	/*   */	
char	atipo;		/* */
int	debflag = 0; 	/* flag - permmite a depuracao da execucao do prog. */
int	varveris;	/* indice da variavel na tabela de variaveis  */

char	*progname;	/* for error messages */
int	lineno = 1;
int	c;		/* global use by warning */
int	lprind = 0;	/* coluna desejada para a impressao formatada  */
char	printbuf[PRTBUF];	/* */	
int	maxlprind = 0;	/*    */	
int	prtvar = 0;	/* caracter especial de formatacao como \l ou \t */	
int	MensFlag = 0;	/* flag - permite a impressao de mensagens  */	
int	flagA = 0;	/*  */	
int	flagX = 1;	/* flag que indica o uso do modulo fora do NTIA. nao esta sendo usada*/
int	flagP = 0;	/*  */
int	warnings = 1;	/*  */
int	flagW = 0;	/*  */
int	flagV = 0;	/*  flag para testar a existencia do arquivo */
int	primP = 1;	/*  */
int	posnum;		/* no. de coluna a partir da qual deve ser iniciada
			   a leitura formatada  */
int	formnum;	/* indice de variavel com formato numerico 
			   para sua leitura */
int	campo = 0;	/* tamanho do campo  */
int	decim = 0;	/* no. de casas decimais  */
int	esque = 0;	/* flag - campo e' alinhado a esquerda na impressao
			   formatada   */
int	setmodo = 0;	/* flag - indica o modo do arquivo */
FILE	*prtffile;	/* apontador para arquivo externo  */
int	nform = 0;	/* no. de arquivos externos  */
int	indvar;		/* indice de variaveis de um arquivo  */
int	pareeof = 1;	/* constante de final de arquivo  */
int	flag_where = 0;	/* indica o tipo de leitura  */

#ifdef MSDOS
int	nlins = 25;
#else
int	nlins = 24;
#endif

int	ncols = 80;



#ifdef VAXVMS
main(argc, argv)
#else
void main(argc, argv)
#endif
int argc;
char *argv[];
{
	int i;
	int iarq;
	long ret;
	char *parea;
	char	buffer[128+128];

/*
#if (defined(IRIX) | defined(SUN)) & !defined(RS6000)
	if (!VerifysInitServer("genese"))
		exit(1);
#endif
*/

#ifdef VAXVMS
	argc = prepvax(argc, argv);
#endif

	progname = "genese";

	signal(SIGINT, termina_ctrl_c);
	signal(SIGSEGV, termina_segfault);

#ifndef MSDOS
	signal(SIGBUS, termina_buserror);
#endif


	/* le a lista de argumentos do modulo */

	iarq = gtarggen(argc, argv);

	parea=(char *) malloc((unsigned) (strlen(argv[iarq])+1));
	if (parea == (char *) NULL)

/* *** Padroniz. msg erro - Adriana 09/01/95
		execerror("Falta memoria","");
*** */

		execerror( PegaMsg (1039),"");

	arquivos[0].nome = parea;
	strcpy(parea, argv[iarq]);

	/* Cria arquivo de saida - da lista de argumentos */

	if ((ret = (long) (arquivos[0].arq.ptraa = cria_arq(argv[iarq])))
				< 0L && ret > -4L) {

		fprintf(stderr, PegaMsg (1034), argv[iarq]);
		termina(1);
	}

	arquivos[0].arqfile = 0;
	arquivos[0].obs = 0L;
	arquivos[0].modo = SAIDA;

	arquivos[0].fim = 0;
	arquivos[0].nch = 0;
	arquivos[0].ch = (int *) NULL;
	arquivos[0].qarea.pch = (struct Ch_Area *) NULL;

	for (i = 0; i < PRTBUF; ++i)
		printbuf[i] = ' ';

#ifdef DEBUG
	debug(0) ;
#endif

	if (MensFlag)
		fprintf(stderr, "Compilando ....\n\n");

	ins[pc++] = FSPC | VFA;	/* inicializacao */

	if (!flagA)
		fprintf(stderr, "\r%d > ", lineno);

	yyparse(); /* Chama o parse  */

	if (MensFlag)
		fprintf(stderr, "\r");

	if (MensFlag)
		fprintf(stderr, "Executando ...\n\n");

#ifdef DEBUG
	if (f_8) {
		printf("Executando:\nPC\tOCTAL \tHEXA\tDECIMAL\n");
		for (i = 0; i < pc; i++)
#ifdef MSDOS
			printf("%2d\t%06o\t%04x\t%05u\n", i, ins[i], ins[i],
				ins[i]);
#else
#ifdef EDIX
			printf("%2d\t%06o\t%04x\t%05u\n", i, ins[i], ins[i],
				ins[i]);
#else
			printf("%2d\t%011o\t%08x\t%010u\n", i, ins[i], ins[i],
				ins[i]);
#endif
#endif
	}
	if (f_4) {
		printabela();
	}
#endif

	pc = 1;
	prtffile = stdout;

	execins(); /* Passa para a execucao das instrucoes  */

#ifdef DEBUG
	if (f_4) {
		printabela();
	}
#endif

	fflush(stdout);

	if (MensFlag)
		fprintf(stderr, "\n%s: ===>>   F I M   <<===\n", progname);

	termina(0);
}

