/***********************************************************************************
 LIBSOC - Biblioteca padro para o SOftware Cientfico

 Direitos Autorais Reservados (c) 2004 Embrapa Informtica Agropecuria

 Esta biblioteca  software livre; voc pode redistribu-la e/ou modific-la sob 
 os termos da Licena Pblica Geral Menor do GNU conforme publicada pela Free 
 Software Foundation; tanto a verso 2.1 da Licena, ou (a seu critrio) 
 qualquer verso posterior.

 Esta biblioteca  distribuda na expectativa de que seja til, porm, SEM NENHUMA 
 GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA 
 FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral Menor do GNU para mais 
 detalhes.

 Voc deve ter recebido uma cpia da Licena Pblica Geral Menor do GNU junto com 
 esta biblioteca; se no, escreva para a Free Software Foundation, Inc., no 
 endereo 59 Temple Street, Suite 330, Boston, MA 02111-1307 USA. 

 ***********************************************************************************

 Parte integrante do SOC - SOftware Cientfico

 Descrio:

    Calcular Log natural da Funcao Gama (ALGama - Logarithm of Gamma Distribution)

 Desenvolvido por:

		Dalton Francisco Andrade
		Maria Fernanda Moura
       	Oscar Ivan Palma Pacheco
		Slvio Roberto Medeiros Evangelista
		Snia Ternes

 Informaes do CVS:
        $Source:
        $Revision:
        $Date:

***********************************************************************************/


/*
** --------------------------------------------------------------------------
**
** Funcao:  ALGama
**
** Objetivo: Calcular Log natural da Funcao Gama
**
** Modo de chamada :
**
**      Val = ALGama( Arg )
**
**           Arg  : argumento da funcao Gama
**           Val  : valor calculado do log. natural da fc. Gama
**
** Funcoes usadas : nenhuma
**
** Observacoes:
**
**      Minimax polynomial approximations are used over the
**      intervals [-inf,0], [0,.5], [.5,1.5], [1.5,4.0],
**      [4.0,12.0], [12.0,+inf].
**
**      See Hart et al, "Computer Approximations",
**      Wiley(1968), p. 130F, and also
**
**      Cody and Hillstrom, "Chebyshev approximations for
**      the natural logarithm of the Gamma function",
**      Mathematics of Computation, 21, April, 1967, P. 198F.
**
**
**      There are some machine-dependent constants used --
**
**         Rmax   --- Largest value for which ALGama
**                    can be safely computed.
**         Rinf   --- Largest floating-point number.
**         Zeta   --- Smallest floating-point number
**                    such that (1 + Zeta) = 1.
**
** --------------------------------------------------------------
*/

#include <stdio.h>
#include <math.h>

#include "const.h"

/* Structured */

static	double P[]= {	 0.0,	/* o primeiro nao existia , e' de 1 a 29 */
			 4.12084318584770E+00 ,
			 8.56898206283132E+01 ,
			 2.43175243524421E+02 ,
			-2.61721858385614E+02 ,
			-9.22261372880152E+02 ,
			-5.17638349802321E+02 ,
			-7.74106407133295E+01 ,
			-2.20884399721618E+00 ,

			 5.15505761764082E+00 ,
			 3.77510679797217E+02 ,
			 5.26898325591498E+03 ,
			 1.95536055406304E+04 ,
			 1.20431738098716E+04 ,
			-2.06482942053253E+04 ,
			-1.50863022876672E+04 ,
			-1.51383183411507E+03 ,

			-1.03770165173298E+04 ,
			-9.82710228142049E+05 ,
			-1.97183011586092E+07 ,
			-8.73167543823839E+07 ,
			 1.11938535429986E+08 ,
			 4.81807710277363E+08 ,
			-2.44832176903288E+08 ,
			-2.40798698017337E+08 ,

			 8.06588089900001E-04 ,
			-5.94997310888900E-04 ,
			 7.93650067542790E-04 ,
			-2.77777777688189E-03 ,
			 8.33333333333330E-02	};

static double Q[] = {	0.0 , 		/* nao existia, de 1 a 24 */
			1.00000000000000E+00 ,
			4.56467718758591E+01 ,
			3.77837248482394E+02 ,
			9.51323597679706E+02 ,
			8.46075536202078E+02 ,
			2.62308347026946E+02 ,
			2.44351966250631E+01 ,
			4.09779292109262E-01 ,

			1.00000000000000E+00 ,
			1.28909318901296E+02 ,
			3.03990304143943E+03 ,
			2.20295621441566E+04 ,
			5.71202553960250E+04 ,
			5.26228638384119E+04 ,
			1.44020903717009E+04 ,
			6.98327414057351E+02 ,

			1.00000000000000E+00 ,
			-2.01527519550048E+03 ,
			-3.11406284734067E+05 ,
			-1.04857758304994E+07 ,
			-1.11925411626332E+08 ,
			-4.04435928291436E+08 ,
			-4.35370714804374E+08 ,
			-7.90261111418763E+07	};

double ALGama( Arg )
double Arg;
{
	double	Rarg;
	double	Alinc;
	double	Scale;
	double	Top;
	double	Bot;
	double	Frac;
	double	Algval;

	int	I;
	int	Iof;
	int	Ilo;
	int	Ihi;

	int	Qminus;		/* BOOLEAN */
	int	Qdoit;		/* BOOLEAN */

	/*
	** ALGama
	**
	** Initialize
	*/

	Algval = Rinf;
	Scale  = 1.0;
	Alinc  = 0.0;
	Frac   = 0.0;
	Rarg   = Arg;
	Iof    = 1;
	Qminus = 0;
	Qdoit  = 1;

	/*
	** Adjust for negative argument
	*/

	if( Rarg < 0.0 ) {
		Qminus = 1;
		Rarg   = -Rarg;
		Top    = (double) (( int ) Rarg);
		Bot    = 1.0;

		if( ( (( int ) ( Top / 2.0 )) * 2.0 ) == 0.0 )
			Bot = -1.0;

		Top    = Rarg - Top;

		if( Top == 0.0 )
			Qdoit = 0;
		else {
			Frac   = Bot * PI / sin( Top * PI );
			Rarg   = Rarg + 1.0;
			Frac   = log ( fabs( Frac ) );
		}

	}

	/*
	** Choose approximation interval
	** based upon argument range
	*/

	if( Rarg == 0.0 )
		Qdoit = 0;

	else {
		if( Rarg <= 0.5 ) {
			Alinc  = -log( Rarg );
			Scale  = Rarg;
			Rarg   = Rarg + 1.0;

			if( Scale < Zeta ) {
				Algval = Alinc;
				Qdoit  = 0;
			}
		}
		else {
			if ( Rarg <= 1.5 )
				Scale = Rarg - 1.0;
			else {
				if( Rarg <= 4.0 ) {
					Scale  = Rarg - 2.0;
					Iof    = 9;
				}
				else {
					if( Rarg <= 12.0 )
						Iof = 17;
					else {
						if( Rarg <= Rmax) {
							Alinc  = ( Rarg - 0.5 )
							* log( Rarg ) - Rarg
							+ Xln2sp;
							Scale  = 1.0 / Rarg;
							Rarg   = Scale * Scale;
							Top    = P[ 25 ];
							for (I=26;I<=29;I++) {
								Top    = Top *
								Rarg + P[ I ];
							}
							Algval = Scale * Top
								+ Alinc;
							Qdoit  = 0;
						}
					}
				}
			}
		}
	}

	/*
	** Common evaluation code for Arg <= 12.
	** Horner's method is used, which seems
	** to give better accuracy than
	** continued fractions.
	*/

	if ( Qdoit ) {

		Ilo    = Iof + 1;
		Ihi    = Iof + 7;

		Top    = P[ Iof ];
		Bot    = Q[ Iof ];

		for ( I = Ilo ; I <= Ihi ; I++ ) {
			Top    = Top * Rarg + P[ I ];
			Bot    = Bot * Rarg + Q[ I ];
		}

		Algval = Scale * ( Top / Bot ) + Alinc;

	}

	if ( Qminus )
		Algval = Frac - Algval;

	return(Algval);

}	/* ALGama */

