/***********************************************************************************
 LIBSOC - Biblioteca padro para o SOftware Cientfico

 Direitos Autorais Reservados (c) 2004 Embrapa Informtica Agropecuria

 Esta biblioteca  software livre; voc pode redistribu-la e/ou modific-la sob 
 os termos da Licena Pblica Geral Menor do GNU conforme publicada pela Free 
 Software Foundation; tanto a verso 2.1 da Licena, ou (a seu critrio) 
 qualquer verso posterior.

 Esta biblioteca  distribuda na expectativa de que seja til, porm, SEM NENHUMA 
 GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA 
 FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral Menor do GNU para mais 
 detalhes.

 Voc deve ter recebido uma cpia da Licena Pblica Geral Menor do GNU junto com 
 esta biblioteca; se no, escreva para a Free Software Foundation, Inc., no 
 endereo 59 Temple Street, Suite 330, Boston, MA 02111-1307 USA. 

 ***********************************************************************************

 Parte integrante do SOC - SOftware Cientfico

 Descrio:

    Calcula a inversa da distribuio Beta. 

 Desenvolvido por:

		Dalton Francisco Andrade
		Maria Fernanda Moura
       	Oscar Ivan Palma Pacheco
		Slvio Roberto Medeiros Evangelista
		Snia Ternes

 Informaes do CVS:
        $Source:
        $Revision:
        $Date:

***********************************************************************************/
/*--------------------------------------------------------------*/
/*                                                              */
/* Function:  BetaInv                                           */
/*                                                              */
/* Purpose:  Calculates inverse Beta distribution               */
/*                                                              */
/* Calling Sequence:                                            */
/*                                                              */
/*      B = BetaInv(      P, Alpha, Beta : REAL                 */
/*                         MaxIter        : INTEGER;            */
/*                         Dprec          : INTEGER;            */
/*                    VAR  Iter           : INTEGER;            */
/*                    VAR  Cprec          : REAL;               */
/*                    VAR  Ierr           : INTEGER ) : REAL;   */
/*                                                              */
/*     P       --- Cumulative probability for which percentage  */
/*                 point is to be found.                        */
/*     Alpha   --- First parameter of Beta distribution         */
/*     Beta    --- Second parameter of Beta distribution        */
/*     MaxIter --- Maximum iterations allowed                   */
/*     Dprec   --- no. of digits of precision desired           */
/*     Iter    --- no. of iterations actually performed         */
/*     Cprec   --- no. of digits of precision actually obtained */
/*     Ierr    --- error indicator                              */
/*                 = 0: OK                                      */
/*                 = 1: argument error                          */
/*                 = 2: evaluation error during iteration       */
/*                                                              */
/*     B      --- Resulting percentage point of Beta dist.      */
/*                                                              */
/* Calls:  CDBeta  (cumulative Beta distribution)               */
/*                                                              */
/* Remarks:                                                     */
/*                                                              */
/*      Because of the relationship between the Beta            */
/*      distribution and the F distribution, this               */
/*      routine can be used to find the inverse of              */
/*      the F distribution.                                     */
/*                                                              */
/*      The percentage point is returned as -1.0                */
/*      if any error occurs.                                    */
/*                                                              */
/*      Newtons' method is used to search for the percentage point. */
/*                                                              */
/*      The algorithm is based upon AS110 from Applied Statistics.  */
/*                                                              */
/*--------------------------------------------------------------*/

#include <stdio.h>
#include <math.h>

#include "pcsgbd.h"
#include "const.h"

double BetaInv( P , Alpha , Beta , MaxIter , Dprec , Iter , Cprec , Ierr)     
double P; 
double Alpha; 
double Beta;
int    MaxIter;
int    Dprec;
int    *Iter;
double *Cprec;
int    *Ierr;
{

	double 	  AUX;
	double    val_beta, Eps, Xim1, Xi, Xip1, Fim1;
	double    Fi, W, Cmplbt, Adj, Sq, R, S, T, G;
	double    A, B, PP, H, A1, B1, Eprec;

	int       Done;		/*   BOOLEAN  */

	int       Jter;

/* BetaInv */

   *Ierr    = 1;
   val_beta = P;
                                   /* Check validity of arguments */

   if( ( Alpha <= 0.0 ) || ( Beta <= 0.0 ) )   goto NOVEMIL;
   if( ( P > 1.0 ) || ( P < 0.0 ) )            goto NOVEMIL;
                                   /* Check for P = 0 or 1        */
   if( ( P == 0.0 ) || ( P == 1.0 ) ) {
         *Iter   = 0;
         *Cprec  = (double) MaxPrec;
         goto NOVEMIL;
   }
                                  /* Set precision */
   if ( Dprec > MaxPrec ) 	Dprec = MaxPrec;
   else 
	if  ( Dprec <= 0 ) 	Dprec = 1;

   *Cprec  = (double) Dprec;

   Eps    = pow(10.0, -2.0 * Dprec );
                                   /* Flip params if needed so that */
                                   /* P for evaluation is <= .5     */
   if( P > 0.5 )  {
         A      = Beta;
         B      = Alpha;
         PP     = 1.0 - P;
   }
   else {
         A      = Alpha;
         B      = Beta;
         PP     = P;
   }

   /* Generate initial approximation.  */
   /* Several different ones used,     */
   /* depending upon parameter values. */

   *Ierr   = 0;

   Cmplbt = ALGama( A ) + ALGama( B ) - ALGama( A + B );
    /* Fi     = ninv( 1.0 - PP ); antes ninv devolvia o valor x para o
	qual P[ X <= x ] era o seu argumento */
   Fi     = ninv( PP );

   if( ( A > 1.0 ) && ( B > 1.0 ) )  {
         R      = ( Fi * Fi - 3.0 ) / 6.0;
         S      = 1.0 / ( A + A - 1.0 );
         T      = 1.0 / ( B + B - 1.0 );
         H      = 2.0 / ( S + T );
         W      = Fi * sqrt( H + R ) / H - ( T - S ) *
                   ( R + 5.0 / 6.0 - 2.0 / ( 3.0 * H ) );
	 AUX    = W + W;
	 if ( AUX < MinExp )	AUX = 0.0;
	 else			AUX = exp(AUX);
         Xi     = A / ( A + B * AUX );
   }
   else {
         R      = B + B;
         T      = 1.0 / ( 9.0 * B );
         T      = R * pow( ( 1.0 - T + Fi * sqrt( T ) ) , 3.0 );

         if( T <= 0.0 )  {
	    AUX    = ( log( ( 1.0 - PP ) * B ) + Cmplbt ) / B ;
	    if ( AUX < MinExp )		AUX = 0.0;
	    else			AUX = exp(AUX);
            Xi     = 1.0 -  AUX;
	 }
         else {
               T      = ( 4.0 * A + R - 2.0 ) / T;
               if( T <= 1.0 )  {
		  AUX = (log( PP * A ) + Cmplbt) / PP ;
	    	  if ( AUX < MinExp )		AUX = 0.0;
	    	  else				AUX = exp(AUX);
                  Xi = AUX;
	       }
               else
                  Xi = 1.0 - 2.0 / ( T + 1.0 );
         }
   }
                                   /* Force initial estimate to */
                                   /* reasonable range.         */
   if ( Xi < 0.0001 )   Xi = 0.0001;
   if ( Xi > 0.9999 )   Xi = 0.9999;
                                   /* Set up Newton-Raphson loop */
   A1     = 1.0 - A;
   B1     = 1.0 - B;
   Fim1   = 0.0;
   Sq     = 1.0;
   Xim1   = 1.0;
   *Iter   = 0;
   Done   = 0;
                                   /* Begin Newton-Raphson loop  */
   do {
      *Iter   = *Iter + 1;
      Done   = (int) (Done || ( *Iter > MaxIter ));
      Fi     = CDBeta( Xi, A, B, Dprec+1, MaxIter,&Eprec,&Jter, Ierr );
      if( *Ierr != 0 )  {
            *Ierr = 2;
            Done = 1;
      }
      else {
	    AUX    = Cmplbt + A1 * log( Xi ) + B1 * log( 1.0 - Xi );
	    if ( AUX < MinExp )		AUX = 0.0;
	    else			AUX = exp(AUX);
            Fi     = ( Fi - PP ) * AUX;
            if( ( Fi * Fim1 ) <= 0.0 )   Xim1 = Sq;
            G      = 1.0;

  DEZ:     do {
               Adj    = G * Fi;
               Sq     = Adj * Adj;
               if( Sq >= Xim1 )   G = G / 3.0;
            } while ( Sq >= Xim1 );

            Xip1     = Xi - Adj;

            if( ( Xip1 < 0.0 ) || ( Xip1 > 1.0 ) )  {
                  G      = G / 3.0;
                  goto DEZ;
            }
            if( Xim1    <= Eps  )   goto TRINTA;
            if( Fi * Fi <= Eps  )   goto TRINTA;

            if( ( Xip1 == 0.0 ) || ( Xip1 == 1.0 ) )  {
                  G     = G / 3.0;
                  goto DEZ;
            }
            if( Xip1 != Xi )  {
                  Xi     = Xip1;
                  Fim1   = Fi;
            }
            else
               Done = 1;
	}
   } while ( !Done );

TRINTA: val_beta = Xi;
   if( P > 0.5 )   val_beta = 1.0 - Xi;

   if ( fabs( Xi - Xim1 ) != 0.0 )  
      *Cprec  = -log10( fabs( Xi - Xim1 ) );
   else
      *Cprec  = (double) MaxPrec;

NOVEMIL: if ( *Ierr != 0 )   val_beta = -1.0;
   return(val_beta);

}   /* BetaInv */
