/***********************************************************************************
 LIBSOC - Biblioteca padro para o SOftware Cientfico

 Direitos Autorais Reservados (c) 2004 Embrapa Informtica Agropecuria

 Esta biblioteca  software livre; voc pode redistribu-la e/ou modific-la sob 
 os termos da Licena Pblica Geral Menor do GNU conforme publicada pela Free 
 Software Foundation; tanto a verso 2.1 da Licena, ou (a seu critrio) 
 qualquer verso posterior.

 Esta biblioteca  distribuda na expectativa de que seja til, porm, SEM NENHUMA 
 GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA 
 FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral Menor do GNU para mais 
 detalhes.

 Voc deve ter recebido uma cpia da Licena Pblica Geral Menor do GNU junto com 
 esta biblioteca; se no, escreva para a Free Software Foundation, Inc., no 
 endereo 59 Temple Street, Suite 330, Boston, MA 02111-1307 USA. 

 ***********************************************************************************

 Parte integrante do SOC - SOftware Cientfico

 Descrio:

    GammaIn -- Calcula a integral em uma gamma incompleta. 

Desenvolvido por: 
		Dalton Francisco Andrade
		Maria Fernanda Moura 		
		Oscar Ivan Palma Pacheco
		Slvio Roberto Medeiros Evangelista 		
		Snia Ternes

 Informaes do CVS:
        $Source:
        $Revision:
        $Date:

***********************************************************************************/


/*--------------------------------------------------------------*/
/*     GammaIn -- Incomplete Gamma integral                     */
/*--------------------------------------------------------------*/

#include <stdio.h>
#include <math.h>

#include "pcsgbd.h" 
#include "const.h" 

/*-------------------------------------------------------------------*/
/*                                                                   */
/* Function:  GammaIn                                                */
/*                                                                   */
/* Purpose:   Evaluates Incomplete Gamma integral                    */
/*                                                                   */
/* Calling Sequence:                                                 */
/*                                                                   */
/*      P     = GammaIn(        Y, P    : REAL;                      */
/*                              Dprec   : INTEGER;                   */
/*                              MaxIter : INTEGER;                   */
/*                         VAR  Cprec   : REAL;                      */
/*                         VAR  Iter    : INTEGER;                   */
/*                         VAR  Ifault  : INTEGER ) : REAL;          */
/*                                                                   */
/*           Y      --- Gamma distrib. value                         */
/*           P      --- Degrees of freedom                           */
/*           Ifault --- error indicator                              */
/*                                                                   */
/*           P      --- Resultant probability                        */
/*                                                                   */
/* Calls:                                                            */
/*                                                                   */
/*      ALGama                                                       */
/*                                                                   */
/* Remarks:                                                          */
/*                                                                   */
/*      Either an infinite series summation or a continued fraction  */
/*      approximation is used, depending upon the argument range.    */
/*                                                                   */
/*-------------------------------------------------------------------*/

double GammaIn( Y, P , Dprec, MaxIter, Cprec, Iter, Ifault )     
double  Y;
double  P; 
int     Dprec;
int     MaxIter;
double  *Cprec;
int     *Iter;
int     *Ifault;
{


double    valret;
double    F;
double    C;
double    A;
double    B;
double    Term;
double    Pn[7];		/* comeca em 1 e vai ate 6 */
double    Gin;
double    An;
double    Rn;
double    Dif;
double    Eps;
int       Done;

/* GammaIn */
                                   /* Check arguments */
	*Ifault  = 1;
	valret   = 1.0;

   	if( ( Y <= 0.0 ) || ( P <= 0.0 ) ) goto nove;

                                   /* Check value of F */
   	*Ifault = 0;


   	F       = P * log( Y ) - ALGama( P + 1.0 ) - Y;


   	if ( F < MinExp ) {
		*Ifault  = 13;
		goto nove;
	}

	F = exp( F );

   	if( F == 0.0 ) {
		*Ifault  = 13;
		 goto nove;
	}

                                  /* Set precision */
   	if ( Dprec > MaxPrec ) 
      				Dprec = MaxPrec;
   	else {
		if (Dprec <= 0) 
      				Dprec = 1;
	}
   	*Cprec  = (double) Dprec;

   	Eps     = pow( 10.0 , (double) -Dprec );

        /* Choose infinite series or */
        /* continued fraction.       */

   	if( ( Y > 1.0 ) && ( Y >= P ) ) { 
		/* Continued Fraction */
         	A       = 1.0 - P;
         	B       = A + Y + 1.0;
         	Term    = 0.0;
         	Pn[ 1 ] = 1.0;
         	Pn[ 2 ] = Y;
         	Pn[ 3 ] = Y + 1.0;
         	Pn[ 4 ] = Y * B;
         	Gin     = Pn[ 3 ] / Pn[ 4 ];
         	Done    = 0;
         	*Iter    = 0;

         	do {

            		*Iter  = *Iter + 1;
            		A      = A + 1.0;
            		B      = B + 2.0;
            		Term   = Term + 1.0;
            		An     = A * Term;

            		Pn[ 5 ] = B * Pn[ 3 ] - An * Pn[ 1 ];
            		Pn[ 6 ] = B * Pn[ 4 ] - An * Pn[ 2 ];

            		if( Pn[ 6 ] != 0.0 ) {

                  		Rn     = Pn[ 5 ] / Pn[ 6 ];
                  		Dif    = fabs( Gin - Rn );

                  		if( Dif <= Eps ) 
                     			if( Dif <= ( Eps * Rn ) ) 
						Done = 1;

                  		Gin    = Rn;

               		}

            		Pn[ 1 ] = Pn[ 3 ];
            		Pn[ 2 ] = Pn[ 4 ];
            		Pn[ 3 ] = Pn[ 5 ];
            		Pn[ 4 ] = Pn[ 6 ];

            		if( fabs( Pn[ 5 ] ) >= Oflo ) {
                  		Pn[ 1 ] = Pn[ 1 ] / Oflo;
                  		Pn[ 2 ] = Pn[ 2 ] / Oflo;
                  		Pn[ 3 ] = Pn[ 3 ] / Oflo;
                  		Pn[ 4 ] = Pn[ 4 ] / Oflo;
               		}

         } while (( *Iter <= MaxIter ) && (!Done));

         Gin    = 1.0 - ( F * Gin * P );

         valret = Gin;
                                   /* Calculate precision of result */

         if (Dif != 0.0 ) 	*Cprec = -log10( Dif );
         else 			*Cprec =  (double) MaxPrec;

      }   /* Continued Fraction */

      else { 	/* Infinite series */

         *Iter    = 0;
         Term     = 1.0;
         C        = 1.0;
         A        = P;
         Done     = 0;

         do {

            A       = A + 1.0;
            Term    = Term * Y / A;
            C       = C + Term;

            *Iter    =  (*Iter) + 1;

         } while (( *Iter <= MaxIter ) && ( ( Term / C ) > Eps ));

         valret = C * F;
                                   /* Calculate precision of result */

         *Cprec = -log10( Term / C );

      }   /* Infinite series */

nove: return(valret);

}
