/***********************************************************************************
LIBSOC - Biblioteca padro para o SOftware Cientfico

Direitos Autorais Reservados (c) 2004 Embrapa Informtica Agropecuria

Esta biblioteca  software livre; voc pode redistribu-la e/ou modific-la sob
os termos da Licena Pblica Geral Menor do GNU conforme publicada pela Free
Software Foundation; tanto a verso 2.1 da Licena, ou (a seu critrio)
qualquer verso posterior.

Esta biblioteca  distribuda na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral Menor do GNU para mais
detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral Menor do GNU junto com
esta biblioteca; se no, escreva para a Free Software Foundation, Inc., no
endereo 59 Temple Street, Suite 330, Boston, MA 02111-1307 USA.

 ***********************************************************************************

 Parte integrante do SOC - SOftware Cientfico

 Descrio:

    xinv --- calcula inversa da qui-quadrado. 
    
 Desenvolvido por:

		Dalton Francisco Andrade
		Maria Fernanda Moura
		Oscar Ivan Palma Pacheco
		Slvio Roberto Medeiros Evangelista
            Snia Ternes

 Informaes do CVS:
        $Source:
        $Revision:
        $Date:

***********************************************************************************/
#include <stdio.h>
#include <math.h>
#include "const.h"
#include "pcsgbd.h"


/*--------------------------------------------------------------------*/
/*            xinv --- Inverse chi-square routine                     */
/*--------------------------------------------------------------------*/
/*                                                                    */
/* Function:  xinv                                                    */
/*                                                                    */
/* Purpose:  Compute inverse chi-square (percentage point)            */
/*                                                                    */
/* Calling Sequence:                                                  */
/*                                                                    */
/*    Cp := xinv( P, V: REAL; VAR Ifault: INTEGER ) : REAL;           */
/*                                                                    */
/*       P      --- tail probability to get percentage point for      */
/*       V      --- degrees of freedom                                */
/*       Ifault --- error indicator                                   */
/*                  = 0:  no error                                    */
/*                  = 1:  probability too small to accurately compute */
/*                        percentage point.                           */
/*                  = 2:  degrees of freedom given as <= 0.           */
/*                  = 3:  chi-square probability evaluation failed.   */
/*                                                                    */
/*       Cp    --- resulting chi-square percentage point              */
/*                                                                    */
/* Calls:    GammaIn                                                  */
/*           ninv                                                     */
/*           ALGama                                                   */
/*                                                                    */
/* Remarks:                                                           */
/*                                                                    */
/*    This is basically AS 91 from Applied Statistics.                */
/*    An initial approximation is improved using a Taylor series      */
/*    expansion.                                                      */
/*                                                                    */
/* 								      */
/*--------------------------------------------------------------------*/

#define  PRECISAO   1.0e-05

double xinv( P, V ) 
double P;
double V;
{
	int Dprec   = 8;
	int MaxIter = 100;

	double XX,C,Ch,Q,P1,P2,T; 
	double X,B,A,G; 
	double S1,S2,S3,S4,S5,S6;
	double Cprec;
	double val_ci;

	int Iter;
	int Ifault;

	/* Inicio xinv */
        /* Test arguments for validity */

	if ( P >= (1.0 - PRECISAO) )	
		P = 1.0 - PRECISAO;
	else {
		if ( P <= PRECISAO ) 
			P = PRECISAO;
	}

	if ( V >= 20000. )
		V = 20000.;
	else {
		if ( V <= 1.e-8 )
			V = 1.e-8;
	}

	Ifault = 2;

	/* Initialize */

	P      = 1.0 - P;
	XX     = V / 2.0;
	G      = ALGama( XX );
	Ifault = 0;
	C      = XX - 1.0;

	/* Starting approx. for small chi-square */

	if( V < ( -1.24 * log( P ) ) ) {
		Ch = pow( P * XX * exp( G + XX * LnTwo ) , ( 1.0 / XX ) );

		if (Ch < PRECISAO ) {
			val_ci = Ch;
			goto nove;
		}
	}
	else {                 /* Starting approx. for v <= .32 */
		if ( V <= 0.32 ) {
			Ch = 0.4;
			A  = log( 1.0 - P );

			do {
				Q  = Ch;
				P1 = 1.0 + Ch * ( 4.67 + Ch );
				P2 = Ch * ( 6.73 + Ch * ( 6.66 + Ch ) );
				T  = -0.5 + ( 4.67 + 2.0 * Ch ) / P1 -
					 (6.73 + Ch * (13.32+3.0 * Ch) ) / P2;
				Ch = Ch-(1.0-exp(A+G+0.5 * Ch + C * LnTwo ) *
						P2 / P1 ) / T;
			} while ( fabs( Q / Ch - 1.0 ) > 0.01 );
		}
		else {
			/* Starting approx. using Wilson and */
			/* Hilferty estimate                 */
			/* X  = ninv( P ); antes ninv devolvia x, onde
				P[X<=x] era seu argumento */
			X  = ninv( 1.0 - P );
			P1 = 2.0 / ( 9.0 * V );
			Ch = V*(pow((double)(X * sqrt( P1 ) + 1.0 - P1),3.0));

			/* Starting approx. for P ---> 1     */

			if ( Ch > ( 2.2 * V + 6.0 ) ) {
				Ch = -2.0 * (log(1.0-P)-C*log( 0.5 * Ch ) + G );
				
			}
		}

		/* We have starting approximation.    */
		/* Begin improvement loop.            */

		do {
			/* Get probability of current approx. */
			/* to percentage point.               */
			Q     = Ch;
			P1    = 0.5 * Ch;
			P2    = P - GammaIn( P1, XX, Dprec, MaxIter, 
					&Cprec, &Iter, &Ifault );

			if ( Ifault == 13 ) {
				if ( Ch < V )	P2 = P - 0.0;
				else		P2 = P - 1.0;
			}

			if ( ( Ifault != 0 ) || ( Iter > MaxIter ) ) 
				Ifault = 3;
			else {
                                /* Calculate seven-term Taylor series */

				T  = P2 * exp(XX*LnTwo+G + P1 - C * log(Ch));
				B  = T / Ch;
				A  = 0.5 * T - B * C;

				S1 = ( 210.0 + A * ( 140.0 + A * ( 105.0 + A * 
					(84.0 + A * (70.0 + 60.0 * A)))))/420.0;
				S2 = ( 420.0 + A * ( 735.0 + A * ( 966.0 + A * 
					( 1141.0 + 1278.0 * A ) ) ) ) / 2520.0;
				S3 = ( 210.0 + A * ( 462.0 + A * ( 707.0 + 
					932.0 * A ) ) ) / 2520.0;
				S4 = ( 252.0 + A * ( 672.0 + 1182.0 * A ) + C *
				     (294.0+A*(889.0 + 1740.0 * A))) / 5040.0;
				S5 = ( 84.0 + 264.0 * A + C * ( 175.0 + 606.0 *
					 A ) ) / 2520.0;
				S6 = (120.0 + C * (346.0 + 127.0 * C)) / 5040.0;
				Ch = Ch + T * ( 1.0 + 0.5 * T * S1 - B * C * 
					( S1 - B  * ( S2 - B * ( S3 - B * 
					( S4 - B * ( S5 - B * S6 ) ) ) ) ) );
			}
		} while (( fabs( ( Q / Ch ) - 1.0 ) > PRECISAO ) && ( Ifault == 0 ));

	}
nove:	if (Ifault == 0) 	val_ci =  Ch;
	else 			val_ci = -1.0;

	return(val_ci);

}   /* xinv */

