/***********************************************************************************
 MULTVAR - Módulo para análise de modelos lineares multivariados

 Direitos Autorais Reservados (c) 2004 Embrapa Informática Agropecuária

 Este programa é software livre; você pode redistribuí-lo e/ou modificá-lo sob os
 termos da Licença Pública Geral GNU conforme publicada pela Free Software
 Foundation; tanto a versão 2 da Licença, como (a seu critério) qualquer versão
 posterior.

 Este programa é distribuído na expectativa de que seja útil, porém, SEM NENHUMA
 GARANTIA; nem mesmo a garantia implícita de COMERCIABILIDADE OU ADEQUAÇÃO A UMA
 FINALIDADE ESPECÍFICA. Consulte a Licença Pública Geral do GNU para mais detalhes.

 Você deve ter recebido uma cópia da Licença Pública Geral do GNU junto com este
 programa; se não, escreva para a Free Software Foundation, Inc., no endereço 59
 Temple Street, Suite 330, Boston, MA 02111-1307 USA.

 **********************************************************************************

 Parte integrante do SOC - SOftware Científico

 Descrição:

	Montagem das principais matrizes para analise multivariada de
	acordo com a rotina MLMXLX.AJU e controle do parser dos comandos do 
	MULTVAR

 Desenvolvido por:

	Maurício Noronha Festa
	Sônia Ternes 
	Sílvio Roberto Medeiros Evangelista

 Informações do CVS:
        $Source: /cvsroot/soc/soc/src/multvar/fontes/mlmaju.c,v $:
        $Revision: 1.1 $:
        $Date: 2005/03/24 16:08:44 $:

***********************************************************************************/
static char *sccsid = "@(#)mlmaju.c	3.4	(NTIA/modlin)	95/05/30";

#include <stdlib.h>
#include <setjmp.h>
#include <string.h>

#include "mod_incl.h"
#include "getoken.h"
#include "pcsgbd.h"
#include "multvar.h"

/*
** Variavel global para possibilitar o "jump" no parser dos comandos
*/
jmp_buf inicio;


extern int imp_h;
extern int imp_e;
extern int imp_er;
extern int imp_v;
extern int imp_u;
extern int fl_uni;
extern int fl_uni_t;
extern int ctr;

/*
** Variaveis globais para controle de titulo e rotulo
*/
extern int ntit;
extern int nrot;

extern char titulo[96];
extern char rotulo[32];
extern char *nome_lin[MAXEFTS];
extern char nome_col[TAM];

extern Efeitos efeitos;

/*
** Montagem das principais matrizes para a analise multivariada e
** controle de execucao dos comandos EFEITO, CP e MEDREP
*/
void mlmaju(nomecol)
char *nomecol;
{
    extern void mod_init();

    int i, ncbeta;
    int token;
    char buffer[64];

    Simbolo_t *esp = (Simbolo_t *) NULL;
    Simbolo_t *xlx = (Simbolo_t *) NULL;
    Simbolo_t *ef = (Simbolo_t *) NULL;
    Simbolo_t *nly = (Simbolo_t *) NULL;
    Simbolo_t *ncx = (Simbolo_t *) NULL;
    Simbolo_t *xy = (Simbolo_t *) NULL;
    Simbolo_t *yy = (Simbolo_t *) NULL;
    Simbolo_t *ixx = (Simbolo_t *) NULL;
    Simbolo_t *beta = (Simbolo_t *) NULL;
    Simbolo_t *aux = (Simbolo_t *) NULL;
    Simbolo_t *cov_beta = (Simbolo_t *) NULL;
    Simbolo_t *v = (Simbolo_t *) NULL;
    Simbolo_t *dp_beta = (Simbolo_t *) NULL;
    Simbolo_t *v1 = (Simbolo_t *) NULL;
    Simbolo_t *nd = (Simbolo_t *) NULL;

    /*
     ** mod_include1.c
     */
    Simbolo_t *ipos = (Simbolo_t *) NULL;
    Simbolo_t *jpos = (Simbolo_t *) NULL;
    Simbolo_t *mpos = (Simbolo_t *) NULL;
    Simbolo_t *npos = (Simbolo_t *) NULL;

    mod_init();

    ipos = faztemp(1, 1);
    jpos = faztemp(1, 1);

    TIPO(ipos) = ID;
    TIPO(jpos) = ID;

    /*
     ** Inicio da traducao
     */

    /*
     ** xlxesp.dat contem no. de var. dependentes, no. de efeitos no modelo,
     ** e indice das colunas ocupadas por estes e mg em xlx:
     **
     ** no.dep;no.ef;0;1;col_fim_ef1; ...;col_fim_efn;col_vd1;...col_vdk
     */
    esp = leia("xlxesp", (char *) NULL, 0, (char *) NULL, 0.0);
    TIPO(esp) = ID;

    /*
     ** Condicao para analise multivariada: nd > 1
     */
    /* nd = esp[1,1]; */
    DADOij(ipos, 0, 0) = 1;
    DADOij(jpos, 0, 0) = 1;
    nd = matsubs(esp, ipos, jpos);
    TIPO(nd) = ID;

    if ((int) DADOij(nd, 0, 0) <= 1)
	mlmerro(46, 1);

    /*
     ** xlxmat e' a matriz inv(x'x) armazenada em disco
     */
    xlx = leia("xlxmat", (char *) NULL, 0, (char *) NULL, 0.0);
    TIPO(xlx) = ID;

    /*
     ** xlxefe.dat contem, para cada efeito, informacao sobre seu tipo,
     ** teste de erro, etc (ver vetor ef[][] no codigo do MODLIN
     */
    ef = leia("xlxefe", (char *) NULL, 0, (char *) NULL, 0.0);
    TIPO(ef) = ID;

    /* nly = xlx[1,1] */
    DADOij(ipos, 0, 0) = 1;
    DADOij(jpos, 0, 0) = 1;
    nly = matsubs(xlx, ipos, jpos);
    TIPO(nly) = ID;

    /* ncx = ncol(xlx) - esp[1,1] */
    ncx = matdif(mNcol(xlx), matsubs(esp, ipos, jpos));
    TIPO(ncx) = ID;
    libtemp();

    /* XY=xlx[1:(nlin(xlx)-esp[1,1]),(ncol(xlx)-esp[1,1]+1): ncol(xlx)]; */
    mpos = mod_gindex(1, (int) (NLINS(xlx) - DADOij(esp, 0, 0)));
    npos =
	mod_gindex((int) (NCOLS(xlx) - DADOij(esp, 0, 0) + 1),
		   (int) NCOLS(xlx));
    xy = matsubs(xlx, mpos, npos);
    TIPO(xy) = ID;
    libtemp();

    /*YY=xlx[(nlin(xlx)-esp[1,1]+1):nlin(xlx),(ncol(xlx)-esp[1,1]+1):ncol(xlx)]; */
    mpos =
	mod_gindex((int) ((NLINS(xlx) - DADOij(esp, 0, 0) + 1)),
		   (int) NLINS(xlx));
    npos =
	mod_gindex((int) (NCOLS(xlx) - DADOij(esp, 0, 0) + 1),
		   (int) NCOLS(xlx));
    yy = matsubs(xlx, mpos, npos);
    TIPO(yy) = ID;
    libtemp();

    /*IXX = inv(xlx[1:(nlin(xlx)-esp[1,1]), 1:(ncol(xlx)-esp[1,1])]); */
    mpos = mod_gindex(1, (int) (NLINS(xlx) - DADOij(esp, 0, 0)));
    npos = mod_gindex(1, (int) (NCOLS(xlx) - DADOij(esp, 0, 0)));
    ixx = mInv(matsubs(xlx, mpos, npos));
    TIPO(ixx) = ID;
    libtemp();

    /* xlx=; */
    libsim(xlx);

    /* BETA = ixx*xy */
    beta = matmult(ixx, xy);
    TIPO(beta) = ID;

    /* AUX  = yy-xy'*ixx*xy; */
    aux = matdif(yy, matmult(mattran(xy), matmult(ixx, xy)));
    TIPO(aux) = ID;
    libtemp();

    /* yy=; xy=; */
    libsim(yy);
    libsim(xy);

    /* COV_BETA = ixx@(aux/(nly-ncx)); */
    cov_beta = matkron(ixx, matdivd(matdif(nly, ncx), aux));
    TIPO(cov_beta) = ID;
    libtemp();

    /* V = sqrt(vdiag(cov_beta)); */
    v = mSqrt(mVdiag(cov_beta));
    TIPO(v) = ID;
    libtemp();

    /* NCBETA = ncol(beta); */
    ncbeta = (int) NCOLS(beta);

    /* DP_BETA = (v[1:ncbeta,])' */
    mpos = mod_gindex(1, ncbeta);
    dp_beta = mattran(matsubs(v, mpos, (Simbolo_t *) 0));
    TIPO(dp_beta) = ID;
    libtemp();

    /* i = ncbeta + 1;
       enquanto ( i <= nlin(v) ) {
       V1 = (v[i:(i+ncbeta-1),])';
       DP_BETA = { dp_beta , v1 };
       i = i + ncbeta;
       }
     */

    i = ncbeta + 1;
    while (i <= (int) NLINS(v)) {
	mpos = mod_gindex(i, i + ncbeta - 1);
	v1 = mod_atribuir(v1, mattran(matsubs(v, mpos, (Simbolo_t *) 0)));
	TIPO(v1) = ID;
	dp_beta = mod_atribuir(dp_beta, vconcat(dp_beta, v1));
	TIPO(dp_beta) = ID;
	i += ncbeta;
	libtemp();
    }

    /* v=; v1=; */
    libsim(v);

    if ((ncbeta + 1) <= (int) NLINS(v))	/* matriz v1 foi criada */
	if (v1 != (Simbolo_t *) NULL)
	    libsim(v1);

    /*
     ** Parser do MULTVAR
     */

    setjmp(inicio);

    iniciat();
    while (1) {
	token = gtoken1(buffer, stdin);

	if (strcmpi(buffer, "testem") == 0) {
	    inicia();
	    mlmger(ixx, esp, ef, nly, ncx, beta, cov_beta, dp_beta, aux);
	} else {
	    if (strcmpi(buffer, "cpm") == 0) {
		inicia();
		mlmcp(esp, ef, ixx, nly, ncx, beta, aux);
	    } else {
		if (strcmpi(buffer, "medrep") == 0) {
		    inicia();
		    mlmmed(ixx, esp, ef, nly, ncx, beta, aux);
		} else {
		    if (strcmpi(buffer, "tlm") == 0) {
			if (ntit != 0) {
			    inicia();
			    iniciat();
			    mlmerro(27, 0);
			} else
			    montatit();
		    } else {
			if (strcmpi(buffer, "}") == 0) {
			    break;
			} else {
			    mlmerro(2, 0);
			}
		    }
		}
	    }
	}
    }

    /*
     ** Liberar area alocada para as matrizes
     */
    libsim(ixx);
    libsim(esp);
    libsim(ef);
    libsim(nly);
    libsim(ncx);
    libsim(beta);
    libsim(cov_beta);
    libsim(dp_beta);
    libsim(aux);

    /*
     ** Apagar arquivos de entrada
     */
    unlink("xlxesp.dat");
    unlink("xlxesp.hdr");
    unlink("xlxmat.dat");
    unlink("xlxmat.hdr");
    unlink("xlxefe.dat");
    unlink("xlxefe.hdr");
}

void inicia()
{
    imp_h = 0;
    imp_e = 0;
    imp_er = 0;
    imp_v = 0;
    imp_u = 1;
    fl_uni = 0;
    fl_uni_t = 0;
    ctr = -1;
    nrot = 0;
    rotulo[0] = '\0';
}

void iniciat()
{
    ntit = 0;
    titulo[0] = '\0';
}
