/***********************************************************************************
 MULTVAR - Módulo para análise de modelos lineares multivariados

 Direitos Autorais Reservados (c) 2004 Embrapa Informática Agropecuária

 Este programa é software livre; você pode redistribuí-lo e/ou modificá-lo sob os
 termos da Licença Pública Geral GNU conforme publicada pela Free Software
 Foundation; tanto a versão 2 da Licença, como (a seu critério) qualquer versão
 posterior.

 Este programa é distribuído na expectativa de que seja útil, porém, SEM NENHUMA
 GARANTIA; nem mesmo a garantia implícita de COMERCIABILIDADE OU ADEQUAÇÃO A UMA
 FINALIDADE ESPECÍFICA. Consulte a Licença Pública Geral do GNU para mais detalhes.

 Você deve ter recebido uma cópia da Licença Pública Geral do GNU junto com este
 programa; se não, escreva para a Free Software Foundation, Inc., no endereço 59
 Temple Street, Suite 330, Boston, MA 02111-1307 USA.

 **********************************************************************************

 Parte integrante do SOC - SOftware Científico

 Descrição:

	Funcoes auxiliares para tratamente de titulo, rotulo, vetor de nomes, etc.
	Funcoes auxiliares para calculo do Univariado em medrep e testem.
	Funcoes para calculo da estatistica de Mauchly.

 Desenvolvido por:

	Maurício Noronha Festa
	Sônia Ternes 
	Sílvio Roberto Medeiros Evangelista

 Informações do CVS:
        $Source: /cvsroot/soc/soc/src/multvar/fontes/mlmaux.c,v $:
        $Revision: 1.1 $:
        $Date: 2005/03/24 16:08:44 $:

***********************************************************************************/
static char *sccsid = "@(#)mlmaux.c	3.4		95/07/26";

#include <stdlib.h>
#include <string.h>

#include "mod_incl.h"
#include "getoken.h"
#include "pcsgbd.h"
#include "multvar.h"

#define	MUDA_PAG 12

/*
** Variaveis globais
*/

extern char titulo[96];
extern char rotulo[32];
extern char string[9];
extern char *nome_lin[MAXEFTS];
extern char nome_col[TAM];

extern Efeitos efeitos;

extern int ntit;
extern int nrot;
extern int fl_imp;
extern int fl_analise;
extern int ctr;

/*
** Montagem do titulo do comando tlm baseado em ntit
*/
void montatit()
{
    int i;
    char c;

    while ((c = xgetc(stdin)) == ' ' || c == '\t' || c == '\n' || c == '\r');	/* tira brancos a direita */
    i = 0;
    while (c != ';' && i < 81) {
	titulo[i++] = (char) c;
	c = xgetc(stdin);
    }

    if (i >= 81)
	mlmerro(29, 0);

    titulo[i] = '\0';
    ntit = 1;
}

/*
** Impressao centralizada de um texto
*/
void imptitulo(texto, branco)
char *texto;
int branco;			/* numero de linhas em branco apos o titulo (0, 1 ou 2) */
{
    int i;
    int j;

    if (texto[0] != '\0') {
	printf("\n");
	i = strlen(texto);
	if (i < 79)
	    i = (80 - i) / 2;
	else
	    i = 0;
	for (j = 0; j < i; ++j)
	    printf(" ");

	if (branco == 0)
	    printf("%s", texto);
	else if (branco == 1)
	    printf("%s\n", texto);
	else
	    printf("%s\n\n", texto);
    }
}

/*
** Montagem do nome das linhas das matrizes H, E, V e Correl, no formato
** usado por printmat(): char **nome_lin
*/
void monta_lin(tipo_u, tipo_an, ndep)
int tipo_u;			/* 1: U = ident(nd); 0: U != ident(nd) */
int tipo_an;			/* comando sendo executado: testem, cpm ou medrep */
int ndep;			/* numero de variaveis dependentes */
{
    int i, k;
    int ini;

    char buf[8];

    ini = EFE_GetNefe(efeitos) - ndep;

    if (tipo_an <= 3) {		/* testem ou cpm */
	if (tipo_u == 1) {
	    /* var_dep1 ... var_depk */
	    for (i = 0; i < ndep; i++) {
		nome_lin[i] = (char *) malloc(16 * sizeof(char));
		strcpy(nome_lin[i], EFE_GetNome(efeitos, i + ini));
	    }
	} else {
	    /* vt1 ... vtk */
	    for (i = 0; i < ndep; i++) {
		nome_lin[i] = (char *) malloc(16 * sizeof(char));
		strcpy(nome_lin[i], "vt");
		itoa(i + 1, buf, 10);
		strcat(nome_lin[i], buf);
	    }
	}
    } else {			/* medrep */
	/* string1 ... stringk */
	k = 0;
	for (i = 0; i < ndep; i++) {
	    if (i != ctr) {
		nome_lin[k] = (char *) malloc(16 * sizeof(char));
		strcpy(nome_lin[k], string);
		itoa(i + 1, buf, 10);
		strcat(nome_lin[k], buf);
		k++;
	    }
	}

    }
}

/*
** Liberacao da area ocupada pelo vetor de nomes
*/
void lib_lin(ndep, tipo)
int ndep;
int tipo;
{
    int i;
    int k;

    if (tipo >= 4) {
	if (ctr == -1)
	    k = ndep;
	else
	    k = ndep - 1;
    } else
	k = ndep;

    for (i = 0; i < k; i++) {
	free(nome_lin[i]);
    }
}

/*
** Montagem do nome das colunas das matrizes H, E, V e Correl, no formato
** usado por printmat(): vetor de nomes separados por virgula
*/
void monta_col(tipo_u, tipo_an, ndep)
int tipo_u;			/* 1: U = ident(nd); 0: U != ident(nd) */
int tipo_an;			/* comando sendo executado: testem, cpm ou medrep */
int ndep;			/* numero de variaveis dependentes */
{
    int i, k;
    int ini;

    char buf[8];

    ini = EFE_GetNefe(efeitos) - ndep;

    if (tipo_an <= 3) {		/* testem ou cpm */
	if (tipo_u == 1) {
	    /* var_dep1 ... var_depk */
	    strcpy(nome_col, EFE_GetNome(efeitos, ini));
	    for (i = 1; i < ndep; i++) {
		strcat(nome_col, ",");
		strcat(nome_col, EFE_GetNome(efeitos, i + ini));
	    }
	} else {
	    /* vt1 ... vtk */
	    strcpy(nome_col, "vt1");
	    for (i = 1; i < ndep; i++) {
		strcat(nome_col, ",");
		strcat(nome_col, "vt");
		itoa(i + 1, buf, 10);
		strcat(nome_col, buf);
	    }
	}
    } else {			/* medrep */
	/* string1 ... stringk */
	if (ctr == 0) {
	    strcpy(nome_col, string);
	    itoa(2, buf, 10);
	    strcat(nome_col, buf);
	    k = 2;
	} else {
	    strcpy(nome_col, string);
	    itoa(1, buf, 10);
	    strcat(nome_col, buf);
	    k = 1;
	}

	for (i = k; i < ndep; i++) {
	    if (i != ctr) {
		strcat(nome_col, ",");
		strcat(nome_col, string);
		itoa(i + 1, buf, 10);
		strcat(nome_col, buf);
	    }
	}

    }
}

/*
** Montagem do rotulo do comando cpm baseado em nrot
*/
void montarot()
{
    int i;
    char c;

    while ((c = xgetc(stdin)) == ' ' || c == '\t' || c == '\n' || c == '\r');	/* tira brancos a direita */
    i = 0;
    while (c != ';' && i < 21) {
	rotulo[i++] = (char) c;
	c = xgetc(stdin);
    }

    if (i >= 21)
	mlmerro(30, 0);

    rotulo[i] = '\0';
    nrot = 1;
}

/*
** Inicializa nova pagina e imprime os titulos para TESTEM e CPM
*/

void putchar1()
{
    if (fl_imp)
	putchar(MUDA_PAG);

    imptitulo(titulo, 0);
    imptitulo("Analise de Variancia Multivariada", 1);
}

/*
** Inicializa nova pagina e imprime os titulos para MEDREP
*/

void putchar2(tipo)
int tipo;
{
    if (fl_imp)
	putchar(MUDA_PAG);

    imptitulo(titulo, 0);

    if (tipo)
	imptitulo("Analise de Variancia de Medidas Repetidas", 0);
    else
	imptitulo("Analise de Variancia de Medidas Repetidas", 1);

    if (tipo) {
	switch (fl_analise) {
	case 1:
	    imptitulo("Transformacao Utilizada: Perfil", 1);
	    break;
	case 2:
	    imptitulo("Transformacao Utilizada: Media", 1);
	    break;
	case 3:
	    imptitulo("Transformacao Utilizada: Helmert", 1);
	    break;
	case 4:
	    imptitulo("Transformacao Utilizada: Contraste", 1);
	    break;
	case 5:
	    imptitulo("Transformacao Utilizada: Polinomial", 1);
	    break;
	}
    }
}

/*
** Imprime titulo especifico para tipo de analise realizada em MEDREP.
*/
void tit_analise()
{
    int i;
    char buffer[128];
    char bufs[10];
    char buf[8];

    for (i = 0; i < strlen(string); i++)
	bufs[i] = toupper(string[i]);
    bufs[i] = '\0';

    switch (fl_analise) {
    case 1:			/* PERFIL */
	strcpy(buffer, bufs);
	strcat(buffer, "i");
	strcat(buffer,
	       " representa o contraste entre o nivel i e o nivel i+1 de ");
	strcat(buffer, bufs);
	imptitulo(buffer, 1);
	break;

    case 2:			/* MEDIA */
	strcpy(buffer, bufs);
	strcat(buffer, "i");
	strcat(buffer,
	       " representa o contraste entre o nivel i e a media");
	imptitulo(buffer, 0);

	strcpy(buffer, "dos outros niveis de ");
	strcat(buffer, bufs);
	imptitulo(buffer, 1);
	break;

    case 3:			/* HELMERT */
	strcpy(buffer, bufs);
	strcat(buffer, "i");
	strcat(buffer,
	       " representa o contraste entre o nivel i e a media");
	imptitulo(buffer, 0);

	strcpy(buffer, "dos subsequentes niveis de ");
	strcat(buffer, bufs);
	imptitulo(buffer, 1);
	break;

    case 4:			/* CONTRASTE */
	strcpy(buffer, bufs);
	strcat(buffer, "i");
	strcat(buffer,
	       " representa o contraste entre o nivel i e o nivel ");
	sprintf(buf, "%d", ctr + 1);
	strcat(buffer, buf);
	strcat(buffer, " de ");
	strcat(buffer, bufs);
	imptitulo(buffer, 1);
	break;

    case 5:			/* POL */
	strcpy(buffer, bufs);
	strcat(buffer, "i");
	strcat(buffer,
	       " representa o contraste polinomial de grau i para ");
	strcat(buffer, bufs);
	imptitulo(buffer, 1);
	break;
    }
}

/*
** Verifica se as matrizes x e y sao iguais.
** Retorna 1 se forem iguais, e 0 caso contrario.
*/
int igualm(x, y)
Simbolo_t *x;
Simbolo_t *y;
{
    int i, j;
    int ok;


    if ((NLINS(x) != NLINS(y)) || (NCOLS(x) != NCOLS(y)))
	return (0);

    ok = 1;
    for (i = 0; i < NLINS(x); i++) {
	for (j = 0; j < NCOLS(x); j++) {
	    if (DADOij(x, i, j) != DADOij(y, i, j)) {
		ok = 0;
		break;
	    }
	}
	if (ok == 0)
	    break;
    }

    return (ok);
}

/*
** Imprime quadro de correspondencia entre variaveis dependentes e string
** para a analise MEDREP
*/
void quadro(ndep, vpol)
int ndep;
Simbolo_t *vpol;
{
    int i, ini;
    char buf[9];
    char buffer[32];

    ini = EFE_GetNefe(efeitos) - ndep;

    printf("\n");
    printf("Variavel Dependente:");

    for (i = 0; i < ndep; i++) {
	if ((i + 1) % 6 == 0)
	    printf("\n                    ");
	printf("%10s", EFE_GetNome(efeitos, i + ini));
    }
    printf("\n\n");

    strcpy(buffer, "Nivel de ");
    strcat(buffer, string);
    printf("%19s:", buffer);

    if (vpol == (Simbolo_t *) NULL) {
	for (i = 0; i < ndep; i++) {
	    if ((i + 1) % 6 == 0)
		printf("\n                    ");
	    printf("%10d", i + 1);
	}
    } else {
	for (i = 0; i < ndep; i++) {
	    if ((i + 1) % 6 == 0)
		printf("\n                    ");
	    itoa((int) DADOij(vpol, 0, i), buf, 10);
	    printf("%10s", buf);
	}
    }
    printf("\n\n");
}

/*
** Executa analise univariada
*/
void exec_univar(tipo, u, esp, ef, dim, nd, ixx, nly, beta, aux)
int tipo;			/* 1: MEDREP; 0: TESTEM */
Simbolo_t *u;
Simbolo_t *esp;
Simbolo_t *ef;
int dim;
Simbolo_t *nd;
Simbolo_t *ixx;
Simbolo_t *nly;
Simbolo_t *beta;
Simbolo_t *aux;
{

    int i, j, k, ierro, quant, ini, primeiro, um;
    int gl_ef, gl_er, ig;

    double SQef, SQer;

    char buffer[16];
    char buf[8];

    Simbolo_t *ipos;
    Simbolo_t *uk;
    Simbolo_t *c;
    Simbolo_t *Cer;
    Simbolo_t *m;
    Simbolo_t *maux;
    Simbolo_t *aux1;
    Simbolo_t *h;
    Simbolo_t *e;

    /*
     ** Efeito Univariado: C,M automaticos para cada efeito e U 
     ** montada a partir do parametro (u): U1 = U[,1] ... Un = U[,n]
     */

    ipos = faztemp(1, 1);
    TIPO(ipos) = ID;

    if (tipo) {
	putchar2(1);
	ig = 0;
    } else {
	putchar1();

	aux1 = mIdent(nd);
	TIPO(aux1) = ID;
	ig = igualm(u, aux1);
	libsim(aux1);

	ini = EFE_GetNefe(efeitos) - (int) DADOij(nd, 0, 0);
    }

    um = 0;
    for (k = 0; k < NCOLS(u); k++) {
	DADOij(ipos, 0, 0) = k + 1;
	uk = matsubs(u, (Simbolo_t *) 0, ipos);
	TIPO(uk) = ID;

	if (k == 0) {
	    if (ig)
		imptitulo("Analise de Variancia das Variaveis Originais",
			  1);
	    else
		imptitulo
		    ("Analise de Variancia das Variaveis Transformadas",
		     1);
	    if (tipo)
		tit_analise();
	}

	if (tipo)
	    i = 1;
	else {
	    if ((EFE_GetNefe(efeitos) - (int) DADOij(nd, 0, 0)) > 1) {
		if (escolhido(0) == 1)
		    i = 1;
		else
		    i = 2;
	    } else
		i = 1;
	}

	primeiro = 1;
	while (i <= (int) (DADOij(esp, 0, 1) + 1.0)) {
	    if ((tipo) || (escolhido(i - 1) == 1)) {
		monta_CM(i, &ierro, &c, &m, uk, esp, ef, dim, nd);

		if (ierro != -1) {
		    Cer = mod_criamat((int)
				      (DADOij(esp, 0, 2 + ierro) -
				       DADOij(esp, 0, 1 + ierro)), dim,
				      0.0);
		    gl_er = (int) NLINS(Cer);
		    libtemp();

		} else {
		    maux = mposto(ixx);
		    gl_er = (int) (DADOij(nly, 0, 0)) -
			(int) (DADOij(maux, 0, 0));
		}

		/*  Calcula  matrizes H e E */
		calc_h_e(ierro, esp, ixx, beta, c, uk, m, aux, &h, &e);

		/* Soma de quadrados/grau liberdade do efeito */
		SQef = DADOij(h, 0, 0);
		gl_ef = (int) NLINS(c);

		SQer = DADOij(e, 0, 0);

		/* imprime tabela ANOVA */
		if (tipo) {
		    if (i == 1) {
			if (k != ctr)
			    j = k + um;
			else {
			    j = k + 1;
			    um = 1;
			}

			/*Indicacao col.U para analise */
			strcpy(buffer, string);
			itoa(j + 1, buf, 10);
			strcat(buffer, buf);
			printf("\nVariavel: %s\n\n", buffer);

			printf
			    ("Fonte de variacao       gl  Soma de quadrados  Quadrado medio   Valor F  PR > F\n");
			printf("                            Parcial\n\n");
			print_anova(tipo, "mg", SQef, SQer, gl_ef, gl_er);

		    } else {
			print_anova(tipo, EFE_GetNome(efeitos, i - 1),
				    SQef, SQer, gl_ef, gl_er);
		    }
		} else {
		    if (primeiro) {
			/*Indicacao col.U para analise */
			if (ig) {
			    printf("\nVariavel: %s\n\n",
				   EFE_GetNome(efeitos, k + ini));
			} else {
			    strcpy(buffer, "vt");
			    itoa(k + 1, buf, 10);
			    strcat(buffer, buf);
			    printf("\nVariavel: %s\n\n", buffer);
			}

			printf
			    ("Fonte de variacao       gl  Soma de quadrados  Quadrado medio   Valor F  PR > F\n");
			printf("                            Parcial\n\n");
			print_anova(tipo, EFE_GetNome(efeitos, i - 1),
				    SQef, SQer, gl_ef, gl_er);

			primeiro = 0;
		    } else {
			print_anova(tipo, EFE_GetNome(efeitos, i - 1),
				    SQef, SQer, gl_ef, gl_er);
		    }
		}

		/*  atualiza indice i  */
		if (DADOij(ef, i - 1, 3) > 0.0) {
		    quant = (int) (DADOij(ef, i - 1, 3)) - i + 1;
		    i += (quant + 1);
		} else {
		    i++;
		}
		libsim(e);
		libsim(h);
		libsim(c);
		libsim(m);
	    } /* efeito nao escolhido em testem */
	    else
		i++;
	}

	libtemp();

	/*
	 ** SQer para o erro padrao residual: E = u'*aux*u 
	 */

	e = matmult(matmult(mattran(uk), aux), uk);
	SQer = DADOij(e, 0, 0);
	TIPO(e) = ID;
	libtemp();

	maux = mposto(ixx);
	gl_er = (int) (DADOij(nly, 0, 0)) - (int) (DADOij(maux, 0, 0));

	print_erro("Residuo", SQer, gl_er);

	libtemp();
	libsim(e);
	libsim(uk);
    }
}

void print_anova(tipo, nome, SQef, SQer, gl_ef, gl_er)
int tipo;			/* 0: TESTEM; 1: Grupo, Univariado; 2: Split-Plot */
char *nome;			/* nome do efeito */
double SQef;
double SQer;
int gl_ef;
int gl_er;
{
    char buf[20];
    double QMef, QMer;
    double F_ef, p_v;

    if (tipo == 2) {
	if (strcmpi(nome, "mg") == 0)
	    /* teste do efeito  definido por "string" */
	    strcpy(buf, string);
	else {
	    if (strlen(nome) < 10) {
		strcpy(buf, nome);
		strcat(buf, "*");
		strcat(buf, string);
	    } else
		sprintf(buf, "%s*%10s", nome, string);
	}
    } else {
	strcpy(buf, nome);
    }

    if (gl_er > 1.0e-14)
	QMer = SQer / (double) gl_er;
    else
	QMer = 9999.9999;

    if (gl_ef > 1.0e-14)
	QMef = SQef / (double) gl_ef;
    else
	QMef = 9999.9999;


    if (QMer > 1.0e-14)
	F_ef = QMef / QMer;
    else
	F_ef = 9999.9999;

    if (F_ef > 9999.0) {
	F_ef = 9999.9999;
	p_v = 0.0;
    }

    if (F_ef > 9999.0) {
	printf("%-20.20s  %4d %17s %16s %9s  %6s\n", buf, gl_ef,
	       rounddec(SQef, 17, 8),
	       rounddec(QMef, 16, 8),
	       rounddec(F_ef, 9, 4), rounddec(p_v, 6, 3));
    } else {
	printf("%-20.20s  %4d %17s %16s %9s  %6s\n", buf, gl_ef,
	       rounddec(SQef, 17, 8),
	       rounddec(QMef, 16, 8),
	       rounddec(F_ef, 9, 4),
	       rounddec(fprob(F_ef, (double) gl_ef, (double) gl_er), 6,
			3));
    }
}

void print_erro(nome, SQer, gl_er)
char *nome;
double SQer;
int gl_er;
{
    double QMer;

    if (gl_er > 1.0e-14)
	QMer = SQer / (double) gl_er;
    else
	QMer = 9999.9999;

    printf("%-20.20s  %4d %17s %16s\n", nome, gl_er,
	   rounddec(SQer, 17, 8), rounddec(QMer, 16, 8));

    printf
	("-------------------------------------------------------------------------------\n");

}

/*
** Calcula Teste de Esfericidade de acordo com Mauchly.
*/

void mauchly(u, e, nd, v)
Simbolo_t *u;
Simbolo_t *e;
Simbolo_t *nd;
double v;
{
    double gl;
    double ndep, qui, pv;

    Simbolo_t *a = (Simbolo_t *) NULL;
    Simbolo_t *b = (Simbolo_t *) NULL;
    Simbolo_t *m1 = (Simbolo_t *) NULL;
    Simbolo_t *e1 = (Simbolo_t *) NULL;
    Simbolo_t *mnd = (Simbolo_t *) NULL;
    Simbolo_t *mlog = (Simbolo_t *) NULL;

    mnd = mod_criamat(1, 1, 1.0);
    TIPO(mnd) = ID;
    DADOij(mnd, 0, 0) = DADOij(nd, 0, 0) - 1.0;

    /* m1 = ((ndep-1)^(ndep-1)*det(e))/(traco(e)^(ndep-1)) */
    m1 = matdivd(matpot(mTraco(e), mnd),
		 matmult(matpot(mnd, mnd), mdet(e)));
    TIPO(m1) = ID;

    /* qui = -(v-(2*ndep^2-3*ndep+3)/(6*(ndep-1))*log(m1) */
    mlog = mLog(m1);
    TIPO(mlog) = ID;

    ndep = DADOij(nd, 0, 0);

    qui =
	(-1.0) * (v -
		  (2.0 * ndep * ndep - 3.0 * ndep +
		   3.0) / (6.0 * (ndep - 1.0))) * DADOij(mlog, 0, 0);

    gl = (0.5 * ndep * (ndep - 1.0)) - 1.0;

    pv = xprob(qui, gl);

    imptitulo("Teste de Esfericidade", 2);
    printf(". Aplicado a Matriz U Utilizada na Transformacao:\n");
    printf("\n");
    printf("  Criterio de Mauchly = %.7lf\n", DADOij(m1, 0, 0));
    printf("\n");
    printf("         Qui-Quadrado             gl         Pr > Qui\n");
    printf("\n");
    /*
       printf("%21.5lf %14d %16.5lf\n",qui,(int)gl,pv);
     */
    printf("%21s %14d %16.5lf\n", rounddec(qui, 21, 5), (int) gl, pv);

    libsim(m1);
    libsim(mnd);
    libsim(mlog);
    libtemp();

    if (fl_analise != 5) {	/* Nao e' do tipo POL */

	/* a,b = gs(u) */
	MOD_GS(u, a, b);

	/* e1 = (inv(a)')*e*inv(a) */
	e1 = matmult(mattran(mInv(a)), matmult(e, mInv(a)));
	TIPO(e1) = ID;
	libtemp();

	mnd = mod_criamat(1, 1, 1.0);
	TIPO(mnd) = ID;
	DADOij(mnd, 0, 0) = DADOij(nd, 0, 0) - 1.0;

	/* m1 = ((ndep-1)^(ndep-1)*det(e1))/(traco(e1)^(ndep-1)) */
	m1 = matdivd(matpot(mTraco(e1), mnd), (matmult(matpot(mnd, mnd),
						       mdet(e1))));
	TIPO(m1) = ID;

	/* qui = -(v-(2*ndep^2-3*ndep+3)/(6*(ndep-1))*log(m1) */
	mlog = mLog(m1);
	TIPO(mlog) = ID;

	ndep = DADOij(nd, 0, 0);
	qui =
	    (-1.0) * (v -
		      (2.0 * ndep * ndep - 3.0 * ndep +
		       3.0) / (6.0 * (ndep - 1.0))) * DADOij(mlog, 0, 0);

	gl = (0.5 * ndep * (ndep - 1.0)) - 1.0;

	pv = xprob(qui, gl);

	printf("\n");
	printf(". Aplicado a Matriz de Componentes Ortonormais:\n");
	printf("\n");
	printf("  Criterio de Mauchly = %.7lf\n", DADOij(m1, 0, 0));
	printf("\n");
	printf("         Qui-Quadrado             gl         Pr > Qui\n");
	printf("\n");
	/*
	   printf("%21.5lf %14d %16.5lf\n",qui,(int)gl,pv);
	 */
	printf("%21s %14d %16.5lf\n", rounddec(qui, 21, 5), (int) gl, pv);

	libsim(a);
	libsim(b);
	libsim(m1);
	libsim(e1);
	libsim(mnd);
	libsim(mlog);
	libtemp();
    }
    printf("\n");
}
