/***********************************************************************************
 MULTVAR - Módulo para análise de modelos lineares multivariados

 Direitos Autorais Reservados (c) 2004 Embrapa Informática Agropecuária

 Este programa é software livre; você pode redistribuí-lo e/ou modificá-lo sob os
 termos da Licença Pública Geral GNU conforme publicada pela Free Software
 Foundation; tanto a versão 2 da Licença, como (a seu critério) qualquer versão
 posterior.

 Este programa é distribuído na expectativa de que seja útil, porém, SEM NENHUMA
 GARANTIA; nem mesmo a garantia implícita de COMERCIABILIDADE OU ADEQUAÇÃO A UMA
 FINALIDADE ESPECÍFICA. Consulte a Licença Pública Geral do GNU para mais detalhes.

 Você deve ter recebido uma cópia da Licença Pública Geral do GNU junto com este
 programa; se não, escreva para a Free Software Foundation, Inc., no endereço 59
 Temple Street, Suite 330, Boston, MA 02111-1307 USA.

 **********************************************************************************

 Parte integrante do SOC - SOftware Científico

 Descrição:

	Calcula e imprime a matriz de correlacoes e nivel de significancia, 
	a partir dos valores da matriz H.

 Desenvolvido por:

	Maurício Noronha Festa
	Sônia Ternes 
	Sílvio Roberto Medeiros Evangelista

 Informações do CVS:
        $Source: /cvsroot/soc/soc/src/multvar/fontes/mlmcor.c,v $:
        $Revision: 1.1 $:
        $Date: 2005/03/24 16:08:44 $:

***********************************************************************************/
static char *sccsid = "@(#)mlmcor.c	3.4	(NTIA/modlin)	95/05/30";

#include <stdlib.h>
#include <string.h>

#include "mod_incl.h"
#include "pcsgbd.h"
#include "multvar.h"

extern Efeitos efeitos;

extern char string[9];
extern char *nome_lin[MAXEFTS];
extern char nome_col[TAM];

extern int ctr;

/*
** Controle de impressao e calculo da matriz de correlacoes e p-value associado 
*/
void mlmcor(u, e, ixx, esp, nly, nd, tipo, ig)
Simbolo_t *u;
Simbolo_t *e;
Simbolo_t *ixx;
Simbolo_t *esp;
Simbolo_t *nly;
Simbolo_t *nd;
int tipo;
int ig;
{
    int i, j, k, l, ind, uml, umc;

    int inic;			/* inicio de linha e coluna a imprimir */
    int fimc;			/* coluna final a imprimir */
    int ncor;			/* limite de linha e coluna a imprimir */
    int lim;			/* numero de colunas a serem impressas por linha */

    double correl;		/* valor da correlacao */
    double pv[7];		/* valor do p_value para cada correl na linha */

    /*
     ** Inicializa tamanhos para impressao
     */
    l = EFE_GetNefe(efeitos) - (int) DADOij(nd, 0, 0);
    inic = 0;
    ncor = NLINS(e);
    lim = 7;
    fimc = (ncor > lim) ? lim : ncor;

    if (ig)
	imptitulo("Matriz de Correlacoes Parciais / Prob > |r|", 2);
    else {
	imptitulo("Matriz de Correlacoes Parciais", 0);
	imptitulo("das Variaveis Transformadas / Prob > |r|", 2);
    }

    /*
     ** Looping de impressao
     */
    printf("\n");
    umc = 0;
    uml = 0;
    do {
	printf("%9.9s ", " ");
	for (j = inic; j < fimc; ++j) {
	    if (tipo >= 4) {
		if (j != ctr)
		    ind = j + umc;
		else {
		    ind = j + 1;
		    umc = 1;
		}
		printa(u, nd, l, ind, ig, tipo);
	    } else
		printa(u, nd, l, j, ig, tipo);
	}
	printf("\n\n");

	for (i = inic; i < ncor; ++i) {
	    if (tipo >= 4) {
		if (i != ctr)
		    ind = i + uml;
		else {
		    ind = i + 1;
		    uml = 1;
		}

		printa(u, nd, l, ind, ig, tipo);
	    } else
		printa(u, nd, l, i, ig, tipo);

	    /* calcula p_value e correl */
	    k = 0;
	    for (j = inic; j <= ((i < (fimc - 1)) ? i : (fimc - 1)); ++j) {
		if (i == j) {
		    correl = 1.0;
		    pv[k++] = 0.0;
		} else {
		    correl = calc_cor(e, i, j);
		    if (fabs(correl) < EPS)
			pv[k++] = 1.0;
		    else {
			if (fabs(correl - 1.0) < EPS)
			    pv[k++] = 0.0;
			else
			    pv[k++] = calc_pv(ixx, nly, correl);
		    }
		}
		printf("%9.4lf ", correl);
	    }

	    printf("\n%9s", " ");

	    /* imprime p_value */
	    k = 0;
	    for (j = inic; j <= ((i < (fimc - 1)) ? i : (fimc - 1)); ++j)
		printf("   (%5.3lf)", pv[k++]);
	    printf("\n\n");

	}
	inic = fimc;
	fimc += lim;
	if ((fimc > ncor) && (inic < ncor)) {
	    fimc = ncor;
	    printf("\n\n");
	}
    } while (fimc <= ncor);
}

/*
** Calculo da correlacao entre as variaveis de resposta i,j
*/
double calc_cor(e, i, j)
Simbolo_t *e;
int i;
int j;
{
    double r;

    if ((DADOij(e, i, i) == 0.0) && (DADOij(e, j, j) == 0))
	r = 1.0;
    else if ((DADOij(e, i, i) == 0.0) || (DADOij(e, j, j) == 0))
	r = 0.0;
    else
	r = DADOij(e, i, j) / sqrt(DADOij(e, i, i) * DADOij(e, j, j));

    return (r);
}

/*
** Calculo do p-value associado a correlacao r
*/
double calc_pv(ixx, nly, r)
Simbolo_t *ixx;
Simbolo_t *nly;
double r;
{
    double ylin;
    double t;
    double p;
    double pv;

    Simbolo_t *mp = (Simbolo_t *) NULL;

    ylin = DADOij(nly, 0, 0);
    mp = mposto(ixx);
    p = DADOij(mp, 0, 0);

    t = (r * sqrt(ylin - p - 1.0)) / sqrt(1.0 - r * r);

    pv = 2.0 * tprob(fabs(t), ylin - p - 1.0);

    return (pv);
}

/*
** Imprime o nome da linha e/ou coluna da matriz de correlacoes.
** Se U = ident(nd), (ig = 1), o nome da variavel de resposta e' 
** usado para impressao, caso contrario (ig = 0), gera os nomes 
** vt1,vt2,... (para comando cpm) ou string1,string2,... (para o
** comando medrep), e imprime como nome da linha e/ou coluna.
*/
void printa(u, nd, i1, i2, ig, tipo)
Simbolo_t *u;
Simbolo_t *nd;
int i1;
int i2;
int ig;
int tipo;
{
    char buf[8];		/* auxiliar para itoa() */
    char buffer[16];		/* string para impressao */

    if (ig) {
	printf("%9.9s ", EFE_GetNome(efeitos, i1 + i2));
    } else {
	if (tipo >= 4)
	    strcpy(buffer, string);	/* medrep */
	else
	    strcpy(buffer, "vt");	/* cpm */

	itoa(i2 + 1, buf, 10);
	strcat(buffer, buf);
	printf("%9.9s ", buffer);
    }
}
