/***********************************************************************************
 MULTVAR - Módulo para análise de modelos lineares multivariados

 Direitos Autorais Reservados (c) 2004 Embrapa Informática Agropecuária

 Este programa é software livre; você pode redistribuí-lo e/ou modificá-lo sob os
 termos da Licença Pública Geral GNU conforme publicada pela Free Software
 Foundation; tanto a versão 2 da Licença, como (a seu critério) qualquer versão
 posterior.

 Este programa é distribuído na expectativa de que seja útil, porém, SEM NENHUMA
 GARANTIA; nem mesmo a garantia implícita de COMERCIABILIDADE OU ADEQUAÇÃO A UMA
 FINALIDADE ESPECÍFICA. Consulte a Licença Pública Geral do GNU para mais detalhes.

 Você deve ter recebido uma cópia da Licença Pública Geral do GNU junto com este
 programa; se não, escreva para a Free Software Foundation, Inc., no endereço 59
 Temple Street, Suite 330, Boston, MA 02111-1307 USA.

 **********************************************************************************

 Parte integrante do SOC - SOftware Científico

 Descrição:

	Parser e analise do comando CP especificado pelo usuario do MULTVAR
	
	Sintaxe: CPM <lista_de_efeitos> [, LISTE H E V];
		[ROTM = <rotulo>;]
		[M = <matriz_m>;]
		[U = <matriz_u>;]
		C = <matriz_c>;

 Desenvolvido por:

	Maurício Noronha Festa
	Sônia Ternes 
	Sílvio Roberto Medeiros Evangelista

 Informações do CVS:
        $Source: /cvsroot/soc/soc/src/multvar/fontes/mlmcp.c,v $:
        $Revision: 1.1 $:
        $Date: 2005/03/24 16:08:44 $:

***********************************************************************************/
static char *sccsid = "@(#)mlmcp.c	3.5	(NTIA/modlin)	95/04/26";


#include "mod_incl.h"

#include <stdlib.h>
#include <setjmp.h>
#include <string.h>
#include <math.h>

#include "getoken.h"
#include "pcsgbd.h"
#include "multvar.h"

/*
** Estruturas usadas
*/
extern Efeitos efeitos;		/* Macros: EFE_GetNome, EFE_GetNefe, EFE_SetNome, 
				   EFE_SetNefe */
static Cps cps;			/* Macros: CP_GetEfe, CP_GetNefe,CP_SetEfe,CP_SetNefe */

extern int imp_h;
extern int imp_e;
extern int imp_er;
extern int imp_v;

/*
** Variaveis globais para controle do rotulo
*/
extern int nrot;
extern char rotulo[32];

/*
** Flags globais para impressao das matrizes U e M em mlmhip()
*/
int flc = 0;
int flm = 0;
int flu = 0;

/*
** Funcao principal para controle de execucao do comando CP
*/
void mlmcp(esp, ef, ixx, nly, ncx, beta, aux)
Simbolo_t *esp;
Simbolo_t *ef;
Simbolo_t *ixx;
Simbolo_t *nly;
Simbolo_t *ncx;
Simbolo_t *beta;
Simbolo_t *aux;
{
    /*
     ** Monta as matrizes C completa, U e M, de acordo com o especificado 
     ** pelo usuario, 
     */

    int i, ind_efe, ierro, ierro1, ier;

    int ndep, ncol;

    int col_c, lin_c;		/* numero de linhas e colunas de C completa */
    int col_m, lin_m;		/* controle de linhas e colunas da matriz M */
    int col_u, lin_u;		/* controle de linhas e colunas da matriz U */

    int col_usu;		/* numero de colunas das matrizes C e M 
				   que o usuario deve especificar */

    int esp_acu[MAXEFEITOSCP];	/* vetor com contagem acumulada das 
				   colunas ocupadas pelos efeitos 
				   especificados no CP */
    int tem_rot = 0;
    int token, primeiro;
    char buffer[64];

    Simbolo_t *U = (Simbolo_t *) NULL;
    Simbolo_t *M = (Simbolo_t *) NULL;
    Simbolo_t *C = (Simbolo_t *) NULL;
    Simbolo_t *linha = (Simbolo_t *) NULL;
    Simbolo_t *temp = (Simbolo_t *) NULL;
    Simbolo_t *temp1 = (Simbolo_t *) NULL;
    Simbolo_t *nd = (Simbolo_t *) NULL;
    Simbolo_t *ipos = (Simbolo_t *) NULL;
    Simbolo_t *jpos = (Simbolo_t *) NULL;

    /*
     ** Monta a estrutura cps, inserindo os efeitos especificados
     ** no comando CP.
     */

    mlm_CP(ef);

    /* 
     ** Verifica teste de erro especifico.
     ** Todos os efeitos do CP devem ter sido especificados sobre o mesmo 
     ** erro.
     */
    for (i = 0; i < CP_GetNefe(cps); i++) {
	ind_efe = CP_GetEfe(cps, i);
	ier = (int) DADOij(ef, ind_efe, 2);
	if (ier != 0) {
	    if (i == 0)
		ierro = (int) DADOij(ef, ind_efe, 2) + 1;
	    else
		ierro1 = (int) DADOij(ef, ind_efe, 2) + 1;
	} else {
	    if (i == 0)
		ierro = -1;
	    else
		ierro1 = -1;
	}
	if (i != 0 && ierro != ierro1)
	    mlmerro(10, 0);
    }

    /*
     ** Calculo do numero de variaveis dependentes
     ** nd = esp[1,1]
     */
    ipos = faztemp(1, 1);
    TIPO(ipos) = ID;
    jpos = faztemp(1, 1);
    TIPO(jpos) = ID;

    DADOij(ipos, 0, 0) = 1;
    DADOij(jpos, 0, 0) = 1;
    nd = matsubs(esp, ipos, jpos);
    TIPO(nd) = ID;

    /*
     ** Calculo das variaveis que especificam as linhas e colunas de C,U e M
     */

    ndep = (int) DADOij(nd, 0, 0);
    ncol = NCOLS(ixx);

    /*
     ** Calculo do numero de colunas da matriz C que o usuario deve 
     ** especificar
     */
    col_usu = 0;
    for (i = 0; i < CP_GetNefe(cps); i++) {
	ind_efe = CP_GetEfe(cps, i);
	if (ind_efe == 0)	/* media geral */
	    col_usu++;
	else {
	    col_usu +=
		(int) (DADOij(esp, 0, 3 + ind_efe) -
		       DADOij(esp, 0, 2 + ind_efe));
	}
    }

    /*
     ** Montagem de esp_acu[]
     */
    esp_acu[0] = 0;
    for (i = 0; i < CP_GetNefe(cps); i++) {
	ind_efe = CP_GetEfe(cps, i);
	if (ind_efe == 0)	/* media geral */
	    esp_acu[i + 1] = 1;
	else {
	    esp_acu[i + 1] = esp_acu[i] +
		(int) (DADOij(esp, 0, 3 + ind_efe) -
		       DADOij(esp, 0, 2 + ind_efe));
	}
    }

    /*
     ** Parser do comando ROT
     */
    token = gtoken1(buffer, stdin);
    if (strcmpi(buffer, "rotm") == 0) {
	if (nrot != 0) {
	    mlmerro(28, 0);
	    nrot = 0;
	} else {
	    montarot();
	    tem_rot = 1;
	}
    }

    /*
     ** Parser das matrizes C, U e M
     */

    flc = flu = flm = 0;
    primeiro = 1;

    while (!flc) {

	if (primeiro == 1) {
	    if (tem_rot == 1) {
		token = gtoken1(buffer, stdin);
		primeiro = 0;
	    }
	} else
	    token = gtoken1(buffer, stdin);

	/* tem que ser: C, U ou M */
	/*
	   if (token != VARIAV)
	   mlmerro(2,0);
	 */

	if (strcmpi(buffer, "c") == 0)
	    flc = 1;
	else if (strcmpi(buffer, "m") == 0)
	    flm = 1;
	else if (strcmpi(buffer, "u") == 0)
	    flu = 1;
	else if (tem_rot == 0)
	    mlmerro(47, 0);	/* msg com rotm */
	else
	    mlmerro(38, 0);	/* msg sem rotm */

	/*
	   if ((flu == 0) && (flm == 0) && (flc == 0))
	   mlmerro(38,0);
	 */

	token = gtoken1(buffer, stdin);

	/* tem que ser = */
	if (token != IGUAL)
	    mlmerro(0, 0);

	if (flc) {
	    lin_c = 0;
	    col_c = 0;

	    C = mod_criamat(1, ncol, 0.0);
	    TIPO(C) = ID;
	    linha = mod_criamat(1, ncol, 0.0);
	    TIPO(linha) = ID;

	    token = gtoken1(buffer, stdin);
	    if (token != NUMBER)
		mlmerro(15, 0);

	    /* Criar uma linha completa da matriz c */

	    while (token != PTVG) {
		if (token != NUMBER && token != VG)
		    mlmerro(3, 0);

		if (token == VG) {
		    /* Concatenar mais uma linha */
		    /* `a matriz C               */

		    if (col_c < col_usu)
			mlmerro(4, 0);


		    if (lin_c != 0) {
			if (NCOLS(C) != NCOLS(linha))
			    mlmerro(6, 0);

			temp = vconcat(C, linha);
			C = mod_atribuir(C, temp);
		    }
		    col_c = 0;
		    lin_c++;
		} else {

		    if (col_c >= col_usu)
			mlmerro(4, 0);

		    /* A primeira linha deve ser colocada em C */
		    if (lin_c == 0)
			DADOij(C, 0, col_c_atual(esp, esp_acu, col_c)) =
			    atof(buffer);
		    else
			DADOij(linha, 0,
			       col_c_atual(esp, esp_acu, col_c)) =
			    atof(buffer);

		    col_c++;
		}
		token = gtoken1(buffer, stdin);
	    }

	    if (col_c != col_usu)
		mlmerro(4, 0);

	    /* Caso exista mais do que uma linha, deve-se 
	       concatenar a ultima linha */

	    if (lin_c > 0) {
		if (NCOLS(C) != NCOLS(linha))
		    mlmerro(6, 0);

		temp = vconcat(C, linha);
		C = mod_atribuir(C, temp);
	    }
	    lin_c++;

	    /* Compatibilidade entre C e M */
	    if (flm != 0)
		if (lin_m != lin_c)
		    mlmerro(5, 0);

	    libsim(linha);
	}

	if (flu == 1) {
	    lin_u = 0;
	    col_u = 0;

	    U = mod_criamat(1, 1, 0.0);
	    TIPO(U) = ID;
	    linha = mod_criamat(1, 1, 0.0);
	    TIPO(linha) = ID;
	    temp1 = faztemp(1, 1);
	    TIPO(temp1) = ID;

	    token = gtoken1(buffer, stdin);
	    if (token != NUMBER)
		mlmerro(15, 0);

	    while (token != PTVG) {
		if (token != NUMBER && token != VG)
		    mlmerro(3, 0);

		if (token == VG) {
		    if (flm != 0)
			if (col_m != col_u)
			    mlmerro(8, 0);

		    /* Concatenar mais uma linha */
		    if (lin_u > 0) {
			if (NCOLS(U) != NCOLS(linha))
			    mlmerro(6, 0);

			temp = vconcat(U, linha);
			U = mod_atribuir(U, temp);
		    }
		    col_u = 0;
		    lin_u++;
		} else {
		    if (lin_u >= ndep)
			mlmerro(7, 0);

		    if (lin_u == 0) {
			if (col_u == 0)
			    DADOij(U, 0, 0) = atof(buffer);
			else {
			    DADOij(temp1, 0, 0) = atof(buffer);
			    temp = hconcat(U, temp1);
			    U = mod_atribuir(U, temp);
			}
		    } else {
			if (col_u == 0)
			    DADOij(linha, 0, 0) = atof(buffer);
			else {
			    if (NCOLS(linha) == NCOLS(U)) {
				DADOij(linha, 0, col_u) = atof(buffer);

			    } else {
				DADOij(temp1, 0, 0) = atof(buffer);
				temp = hconcat(linha, temp1);
				linha = mod_atribuir(linha, temp);
			    }
			}
		    }

		    col_u++;
		}
		token = gtoken1(buffer, stdin);
	    }

	    /* Caso exista mais do que uma linha, deve-se 
	       concatenar a ultima linha */
	    if (lin_u > 0) {
		if (NCOLS(U) != NCOLS(linha))
		    mlmerro(6, 0);

		temp = vconcat(U, linha);
		U = mod_atribuir(U, temp);
	    }
	    lin_u++;

	    if (lin_u != ndep)
		mlmerro(7, 0);


	    /* Compatibilidade entre U e M */

	    if (flm != 0)
		if (col_m != col_u)
		    mlmerro(8, 0);
	    flu = 2;
	    libsim(linha);
	    libsim(temp1);
	}


	if (flm == 1) {
	    if (flu != 0)
		mlmerro(9, 0);
	    lin_m = 0;
	    col_m = 0;

	    M = mod_criamat(1, 1, 0.0);
	    TIPO(M) = ID;
	    linha = mod_criamat(1, 1, 0.0);
	    TIPO(linha) = ID;
	    temp1 = faztemp(1, 1);
	    TIPO(temp1) = ID;

	    token = gtoken1(buffer, stdin);
	    if (token != NUMBER)
		mlmerro(15, 0);

	    while (token != PTVG) {
		if (token != NUMBER && token != VG)
		    mlmerro(3, 0);

		if (token == VG) {

		    /* Concatenar mais uma linha */
		    if (lin_m > 0) {
			if (NCOLS(M) != NCOLS(linha))
			    mlmerro(6, 0);

			temp = vconcat(M, linha);
			M = mod_atribuir(M, temp);
		    }
		    col_m = 0;
		    lin_m++;
		} else {
		    if (col_m >= ndep)
			mlmerro(6, 0);

		    if (lin_m == 0) {
			if (col_m == 0)
			    DADOij(M, 0, 0) = atof(buffer);
			else {
			    DADOij(temp1, 0, 0) = atof(buffer);
			    temp = hconcat(M, temp1);
			    M = mod_atribuir(M, temp);
			}
		    } else {
			if (col_m == 0)
			    DADOij(linha, 0, 0) = atof(buffer);
			else {
			    if (NCOLS(linha) == NCOLS(M)) {
				DADOij(linha, 0, col_m) = atof(buffer);
			    } else {
				DADOij(temp1, 0, 0) = atof(buffer);
				temp = hconcat(linha, temp1);
				linha = mod_atribuir(linha, temp);
			    }
			}
		    }
		    col_m++;
		}
		token = gtoken1(buffer, stdin);
	    }
	    if (lin_m > 0) {
		if (NCOLS(M) != NCOLS(linha))
		    mlmerro(6, 0);

		temp = vconcat(M, linha);
		M = mod_atribuir(M, temp);
	    }
	    lin_m++;
	    flm = 2;
	    libsim(linha);
	    libsim(temp1);
	}
	primeiro = 0;
    }


    /* montagem de U e M se nao tiverem sido especificadas */
    if (flu == 0) {
	/* Criacao da matriz U automatica */
	U = mIdent(nd);
	TIPO(U) = ID;

	if (flm != 0)
	    if (NCOLS(M) != NCOLS(U))
		mlmerro(8, 0);
    }

    if (flm == 0) {
	/* Criacao da matriz M automatica */
	M = mod_criamat(NLINS(C), NCOLS(U), 0.0);
	TIPO(M) = ID;
    }

    /*
     ** Chamada a MLMXLX.HIP
     */

    if (rotulo[0] == '\0')
	mlmhip(-1, ierro, esp, ixx, nly, ncx, nd, beta, C, U, M, aux, 2);
    else
	mlmhip(-1, ierro, esp, ixx, nly, ncx, nd, beta, C, U, M, aux, 3);

    libsim(C);
    libsim(U);
    libsim(M);
    libtemp();

}

/*
** Dada uma coluna da matriz C especificada pelo usuario,
** retorna a coluna correspondente na matriz C completa
*/
int col_c_atual(esp, esp_acu, col_c)
Simbolo_t *esp;
int *esp_acu;
int col_c;
{
    int i, j;
    int ind_efe;
    int corr;

    ind_efe = -1;

    /*
     ** Definicao do intervalo onde se encontra col_c, ou seja,
     ** a qual efeito do CP esta coluna se refere
     */
    for (i = 0; i <= CP_GetNefe(cps); i++) {
	if ((col_c >= esp_acu[i]) && (col_c < esp_acu[i + 1])) {
	    ind_efe = CP_GetEfe(cps, i);
	    break;
	}
    }

    /* 
     ** Determina o numero de efeitos anteriores que devem ser 
     ** considerados 
     */

    corr = 0;
    for (j = 0; j < i; j++) {
	corr = corr + esp_acu[j + 1] - esp_acu[j];
    }

    if (ind_efe == -1) {
	mlmerro(26, 1);
	setOSstdin();
#ifdef MEM_DEBUG
	if (Mem_Used() != 0) {
	    printf("*** Lista de memoria nao esta' vazia ***\n");
	    Mem_Store("multvar");
	    exit(2);
	}
#endif
	exit(1);
    }

    /*
     ** Calculo da posicao final na matriz C completa
     */
    j = col_c - corr + (int) (DADOij(esp, 0, (2 + ind_efe)));

    return (j);
}

/*
** Parser do comando CP
*/
void mlm_CP(ef)
Simbolo_t *ef;
{
    int m, n, i, cpflag;
    int j, nao_existe;
    int c;
    char token[64];
    char buffer[256];
    cpflag = 0;


    CP_SetNefe(cps, 0);

    /* estou fazendo o parse do comando cp */

    if ((c = gtoken1(token, stdin)) != VARIAV && c != PTVG)
	mlmerro(41, 0);

    if (c != PTVG) {
	m = 0;			/* contador de niveis de iteracao */
	do {
	    buffer[0] = '\0';
	  l1:;
	    do {
		strcat(buffer, token);
		c = gtoken1(token, stdin);
		if (c == STAR) {
		    strcat(buffer, "*");
		    n = gtoken1(token, stdin);
		}
	    } while (c == STAR);

	    if (c == LP) {
		strcat(buffer, "(");
		while ((c = gtoken1(token, stdin)) != RP) {
		    strcat(buffer, token);
		    strcat(buffer, " ");
		}
		buffer[strlen(buffer) - 1] = '\0';
		strcat(buffer, ")");
		c = gtoken1(token, stdin);
	    }
	    if (c == STAR) {
		strcat(buffer, "*");
		c = gtoken1(token, stdin);
		goto l1;
	    }

	    for (i = 0; i < EFE_GetNefe(efeitos); ++i) {
		if (DADOij(ef, i, 3) == 0.0) {
		    /* Nao e' efeito gerado */
		    if (strcmpi(EFE_GetNome(efeitos, i), buffer) == 0) {
			if ((i > 0) && (DADOij(ef, i, 0) == -2.0))
			    mlmerro(11, 0);

			/* verifica se o efeito ja' foi
			 ** especificado no comando CP
			 */
			nao_existe = 1;

			for (j = 0; j < CP_GetNefe(cps); j++) {
			    if (i == CP_GetEfe(cps, j)) {
				nao_existe = 0;
				break;
			    }
			}

			if (nao_existe) {
			    cpflag++;
			    if (cpflag >= MAXEFEITOSCP)
				mlmerro(12, 0);

			    /* cps.ef[cps.nef++] = i; */
			    CP_SetEfe(cps, CP_GetNefe(cps), i);
			    CP_SetNefe(cps, 1 + CP_GetNefe(cps));
			}
			break;
		    }
		}
	    }
	    if (i >= EFE_GetNefe(efeitos)) {
		mlmerro(13, 0);
	    }
	} while (c == VARIAV);

	if (c == VG) {
	    c = gtoken1(token, stdin);
	    if (strcmpi(token, "liste") != 0)
		mlmerro(43, 0);

	    while (1) {
		c = gtoken1(token, stdin);

		if ((c == PTVG) || (c == VG))
		    break;

		if (strcmpi(token, "h") == 0)
		    imp_h = 1;
		else {
		    if (strcmpi(token, "e") == 0) {
			imp_e = 1;
			imp_er = 1;
		    } else {
			if (strcmpi(token, "v") == 0) {
			    imp_v = 1;
			} else
			    mlmerro(32, 0);
		    }
		}
	    }
	    if (c == VG)
		mlmerro(14, 0);
	} else {
	    if (c != PTVG) {
		mlmerro(19, 0);
	    }
	}
    } else {
	mlmerro(42, 0);
    }

    /*
     ** Verifica se os efeitos do CP foram especificados na mesma
     ** ordem do comando MOD
     */

    j = CP_GetEfe(cps, 0);
    for (i = 1; i < CP_GetNefe(cps); i++) {
	c = CP_GetEfe(cps, i);
	if (c < j)
	    mlmerro(16, 0);
	else
	    j = c;
    }
}
