/***********************************************************************************
 MULTVAR - Módulo para análise de modelos lineares multivariados

 Direitos Autorais Reservados (c) 2004 Embrapa Informática Agropecuária

 Este programa é software livre; você pode redistribuí-lo e/ou modificá-lo sob os
 termos da Licença Pública Geral GNU conforme publicada pela Free Software
 Foundation; tanto a versão 2 da Licença, como (a seu critério) qualquer versão
 posterior.

 Este programa é distribuído na expectativa de que seja útil, porém, SEM NENHUMA
 GARANTIA; nem mesmo a garantia implícita de COMERCIABILIDADE OU ADEQUAÇÃO A UMA
 FINALIDADE ESPECÍFICA. Consulte a Licença Pública Geral do GNU para mais detalhes.

 Você deve ter recebido uma cópia da Licença Pública Geral do GNU junto com este
 programa; se não, escreva para a Free Software Foundation, Inc., no endereço 59
 Temple Street, Suite 330, Boston, MA 02111-1307 USA.

 **********************************************************************************

 Parte integrante do SOC - SOftware Científico

 Descrição:
 
	Parser do comando TESTEM, fazendo geracao automatica das matrizes C,U e M
	para os efeitos especificados pelo usuario.

	Sintaxe: TESTEM <lista_de_efeitos ou TODOS> [, U = ( matriz_u)]
			[, LISTE = H E V] [,univar];

 Desenvolvido por:

	Maurício Noronha Festa
	Sônia Ternes 
	Sílvio Roberto Medeiros Evangelista

 Informações do CVS:
        $Source: /cvsroot/soc/soc/src/multvar/fontes/mlmger.c,v $:
        $Revision: 1.1 $:
        $Date: 2005/03/24 16:08:44 $:

***********************************************************************************/
static char *sccsid = "@(#)mlmger.c	3.3	(NTIA/modlin)	95/04/18";


#include "mod_incl.h"

#include <stdlib.h>
#include <setjmp.h>
#include <string.h>
#include <math.h>

#include "getoken.h"
#include "pcsgbd.h"
#include "multvar.h"


/*
** Estruturas usadas
*/
extern Efeitos efeitos;		/* Macros: EFE_GetNome, EFE_GetNefe, EFE_SetNome, 
				   EFE_SetNefe */
static EfUsu ef_usu;		/* Macros: EU_GetEfe, EU_GetNefe, EU_SetEfe, 
				   EU_SetNefe */

extern int imp_h;
extern int imp_e;
extern int imp_er;
extern int imp_v;
extern int fl_uni_t;

/* 
** Funcao principal para execucao do comando EFEITO
*/
void mlmger(ixx, esp, ef, nly, ncx, beta, cov_beta, dp_beta, aux)
Simbolo_t *ixx;
Simbolo_t *esp;
Simbolo_t *ef;
Simbolo_t *nly;
Simbolo_t *ncx;
Simbolo_t *beta;
Simbolo_t *cov_beta;
Simbolo_t *dp_beta;
Simbolo_t *aux;
{
    int fl_todos;
    int fl_u = 0;

    int i, ierro, quant;
    int dim;

    Simbolo_t *nd = (Simbolo_t *) NULL;
    Simbolo_t *u = (Simbolo_t *) NULL;
    Simbolo_t *m = (Simbolo_t *) NULL;
    Simbolo_t *c = (Simbolo_t *) NULL;

    /*
     ** mod_include1.c
     */
    Simbolo_t *ipos = (Simbolo_t *) NULL;
    Simbolo_t *jpos = (Simbolo_t *) NULL;
    Simbolo_t *mpos = (Simbolo_t *) NULL;
    Simbolo_t *npos = (Simbolo_t *) NULL;

    ipos = faztemp(1, 1);
    jpos = faztemp(1, 1);

    TIPO(ipos) = ID;
    TIPO(jpos) = ID;

    /*
     ** Inicio do parser : monta a estrutura ef_usu com o indice dos efeitos
     ** escolhidos pelo usuario para realizar a analise geral
     */

    /* nd = esp[1,1]; */
    DADOij(ipos, 0, 0) = 1;
    DADOij(jpos, 0, 0) = 1;
    nd = matsubs(esp, ipos, jpos);
    TIPO(nd) = ID;

    /* dim = ncol(ixx); */
    dim = NCOLS(ixx);

    /* chama parser */
    fl_todos = 0;
    mlm_GER(ef, esp, &fl_todos, &fl_u, (int) DADOij(nd, 0, 0), &u);

    if (fl_u == 0) {
	/* U = ident(nd); */
	u = mIdent(nd);
	TIPO(u) = ID;
    }

    /* para teste de mg : montagem de C, U e M */

    if (fl_todos == 1)
	i = 2;			/* retira mg da analise */
    else {
	if (escolhido(0) == 1)	/* verifica se mg foi especificado */
	    i = 1;
	else
	    i = 2;
    }

    /* while(_i <= (esp[1,2]+1)) */
    while (i <= (int) (DADOij(esp, 0, 1) + 1.0)) {

	if ((fl_todos == 1) || (escolhido(i - 1) == 1)) {

	    monta_CM(i, &ierro, &c, &m, u, esp, ef, dim, nd);

	    /*
	     ** Chamada a MLMXLX.HIP
	     */
	    mlmhip(i - 1, ierro, esp, ixx, nly, ncx, nd, beta, c, u, m,
		   aux, 1);

	    /* if (ef[_i,4] > 0) */
	    if (DADOij(ef, i - 1, 3) > 0.0) {

		/* quant_ac = ef[_i,4] - _i + 1; */
		quant = (int) (DADOij(ef, i - 1, 3)) - i + 1;
		i += (quant + 1);
	    } else {
		i++;
	    }
	} else {
	    i++;
	}
    }
    libsim(c);
    libsim(m);			/* Fim da analise via mlmhip() */

    if (fl_uni_t) {
	/*
	 ** Executa analise univariada
	 */
	exec_univar(0, u, esp, ef, dim, nd, ixx, nly, beta, aux);
	fl_uni_t = 0;
    }

    libsim(u);

}

/*
** Parser do comando EFEITO
*/
void mlm_GER(ef, esp, fl_todos, fl_u, ndep, U)
Simbolo_t *ef;
Simbolo_t *esp;
int *fl_todos;
int *fl_u;
int ndep;
Simbolo_t **U;
{
    int m, n, i, c, euflag;
    int j, nao_existe, fim;
    int lin_u, col_u;

    char token[64];
    char buffer[256];

    Simbolo_t *linha = (Simbolo_t *) NULL;
    Simbolo_t *temp1 = (Simbolo_t *) NULL;
    Simbolo_t *temp = (Simbolo_t *) NULL;

    euflag = 0;

    EU_SetNefe(ef_usu, 0);

    /* estou fazendo o parser do comando TESTEM */

    if ((c = gtoken1(token, stdin)) != VARIAV && c != PTVG)
	mlmerro(1, 0);

    if (c != PTVG) {
	if (strcmpi(token, "todos") == 0) {
	    if (DADOij(esp, 0, 1) < 1.0)
		mlmerro(35, 0);
	    *fl_todos = 1;

	    /* Preenche estrutura ef_usu */
	    fim = EFE_GetNefe(efeitos) - ndep;
	    for (i = 1; i < fim; i++) {
		EU_SetEfe(ef_usu, EU_GetNefe(ef_usu), i);
		EU_SetNefe(ef_usu, 1 + EU_GetNefe(ef_usu));
	    }

	    c = gtoken1(token, stdin);
	}
	if (*fl_todos == 0) {
	    m = 0;		/* contador de niveis de iteracao */
	    do {
		buffer[0] = '\0';
	      l1:;
		do {
		    strcat(buffer, token);
		    c = gtoken1(token, stdin);
		    if (c == STAR) {
			strcat(buffer, "*");
			n = gtoken1(token, stdin);
		    }
		} while (c == STAR);

		if (c == LP) {
		    strcat(buffer, "(");
		    while ((c = gtoken1(token, stdin)) != RP) {
			strcat(buffer, token);
			strcat(buffer, " ");
		    }
		    buffer[strlen(buffer) - 1] = '\0';
		    strcat(buffer, ")");
		    c = gtoken1(token, stdin);
		}
		if (c == STAR) {
		    strcat(buffer, "*");
		    c = gtoken1(token, stdin);
		    goto l1;
		}

		for (i = 0; i < EFE_GetNefe(efeitos); ++i) {
		    if (DADOij(ef, i, 3) == 0.0) {	/* Nao e' efeito gerado */
			if (strcmpi(EFE_GetNome(efeitos, i), buffer) == 0) {
			    if ((i > 0) && (DADOij(ef, i, 0) == -2.0))
				mlmerro(39, 0);

			    /* verifica se o efeito
			     ** ja' foi especificado 
			     ** no comando TESTEM
			     */
			    nao_existe = 1;

			    for (j = 0; j < EU_GetNefe(ef_usu); j++) {
				if (i == EU_GetEfe(ef_usu, j)) {
				    nao_existe = 0;
				    break;
				}
			    }

			    if (nao_existe) {
				euflag++;
				if (euflag >= MAXEFTS)
				    mlmerro(18, 0);

				EU_SetEfe(ef_usu, EU_GetNefe(ef_usu), i);
				EU_SetNefe(ef_usu, 1 + EU_GetNefe(ef_usu));
			    }
			    break;
			}
		    }
		}
		if (i >= EFE_GetNefe(efeitos)) {
		    mlmerro(13, 0);
		}
	    } while (c == VARIAV);
	}

	if (c == VG) {
	    c = gtoken1(token, stdin);
	    if (strcmpi(token, "u") == 0) {
		*fl_u = 1;
		lin_u = 0;
		col_u = 0;

		*U = mod_criamat(1, 1, 0.0);
		TIPO(*U) = ID;
		linha = mod_criamat(1, 1, 0.0);
		TIPO(linha) = ID;
		temp1 = faztemp(1, 1);
		TIPO(temp1) = ID;

		/* parser de u */
		c = gtoken1(token, stdin);
		if (c != IGUAL)
		    mlmerro(0, 0);

		c = gtoken1(token, stdin);
		if (c != LP)
		    mlmerro(37, 0);

		c = gtoken1(token, stdin);
		while (c != RP) {
		    if (c != NUMBER && c != VG)
			mlmerro(3, 0);

		    if (c == VG) {
			if (col_u > ndep)
			    mlmerro(36, 0);

			/* Concatenar mais uma linha */
			if (lin_u > 0) {
			    if (NCOLS(*U) != NCOLS(linha))
				mlmerro(6, 0);

			    temp = vconcat(*U, linha);
			    *U = mod_atribuir(*U, temp);
			}
			col_u = 0;
			lin_u++;
		    } else {
			if (lin_u >= ndep)
			    mlmerro(7, 0);
			if (lin_u == 0) {
			    if (col_u == 0)
				DADOij(*U, 0, 0) = atof(token);
			    else {
				DADOij(temp1, 0, 0) = atof(token);
				temp = hconcat(*U, temp1);
				*U = mod_atribuir(*U, temp);
			    }
			} else {
			    if (col_u == 0)
				DADOij(linha, 0, 0) = atof(token);
			    else {
				if (NCOLS(linha) == NCOLS(*U)) {
				    DADOij(linha, 0, col_u) = atof(token);

				} else {
				    DADOij(temp1, 0, 0) = atof(token);
				    temp = hconcat(linha, temp1);
				    linha = mod_atribuir(linha, temp);
				}
			    }
			}

			col_u++;
		    }
		    c = gtoken1(token, stdin);
		}

		/*
		 ** Caso exista mais do que uma linha, deve-se 
		 ** concatenar a ultima linha
		 */
		if (lin_u > 0) {
		    if (NCOLS(*U) != NCOLS(linha))
			mlmerro(6, 0);

		    temp = vconcat(*U, linha);
		    *U = mod_atribuir(*U, temp);
		}
		lin_u++;

		if (lin_u != ndep)
		    mlmerro(7, 0);

		libsim(linha);
		libsim(temp1);
		libsim(temp);

		c = gtoken1(token, stdin);
		if (c == VG) {
		    c = gtoken1(token, stdin);
		    if (strcmpi(token, "liste") == 0) {

			while (1) {
			    c = gtoken1(token, stdin);

			    if ((c == PTVG) || (c == VG))
				break;

			    if (strcmpi(token, "h") == 0)
				imp_h = 1;
			    else {
				if (strcmpi(token, "e") == 0) {
				    imp_e = 1;
				    imp_er = 1;
				} else {
				    if (strcmpi(token, "v") == 0) {
					imp_v = 1;
				    } else
					mlmerro(32, 0);
				}
			    }
			}
			if (c == VG) {
			    c = gtoken1(token, stdin);
			    if (strcmpi(token, "univar") == 0) {
				fl_uni_t = 1;
				c = gtoken1(token, stdin);
				if (c != PTVG)
				    mlmerro(14, 0);
			    } else
				mlmerro(40, 0);
			} else {
			    if (c != PTVG)
				mlmerro(19, 0);
			}
		    } else {
			if (strcmpi(token, "univar") == 0) {
			    fl_uni_t = 1;
			    c = gtoken1(token, stdin);
			    if (c != PTVG)
				mlmerro(14, 0);
			} else
			    mlmerro(44, 0);
		    }
		} else {
		    if (c != PTVG)
			mlmerro(19, 0);
		}
	    } else {
		if (strcmpi(token, "liste") == 0) {

		    while (1) {
			c = gtoken1(token, stdin);

			if ((c == PTVG) || (c == VG))
			    break;

			if (strcmpi(token, "h") == 0)
			    imp_h = 1;
			else {
			    if (strcmpi(token, "e") == 0) {
				imp_e = 1;
				imp_er = 1;
			    } else {
				if (strcmpi(token, "v") == 0) {
				    imp_v = 1;
				} else
				    mlmerro(32, 0);
			    }
			}
		    }
		    if (c == VG) {
			c = gtoken1(token, stdin);
			if (strcmpi(token, "univar") == 0) {
			    fl_uni_t = 1;
			    c = gtoken1(token, stdin);
			    if (c != PTVG)
				mlmerro(14, 0);
			} else
			    mlmerro(40, 0);
		    } else {
			if (c != PTVG)
			    mlmerro(19, 0);
		    }
		} else {
		    if (strcmpi(token, "univar") == 0) {
			fl_uni_t = 1;
			c = gtoken1(token, stdin);
			if (c != PTVG)
			    mlmerro(14, 0);
		    } else
			mlmerro(45, 0);
		}
	    }
	} else {
	    if (c != PTVG) {
		mlmerro(19, 0);
	    }
	}
    } else {
	mlmerro(23, 0);
    }

    /*
     ** Verifica se os efeitos do TESTEM foram especificados na mesma
     ** ordem do comando MOD
     */

    for (i = 0; i < EU_GetNefe(ef_usu) - 1; i++) {
	if (CP_GetEfe(ef_usu, i) > CP_GetEfe(ef_usu, i + 1))
	    mlmerro(16, 0);
    }
}

/* 
** Verifica se o efeito de indice ind foi definido no comando TESTEM
*/
int escolhido(ind)
int ind;
{
    int i;
    int esc;

    esc = 0;
    for (i = 0; i < EU_GetNefe(ef_usu); i++) {
	if (EU_GetEfe(ef_usu, i) == ind) {
	    esc = 1;
	    break;
	}
	if (EU_GetEfe(ef_usu, i) > ind)
	    break;
    }
    return (esc);
}

/*
** Monta automaticamente as matrizes C e M para o efeito i
*/
void monta_CM(i, ierro, c, m, u, esp, ef, dim, nd)
int i;				/* indice do efeito */
int *ierro;			/* indice do efeito de erro */
Simbolo_t **c;
Simbolo_t **m;
Simbolo_t *u;
Simbolo_t *esp;
Simbolo_t *ef;
int dim;
Simbolo_t *nd;
{
    int j, k, l;
    int tam, quant;

    /* if (ef[_i,4] > 0) */
    if (DADOij(ef, i - 1, 3) > 0.0) {

	/* quant_ac = ef[_i,4] - _i + 1; */
	quant = (int) (DADOij(ef, i - 1, 3)) - i + 1;

	/* tam_ac = esp[1,3+quant_ac+_i] - esp[1,2+_i]; */
	tam =
	    (int) (DADOij(esp, 0, 2 + quant + i) - DADOij(esp, 0, 1 + i));

	/* M = criamat(tam_ac,nd,0) */
	*m = mod_criamat(tam, NCOLS(u), 0.0);
	TIPO(*m) = ID;

	/* C = criamat(tam_ac,dim,0); */
	*c = mod_criamat(tam, dim, 0.0);
	TIPO(*c) = ID;

	/* _j = esp[1, 2+_i] */
	j = (int) (DADOij(esp, 0, 1 + i));
	k = j + 1;
	l = 1;

	/* while(_j < esp[1,3+quant_ac+_i]) */
	while (j < (int) (DADOij(esp, 0, 2 + quant + i))) {

	    /* C[_l, _k] = 1; */
	    DADOij(*c, l - 1, k - 1) = 1.0;

	    k++;
	    j++;
	    l++;
	}

	/*
	 ** Verifica teste de erro especifico
	 */
	if (DADOij(ef, i - 1, 2) != 0.0)
	    *ierro = (int) DADOij(ef, i - 1, 2) + 1;
	else
	    *ierro = -1;

    } else {
	/* C = criamat(esp[1,3+_i] - esp[1,2+_i],dim,0); */
	*c = mod_criamat((int)
			 (DADOij(esp, 0, 2 + i) - DADOij(esp, 0, 1 + i)),
			 dim, 0.0);
	TIPO(*c) = ID;

	/* M = criamat(esp[1,3+_i] - esp[1,2+_i],nd,0); */
	*m = mod_criamat((int)
			 (DADOij(esp, 0, 2 + i) - DADOij(esp, 0, 1 + i)),
			 NCOLS(u), 0.0);
	TIPO(*m) = ID;

	/* j = esp[1,2+ _i] */
	j = (int) (DADOij(esp, 0, 1 + i));

	k = j + 1;
	l = 1;

	/* while(_j < esp[1, 3+_i]) */
	while (j < (int) (DADOij(esp, 0, 2 + i))) {

	    /* C[_l, _k] = 1 */
	    DADOij(*c, l - 1, k - 1) = 1.0;

	    k++;
	    j++;
	    l++;
	}

	/*
	 ** Verifica teste de erro especifico
	 */
	if (DADOij(ef, i - 1, 2) != 0.0)
	    *ierro = (int) DADOij(ef, i - 1, 2) + 1;
	else
	    *ierro = -1;

    }
}
