/***********************************************************************************
 MULTVAR - Módulo para análise de modelos lineares multivariados

 Direitos Autorais Reservados (c) 2004 Embrapa Informática Agropecuária

 Este programa é software livre; você pode redistribuí-lo e/ou modificá-lo sob os
 termos da Licença Pública Geral GNU conforme publicada pela Free Software
 Foundation; tanto a versão 2 da Licença, como (a seu critério) qualquer versão
 posterior.

 Este programa é distribuído na expectativa de que seja útil, porém, SEM NENHUMA
 GARANTIA; nem mesmo a garantia implícita de COMERCIABILIDADE OU ADEQUAÇÃO A UMA
 FINALIDADE ESPECÍFICA. Consulte a Licença Pública Geral do GNU para mais detalhes.

 Você deve ter recebido uma cópia da Licença Pública Geral do GNU junto com este
 programa; se não, escreva para a Free Software Foundation, Inc., no endereço 59
 Temple Street, Suite 330, Boston, MA 02111-1307 USA.

 **********************************************************************************

 Parte integrante do SOC - SOftware Científico

 Descrição:

	Parser e execucao do comando MEDREP do MULTVAR

	Sintaxe: MEDREP <string_para_output>, <tipo_da_analise> [, LISTE H E V]
	  		[, univar];
	        tipo_da_analise = PERFIL ou MEDIA=n ou HELMERT ou CONTRASTE=n ou
		POL=n1 n2 ... ndep

 Desenvolvido por:

	Maurício Noronha Festa
	Sônia Ternes 
	Sílvio Roberto Medeiros Evangelista

 Informações do CVS:
        $Source: /cvsroot/soc/soc/src/multvar/fontes/mlmmed.c,v $:
        $Revision: 1.1 $:
        $Date: 2005/03/24 16:08:44 $:

***********************************************************************************/
static char *sccsid = "@(#)mlmmed.c	3.5	(NTIA/modlin)	95/07/26";


#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "mod_incl.h"
#include "multvar.h"
#include "getoken.h"
#include "pcsgbd.h"

/*
** Estruturas usadas
*/
extern Efeitos efeitos;		/* Macros: EFE_GetNome, EFE_GetNefe, EFE_SetNome, 
				   EFE_SetNefe */

/*
** Variaveis Globais
*/
extern int fl_uni;
extern int fl_analise;
extern int imp_h;
extern int imp_e;
extern int imp_er;
extern int imp_v;

int ctr = -1;			/* indice da variavel de contraste ou media */

extern char string[9];

Simbolo_t *vpol = (Simbolo_t *) NULL;

/*
** Estrutura para armazenar valores para calculo dos testes F ajustados
** de acordo com Greenhouse-Geisser e Huynh-Feldt
*/

struct _F_Ajustado {
    double gg;			/* epsilon de Greenhouse-Geisser */
    double hf;			/* epsilon de huynh-Feldt */
    int nefe;			/* numero de efeitos */
    int ind[MAXEFTS];		/* indice dos efeitos */
    double vf[MAXEFTS];		/* Valor F para cada efeito */
    double p_gg[MAXEFTS];	/* prob para cada efeito segundo GG */
    double p_hf[MAXEFTS];	/* prob para cada efeito segundo HF */
};

typedef struct _F_Ajustado _TFA;	/* definicao do tipo */

_TFA tfa;			/* variavel do tipo da estrutura */

/*
** Funcao principal para execucao do comando MEDREP
*/
void mlmmed(ixx, esp, ef, nly, ncx, beta, aux)
Simbolo_t *ixx;
Simbolo_t *esp;
Simbolo_t *ef;
Simbolo_t *nly;
Simbolo_t *ncx;
Simbolo_t *beta;
Simbolo_t *aux;
{
    int i, j, k;
    double x;

    Simbolo_t *u = (Simbolo_t *) NULL;
    Simbolo_t *mb = (Simbolo_t *) NULL;
    Simbolo_t *mc = (Simbolo_t *) NULL;
    Simbolo_t *d = (Simbolo_t *) NULL;
    Simbolo_t *ti = (Simbolo_t *) NULL;
    Simbolo_t *nd = (Simbolo_t *) NULL;
    Simbolo_t *ind = (Simbolo_t *) NULL;
    Simbolo_t *mpos = (Simbolo_t *) NULL;
    Simbolo_t *ipos = (Simbolo_t *) NULL;
    Simbolo_t *jpos = (Simbolo_t *) NULL;

    ipos = faztemp(1, 1);
    jpos = faztemp(1, 1);

    TIPO(ipos) = ID;
    TIPO(jpos) = ID;

    /* nd = esp[1,1]; */
    DADOij(ipos, 0, 0) = 1;
    DADOij(jpos, 0, 0) = 1;
    nd = matsubs(esp, ipos, jpos);
    TIPO(nd) = ID;


    /*
     ** Parser do comando MEDREP
     */
    fl_analise = 0;
    mlm_MED((int) DADOij(nd, 0, 0));

    switch (fl_analise) {
    case 1:
	/* 
	 ** Montagem da matriz U para analise de PERFIL 
	 */
	u = mod_criamat((int) DADOij(nd, 0, 0), (int) DADOij(nd, 0, 0) - 1,
			0.0);
	TIPO(u) = ID;

	for (j = 0; j < NCOLS(u); j++) {
	    DADOij(u, j, j) = 1.0;
	    DADOij(u, j + 1, j) = -1.0;
	}

	analise(u, ixx, esp, ef, nly, ncx, beta, aux, nd,
		(Simbolo_t *) NULL);
	libsim(u);
	break;
    case 2:
	ctr--;			/* primeira variavel tem indice 0 */
	if (ctr >= (int) DADOij(nd, 0, 0))
	    mlmerro(25, 0);
	/* 
	 ** Montagem da matriz U para analise de MEDIA 
	 */
	x = -1.0 / (DADOij(nd, 0, 0) - 1.0);
	u = mod_criamat((int) DADOij(nd, 0, 0), (int) DADOij(nd, 0, 0), x);
	TIPO(u) = ID;

	/* Inicializa U com valor 1.0 na diagonal e x fora dela */
	for (i = 0; i < NLINS(u); i++)
	    for (j = 0; j < NCOLS(u); j++)
		if (i == j)
		    DADOij(u, i, j) = 1.0;

	/* Monta vetor de indices para submatriz de U */
	ind = mod_criamat(1, (int) DADOij(nd, 0, 0) - 1, 0.0);
	TIPO(ind) = ID;

	k = 0;
	for (i = 0; i < (int) DADOij(nd, 0, 0); i++) {
	    if (ctr != i)
		DADOij(ind, 0, k++) = i + 1;
	}

	/* retira de U a coluna referente a ctr */
	u = mod_atribuir(u, matsubs(u, (Simbolo_t *) 0, ind));
	TIPO(u) = ID;
	libsim(ind);
	libtemp();

	analise(u, ixx, esp, ef, nly, ncx, beta, aux, nd,
		(Simbolo_t *) NULL);
	libsim(u);
	break;
    case 3:
	/* 
	 ** Montagem da matriz U para analise de HELMERT 
	 */
	u = mod_criamat((int) DADOij(nd, 0, 0), (int) DADOij(nd, 0, 0) - 1,
			0.0);
	TIPO(u) = ID;
	for (i = 0; i < NLINS(u); i++) {
	    for (j = 0; j < NCOLS(u); j++) {
		if (i == j)
		    DADOij(u, j, j) = 1.0;
		else if (i > j) {
		    x = -1.0 / (double) (NCOLS(u) - j);
		    DADOij(u, i, j) = x;
		} else
		    DADOij(u, i, j) = 0.0;
	    }
	}

	analise(u, ixx, esp, ef, nly, ncx, beta, aux, nd,
		(Simbolo_t *) NULL);
	libsim(u);
	break;
    case 4:
	ctr--;			/* primeira variavel tem indice 0 */
	if (ctr >= (int) DADOij(nd, 0, 0))
	    mlmerro(25, 0);

	/* 
	 ** Montagem da matriz U para analise de CONTRASTE 
	 */
	u = mod_criamat((int) DADOij(nd, 0, 0), (int) DADOij(nd, 0, 0) - 1,
			0.0);
	TIPO(u) = ID;
	for (i = 0; i < NLINS(u); i++) {
	    if (i == ctr) {
		for (j = 0; j < NCOLS(u); j++) {
		    DADOij(u, i, j) = -1.0;
		}
	    } else {
		if (i < ctr) {
		    DADOij(u, i, i) = 1.0;
		} else {
		    DADOij(u, i, i - 1) = 1.0;
		}
	    }
	}

	analise(u, ixx, esp, ef, nly, ncx, beta, aux, nd,
		(Simbolo_t *) NULL);
	libsim(u);
	break;
    case 5:
	/* 
	 ** Montagem da matriz U para analise POLINOMIAL 
	 */
	d = mod_criamat((int) DADOij(nd, 0, 0), 1, 0.0);
	TIPO(d) = ID;
	ti = mod_criamat((int) DADOij(nd, 0, 0), 1, 0.0);
	TIPO(ti) = ID;

	TIPO(vpol) = ID;
	for (i = 0; i < (int) DADOij(nd, 0, 0); i++) {
	    DADOij(ipos, 0, 0) = i;
	    /* t[,i] = vpol#i */
	    ti = mod_atribuir(ti, mattran(matepot(vpol, ipos)));
	    if (i == 0)
		d = mod_atribuir(d, ti);
	    else
		d = mod_atribuir(d, hconcat(d, ti));
	}

	/* mb,mc = gs(d); */
	MOD_GS(d, mb, mc);
	TIPO(d) = ID;
	TIPO(mb) = ID;
	TIPO(mc) = ID;

	/* u = mc[,2:nd] */
	mpos = mod_gindex(2, NCOLS(mc));
	u = matsubs(mc, (Simbolo_t *) 0, mpos);
	TIPO(u) = ID;
	libtemp();

	libsim(ti);
	libsim(d);
	libsim(mb);
	libsim(mc);

	analise(u, ixx, esp, ef, nly, ncx, beta, aux, nd, vpol);
	libsim(u);
	libsim(vpol);
	break;
    }
}

/*
** Parser do comando MEDREP
*/
void mlm_MED(nd)
int nd;
{
    char token[64];
    int c, i;

    c = gtoken1(token, stdin);
    if (c != VARIAV)
	mlmerro(20, 0);

    if (strlen(token) < 8) {
	strcpy(string, token);
    } else {
	sprintf(string, "%8s", token);
    }

    c = gtoken1(token, stdin);
    if (c != VG)
	mlmerro(21, 0);

    c = gtoken1(token, stdin);
    if (strcmpi(token, "perfil") == 0) {
	fl_analise = 1;
	c = gtoken1(token, stdin);
    } else {
	if (strcmpi(token, "media") == 0) {
	    fl_analise = 2;
	    c = gtoken1(token, stdin);
	    if (c != IGUAL)
		mlmerro(0, 0);
	    c = gtoken1(token, stdin);
	    if (c != NUMBER)
		mlmerro(24, 0);
	    ctr = atoi(token);
	    c = gtoken1(token, stdin);
	} else {
	    if (strcmpi(token, "helmert") == 0) {
		fl_analise = 3;
		c = gtoken1(token, stdin);
	    } else {
		if (strcmpi(token, "contraste") == 0) {
		    fl_analise = 4;
		    c = gtoken1(token, stdin);
		    if (c != IGUAL)
			mlmerro(0, 0);
		    c = gtoken1(token, stdin);
		    if (c != NUMBER)
			mlmerro(24, 0);
		    ctr = atoi(token);
		    c = gtoken1(token, stdin);
		} else {
		    if (strcmpi(token, "pol") == 0) {
			fl_analise = 5;
			vpol = mod_criamat(1, nd, 0.0);

			c = gtoken1(token, stdin);
			if (c != IGUAL) {
			    if ((c != VG) && (c != PTVG)) {
				mlmerro(34, 0);
			    }
			    for (i = 0; i < nd; i++)
				DADOij(vpol, 0, i) = (double) (i + 1);
			} else {
			    i = 0;
			    while (1) {
				c = gtoken1(token, stdin);
				if ((c == PTVG) || (c == VG))
				    break;

				if (c != NUMBER)
				    mlmerro(24, 0);

				if (i >= nd)
				    mlmerro(31, 0);

				DADOij(vpol, 0, i) = atof(token);
				i++;
			    }
			    if (i != nd)
				mlmerro(31, 0);
			}
		    } else
			mlmerro(22, 0);
		}
	    }
	}
    }

    if (c == VG) {
	c = gtoken1(token, stdin);
	if (strcmpi(token, "liste") == 0) {

	    while (1) {
		c = gtoken1(token, stdin);

		if ((c == PTVG) || (c == VG))
		    break;

		if (strcmpi(token, "h") == 0)
		    imp_h = 1;
		else {
		    if (strcmpi(token, "e") == 0) {
			imp_e = 1;
			imp_er = 1;
		    } else {
			if (strcmpi(token, "v") == 0) {
			    imp_v = 1;
			} else {
			    mlmerro(32, 0);
			}
		    }
		}
	    }
	    if (c == VG) {
		c = gtoken1(token, stdin);
		if (strcmpi(token, "univar") == 0) {
		    fl_uni = 1;
		    c = gtoken1(token, stdin);
		    if (c != PTVG)
			mlmerro(14, 0);
		} else
		    mlmerro(40, 0);
	    }
	} else {
	    if (strcmpi(token, "univar") == 0) {
		fl_uni = 1;
		c = gtoken1(token, stdin);
		if (c != PTVG)
		    mlmerro(14, 0);
	    } else
		mlmerro(44, 0);
	}
    } else {
	if (c != PTVG) {
	    mlmerro(19, 0);
	}
    }
}

/*
** Calcula as estatisticas para o tipo de analise escolhida pelo usuario
*/
void analise(u, ixx, esp, ef, nly, ncx, beta, aux, nd, vpol)
Simbolo_t *u;
Simbolo_t *ixx;
Simbolo_t *esp;
Simbolo_t *ef;
Simbolo_t *nly;
Simbolo_t *ncx;
Simbolo_t *beta;
Simbolo_t *aux;
Simbolo_t *nd;
Simbolo_t *vpol;
{
    void gg_hf();
    void imp_gg_hf();

    int i;
    int dim, ierro, quant;
    int gl_ef, gl_er;

    double SQer, SQef, A, B;

    Simbolo_t *m = (Simbolo_t *) NULL;
    Simbolo_t *c = (Simbolo_t *) NULL;
    Simbolo_t *h = (Simbolo_t *) NULL;
    Simbolo_t *e = (Simbolo_t *) NULL;
    Simbolo_t *e1 = (Simbolo_t *) NULL;
    Simbolo_t *uk = (Simbolo_t *) NULL;
    Simbolo_t *u1 = (Simbolo_t *) NULL;
    Simbolo_t *ma = (Simbolo_t *) NULL;
    Simbolo_t *mb = (Simbolo_t *) NULL;
    Simbolo_t *ma1 = (Simbolo_t *) NULL;
    Simbolo_t *mee = (Simbolo_t *) NULL;
    Simbolo_t *mhe = (Simbolo_t *) NULL;
    Simbolo_t *Cer = (Simbolo_t *) NULL;
    Simbolo_t *maux = (Simbolo_t *) NULL;
    Simbolo_t *mepi = (Simbolo_t *) NULL;
    Simbolo_t *mgg1 = (Simbolo_t *) NULL;
    Simbolo_t *mgg2 = (Simbolo_t *) NULL;
    Simbolo_t *ipos = (Simbolo_t *) NULL;
    Simbolo_t *raiz = (Simbolo_t *) NULL;

    dim = NCOLS(ixx);

    /*
     ** Impressao dos titulos iniciais
     */
    putchar2(1);
    quadro((int) DADOij(nd, 0, 0), vpol);

    /*
     ** Imprime matriz de correlacoes com e = residual
     */
    if (imp_e) {
	u1 = mIdent(nd);
	TIPO(u1) = ID;
	libtemp;

	/* E = u'*aux*u; */
	e1 = matmult(matmult(mattran(u1), aux), u1);
	TIPO(e1) = ID;
	libtemp();

	mlmcor(u1, e1, ixx, esp, nly, nd, 0, 1);

	libsim(e1);
	libsim(u1);
    }

    /* 
     ** Efeito de Resposta: C,M automaticos para mg e U igual parametro (u)
     */

    monta_CM(1, &ierro, &c, &m, u, esp, ef, dim, nd);
    mlmhip(0, ierro, esp, ixx, nly, ncx, nd, beta, c, u, m, aux, 5);
    libsim(c);
    libsim(m);

    /*
     ** Paralelismo: C,M automaticos para cada efeito e U igual parametro (u)
     */

    i = 2;
    while (i <= (int) (DADOij(esp, 0, 1) + 1.0)) {
	monta_CM(i, &ierro, &c, &m, u, esp, ef, dim, nd);

	mlmhip(i - 1, ierro, esp, ixx, nly, ncx, nd, beta, c, u, m, aux,
	       4);

	if (DADOij(ef, i - 1, 3) > 0.0) {
	    quant = (int) (DADOij(ef, i - 1, 3)) - i + 1;
	    i += (quant + 1);
	} else {
	    i++;
	}

	libsim(c);
	libsim(m);
    }

    /*
     ** Efeito de Grupo: C,M automaticos para cada efeito e U especifico
     */

    if (DADOij(esp, 0, 1) > 0.0) {
	/* Nao executa se modelo tiver apenas mg */

	raiz = mSqrt(nd);
	TIPO(raiz) = ID;

	u1 = mod_criamat((int) DADOij(nd, 0, 0), 1,
			 1.0 / DADOij(raiz, 0, 0));
	TIPO(u1) = ID;

	libsim(raiz);
	libtemp();

	putchar2(0);

	imptitulo("Analise de Variancia Entre Individuos", 1);
	i = 2;
	while (i <= (int) (DADOij(esp, 0, 1) + 1.0)) {
	    monta_CM(i, &ierro, &c, &m, u1, esp, ef, dim, nd);

	    if (ierro != -1) {
		Cer = mod_criamat((int) (DADOij(esp, 0, 2 + ierro) -
					 DADOij(esp, 0, 1 + ierro)), dim,
				  0.0);
		gl_er = (int) NLINS(Cer);
		libtemp();
	    } else {
		maux = mposto(ixx);
		gl_er =
		    (int) (DADOij(nly, 0, 0)) - (int) (DADOij(maux, 0, 0));
	    }

	    /*  Calcula  matrizes H e E */
	    calc_h_e(ierro, esp, ixx, beta, c, u1, m, aux, &h, &e);

	    /*  Soma de quadrados e grau de liberdade do efeito */
	    SQef = DADOij(h, 0, 0);
	    gl_ef = (int) NLINS(c);

	    SQer = DADOij(e, 0, 0);

	    /* imprime tabela ANOVA */
	    if (i == 2) {
		printf("\n");
		printf
		    ("Fonte de variacao       gl  Soma de quadrados  Quadrado medio   Valor F  PR > F\n");
		printf("                            Parcial\n\n");
		print_anova(1, EFE_GetNome(efeitos, i - 1), SQef, SQer,
			    gl_ef, gl_er);

	    } else {
		print_anova(1, EFE_GetNome(efeitos, i - 1), SQef, SQer,
			    gl_ef, gl_er);
	    }

	    /*  atualiza indice i  */
	    if (DADOij(ef, i - 1, 3) > 0.0) {
		quant = (int) (DADOij(ef, i - 1, 3)) - i + 1;
		i += (quant + 1);
	    } else {
		i++;
	    }
	}

	libtemp();
	libsim(e);
	libsim(h);

	/*
	 ** SQer para o erro padrao residual: E = u'*aux*u 
	 */

	e = matmult(matmult(mattran(u1), aux), u1);
	SQer = DADOij(e, 0, 0);
	TIPO(e) = ID;
	libtemp();

	maux = mposto(ixx);
	gl_er = (int) (DADOij(nly, 0, 0)) - (int) (DADOij(maux, 0, 0));

	print_erro("Residuo", SQer, gl_er);

	libtemp();
	libsim(e);
	libsim(u1);
	libsim(c);
	libsim(m);

    }


    /* Fim da analise de Grupo */
    /*
     ** Efeito  de  Split-Plot: C,M automaticos para cada efeito e
     ** U igual ao parametro (u)
     */
    putchar2(0);

    /*  ma,mb = gs(u) */
    MOD_GS(u, ma, mb);

    /* ma1 = inv(ma) */
    ma1 = mInv(ma);
    TIPO(ma1) = ID;
    libtemp();

    imptitulo("Analise de Variancia Dentro de Individuos", 1);
    i = 1;
    while (i <= (int) (DADOij(esp, 0, 1) + 1.0)) {
	monta_CM(i, &ierro, &c, &m, u, esp, ef, dim, nd);

	if (ierro != -1) {
	    Cer = mod_criamat((int) (DADOij(esp, 0, 2 + ierro) -
				     DADOij(esp, 0, 1 + ierro)), dim, 0.0);
	    gl_er = (int) NLINS(Cer);
	    libtemp();

	} else {
	    maux = mposto(ixx);
	    gl_er = ((int) NLINS(u) - 1) *
		((int) (DADOij(nly, 0, 0)) - (int) (DADOij(maux, 0, 0)));
	}

	/*  Calcula  matrizes H e E */
	calc_h_e(ierro, esp, ixx, beta, c, u, m, aux, &h, &e);

	/*  Soma de quadrados e grau de  liberdade do efeito */

	/* mhe = ma1'*h*ma1  */
	mhe = matmult(matmult(mattran(ma1), h), ma1);
	TIPO(mhe) = ID;
	libtemp();

	maux = mTraco(mhe);
	SQef = DADOij(maux, 0, 0);

	gl_ef = ((int) NLINS(u) - 1) * (int) NLINS(c);

	/*  Soma de quadrados e grau de  liberdade do erro */

	/* mee = ma1'*e*ma1  */
	mee = matmult(matmult(mattran(ma1), e), ma1);
	TIPO(mee) = ID;
	libtemp();

	maux = mTraco(mee);
	SQer = DADOij(maux, 0, 0);

	/*  gl_er = gl_ef   usado como teste de  erro */

	/* Calcula Greenhouse-Geisser e Huynh-Feldt */
	if ((int) DADOij(nd, 0, 0) > 2) {
	    if (i == 1) {
		tfa.nefe = 0;

		/* mepi = e*inv(u'u) */
		mepi = matmult(e, mInv(matmult(mattran(u), u)));
		TIPO(mepi) = ID;
		libtemp();

		/* mgg1 = (traco(mepi))^2 */
		mgg1 = matmult(mTraco(mepi), mTraco(mepi));
		TIPO(mgg1) = ID;
		libtemp();

		/* mgg2 = traco(m_epi^2) */
		mgg2 = mTraco(matmult(mepi, mepi));
		TIPO(mgg2) = ID;
		libtemp();

		/* gg = mgg1 / (ncol(u)*mgg2) */
		tfa.gg = DADOij(mgg1, 0, 0) /
		    (NCOLS(u) * DADOij(mgg2, 0, 0));

		/* A = (n*(p-1)*gg)-2 */
		/* B = (p-1)*((n-g)-(p-1)*gg) */
		/* hf = A/B */
		/* n: n. de obs. = NLINS(y) = nly */
		/* p: n. de var. dep. = NCOLS(y) = nd */
		/* g: = NCOLS(x) = NLINS(beta) */

		A = (DADOij(nly, 0, 0) * (DADOij(nd, 0, 0) - 1.0) *
		     tfa.gg) - 2.0;

		B = (DADOij(nd, 0, 0) - 1.0) * ((DADOij(nly, 0, 0) -
						 NLINS(beta)) - (DADOij(nd,
									0,
									0)
								 -
								 1.0) *
						tfa.gg);

		if (fabs(B) < 0.0000001)
		    tfa.hf = 1.0;
		else
		    tfa.hf = A / B;

		if (tfa.hf > 0.9999999)
		    tfa.hf = 1.0;

		libsim(mepi);
		libsim(mgg1);
		libsim(mgg2);
	    }
	    gg_hf(i, SQef, SQer, gl_ef, gl_er);
	}


	/* imprime tabela ANOVA */
	if (i == 1) {
	    printf("\n");
	    printf
		("Fonte de variacao       gl  Soma de quadrados  Quadrado medio   Valor F  PR > F\n");
	    printf("                            Parcial\n\n");

	    print_anova(2, "mg", SQef, SQer, gl_ef, gl_er);
	} else {
	    print_anova(2, EFE_GetNome(efeitos, i - 1), SQef, SQer, gl_ef,
			gl_er);
	}

	/*  atualiza indice i  */
	if (DADOij(ef, i - 1, 3) > 0.0) {
	    quant = (int) (DADOij(ef, i - 1, 3)) - i + 1;
	    i += (quant + 1);
	} else {
	    i++;
	}

	libsim(c);
	libsim(m);		/* Fim da analise de Split-Plot */
    }

    libtemp();
    libsim(ma);
    libsim(mb);
    libsim(mee);
    libsim(h);
    libsim(e);

    /* 
     ** SQer para o erro  padrao residual: E=u'*aux*u 
     */
    e = matmult(matmult(mattran(u), aux), u);
    TIPO(e) = ID;
    libtemp();

    mee = matmult(matmult(mattran(ma1), e), ma1);
    TIPO(mee) = ID;
    libtemp();

    maux = mTraco(mee);
    SQer = DADOij(maux, 0, 0);

    libtemp();

    maux = mposto(ixx);
    gl_er = ((int) NLINS(u) - 1) *
	((int) (DADOij(nly, 0, 0)) - (int) DADOij(maux, 0, 0));

    print_erro("Residuo", SQer, gl_er);

    libtemp();
    libsim(mee);
    libsim(e);
    libsim(ma1);		/* Fim da analise de Split-Plot */

    if ((int) DADOij(nd, 0, 0) > 2)	/* Imprime F Ajustados */
	imp_gg_hf();

    if (fl_uni) {

	/*
	 ** Executa analise univariada
	 */
	exec_univar(1, u, esp, ef, dim, nd, ixx, nly, beta, aux);

	fl_uni = 0;
    }
}

/*
** Preenche estrutura tfa
*/
void gg_hf(i, SQef, SQer, gl_ef, gl_er)
int i;
double SQef;
double SQer;
int gl_ef;
int gl_er;
{
    double QMef, QMer, ValorF;

    if (gl_er > 1.0e-14)
	QMer = SQer / (double) gl_er;
    else
	QMer = 9999.9999;

    if (gl_ef > 1.0e-14)
	QMef = SQef / (double) gl_ef;
    else
	QMef = 9999.9999;

    if (QMer > 1.0e-14)
	ValorF = QMef / QMer;
    else
	ValorF = 9999.99;

    if (ValorF > 9999.0) {
	tfa.vf[tfa.nefe] = 9999.9999;
	tfa.p_gg[tfa.nefe] = 0.0;
	tfa.p_hf[tfa.nefe] = 0.0;
    } else {
	tfa.vf[tfa.nefe] = ValorF;
	tfa.p_gg[tfa.nefe] = fprob(ValorF, (double) (gl_ef) * tfa.gg,
				   (double) (gl_er) * tfa.gg);
	tfa.p_hf[tfa.nefe] = fprob(ValorF, (double) (gl_ef) * tfa.hf,
				   (double) (gl_er) * tfa.hf);
    }

    tfa.ind[tfa.nefe++] = i;
}

/*
** Imprime F ajustados armazenados na estrutura tfa
*/
void imp_gg_hf()
{
    int k;
    char buf[20];

    imptitulo("Testes F Ajustados", 2);

    printf(". Epsilon de Greenhouse-Geisser = %12.7lf\n", tfa.gg);
    printf(". Epsilon de Huynh-Feldt        = %12.7lf\n", tfa.hf);
    printf("\n");

    printf
	("Fonte de Variacao       Valor F          Pr > F          Pr > F\n");
    printf
	("                                     Ajuste G-G      Ajuste H-F\n");
    printf("\n");

    for (k = 0; k < tfa.nefe; k++) {

	if (k == 0)
	    strcpy(buf, string);
	else {
	    if (strlen(EFE_GetNome(efeitos, tfa.ind[k] - 1)) < 10) {
		strcpy(buf, EFE_GetNome(efeitos, tfa.ind[k] - 1));
		strcat(buf, "*");
		strcat(buf, string);
	    } else
		sprintf(buf, "%s*%10s",
			EFE_GetNome(efeitos, tfa.ind[k] - 1), string);
	}

	/*
	   printf("%-20s %10.4lf %15.5lf %15.5lf\n",buf,tfa.vf[k],
	   tfa.p_gg[k], tfa.p_hf[k]);
	 */
	printf("%-20s %10s %15.5lf %15.5lf\n", buf,
	       rounddec(tfa.vf[k], 10, 4), tfa.p_gg[k], tfa.p_hf[k]);
    }
}
