/***********************************************************************************
 MULTVAR - Módulo para análise de modelos lineares multivariados

 Direitos Autorais Reservados (c) 2004 Embrapa Informática Agropecuária

 Este programa é software livre; você pode redistribuí-lo e/ou modificá-lo sob os
 termos da Licença Pública Geral GNU conforme publicada pela Free Software
 Foundation; tanto a versão 2 da Licença, como (a seu critério) qualquer versão
 posterior.

 Este programa é distribuído na expectativa de que seja útil, porém, SEM NENHUMA
 GARANTIA; nem mesmo a garantia implícita de COMERCIABILIDADE OU ADEQUAÇÃO A UMA
 FINALIDADE ESPECÍFICA. Consulte a Licença Pública Geral do GNU para mais detalhes.

 Você deve ter recebido uma cópia da Licença Pública Geral do GNU junto com este
 programa; se não, escreva para a Free Software Foundation, Inc., no endereço 59
 Temple Street, Suite 330, Boston, MA 02111-1307 USA.

 **********************************************************************************

 Parte integrante do SOC - SOftware Científico

 Descrição:
	
	Usado como interface de um program de traducao de CM para C
	contem funcoes que devem ser usada no lugar de outras funcoes
	do codigo do modulo CM.

 Desenvolvido por:

	Maurício Noronha Festa
	Sônia Ternes 
	Sílvio Roberto Medeiros Evangelista

 Informações do CVS:
        $Source: /cvsroot/soc/soc/src/multvar/fontes/modulo.c,v $:
        $Revision: 1.1 $:
        $Date: 2005/03/24 16:08:44 $:

***********************************************************************************/
static char *sccsid = "@(#)modulo.c	3.3	(NTIA/modlin)	95/04/27";

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "cm_cm01.h"
#include "pcsgbd.h"

#ifdef MSDOS
# include "y_tab.h"
#else
#ifdef VAXVMS
#include "ytab.h"
#else
# include "y.tab.h"
#endif
#endif

#include "modulo.h"

/*
** Variaveis globais auxiliares para retorno de funcoes
*/
Simbolo_t *md1;
Simbolo_t *md2;
Simbolo_t *md3;

/*
** Variaveis globais usadas na interface da traducao
*/

extern Simbolo_t *tabsimb;	/* tabela de simbolos */
extern Simbolo_t falso_no;

extern int fl_mis;		/* indica se e' para ler valores "missing" ou nao */
extern int _colunas;		/*Numero de cols. do terminal ou formulario de saida */

extern FILE *fsaida;		/*Determina o local (video,impressora,etc) do "output */

extern char _diretorio[];	/* Diretorio para leitura/gravacao de matrizes */

/* -------- Apenas para compatibilidade ---------------------*/

extern int parametro;		/* esta variavel nao serve para nada */
			/* ela foi colocada para garantir compatibilidade */
			/* com a lib do cm */
extern int debugflag;		/* Idem anterior */
extern int numerro;		/* idem anterior */

/* ----------------------------------------------------------*/

/* 
** mod_init() : funcao basica que deve ser chamada antes de qualquer 
**		comando, pois inicializa a tabela de simbolos. Esta 
**		tabela e' manipulada para armazenar ou apagar matrizes 
*/

void mod_init()
{
    tabsimb = &falso_no;	/* tabela de simbolos */

    falso_no.nome = "!falso_no!";
    falso_no.tipo = 0;
    falso_no.u.val = (Matriz_t *) 0;
    falso_no.proximo = (struct Simbolo *) 0;
    falso_no.previo = (struct Simbolo *) 0;

    instale("", 9999, (Matriz_t *) 0);	/* Indica inicio da lista */
    /* para efeito das matrizes geradas */
    /* (temporarias ou nao). Isto e' */
    /* interessante porque e' possivel */
    /* , se for necessario, colocar */
    /* matrizes fixas antes deste ponto */

    fsaida = stdout;		/* Saida sera' direcionada para video */
    _diretorio[0] = '\0';
}


/* 
** mod_atribuir(x, y) : utilizada para facilitar a tarefa de atribuicao de 
**		        matrizes. Atribui y `a matriz x, define x como matriz 
**		        nao temporaria (ID), libera as matrizes temporarias 
**			criadas pelo modulo (na execucao dos comandos), e,
**			finalmente, retorna a nova matriz x
**  Observacao: Esta funcao precisou ser desenvolvida porque as matrizes    
**		geradas nao possuem nome. Neste caso, toda matriz e' criada 
**		como temporaria para, depois, ser definido o seu tipo. Nao 
**		se deve usar a funcao atribua do "cm".			  
**
*/

Simbolo_t *mod_atribuir(x, y)
Simbolo_t *x;
Simbolo_t *y;
{
    /* libere o espaco ocupado pela matriz apontada por x */
    /* No caso dela ja' existir */

    if (x != (Simbolo_t *) NULL) {
	libsim(x);
    }

    /* faca uma copia da matriz apontada por y  e efetue a atribuicao */
    x = faztemp(NLINS(y), NCOLS(y));
    copmat(MATRIZ(x), MATRIZ(y));
    TIPO(x) = ID;

    return (x);
}

/* 
** execerro() : imprime uma mensagem de erro. Foi criada para garantir
**		compatibilidade com o "cm", dado que as funcoes do "cm" 
**		podem chamar a funcao execerro para imprimir "msgs" de 
**		erro. Observe que a funcao execerro do proprio "cm" nao 
**		poderia ser utilizada, pois ela utilizava outras variaveis 
**		globais e, tambem, executava um longjmp.
*/

void execerro(s, t)
char *s, *t;
{

    fprintf(stderr, "*** %s", s);

    if (t)
	fprintf(stderr, " %s", t);

    fprintf(stderr, "\n");
    setOSstdin();
    exit(1);
}


/* 
** mod_gs() : funcao intermediaria, chamada pela macro MOD_GS, para traducao
**	      da funcao GS do CM.
*/

void mod_gs(d1)
Simbolo_t *d1;
{

    md2 = faztemp(NLINS(d1), NCOLS(d1));

    if (NLINS(d1) >= NCOLS(d1)) {
	md1 = faztemp(NCOLS(d1), NCOLS(d1));

	mgs1(d1, md1, md2);
    } else {
	md1 = faztemp(NLINS(d1), NLINS(d1));

	mgs(d1, md1, md2);
    }
}

/*
** mod_criamat() : funcao que representa a funcao CRIAMAT do CM
*/

Simbolo_t *mod_criamat(nl, nc, x)
int nl, nc;
double x;
{
    int i, j;
    Simbolo_t *sr;

    if (nl <= 0 || nc <= 0)
	execerro("CRIAMAT", "nao posso criar matriz desta dimensao");

    sr = faztemp(nl, nc);

    for (i = 0; i < nl; i++) {
	for (j = 0; j < nc; j++) {
#ifdef EDIX
	    dp = &(DADOij(sr, i, j));
	    *dp = x;
#else
	    DADOij(sr, i, j) = x;
#endif
	}
    }

    return (sr);
}

/* 
** mod_meigen() : funcao intermediaria, chamada pela macro MOD_AUTO, para 
**		  traducao da funcao AUTO do CM.
*/

void mod_meigen(d1)
Simbolo_t *d1;
{
    int i, j;
    int n;
    int nm;
    int matz;

    nm = (int) NLINS(d1);
    n = nm;

    if (NLINS(d1) != NCOLS(d1)) {
	execerro("Auto", "Matriz nao e' quadrada.");
    }

    matz = 0;
    for (i = 0; i < n; ++i) {
	for (j = i; j < n; ++j) {
	    if (fuzz(DADOij(d1, i, j), DADOij(d1, j, i)) != 0) {
		matz = 1;
		break;
	    }
	}
    }

    md1 = faztemp(NLINS(d1), matz + 1);
    md2 = faztemp(NLINS(d1), NCOLS(d1));

    if (matz) {
	int ierr;
	int *iv1;
	double *fv1;
	double *A;
	double *wr;
	double *wi;
	double *z;

	if ((iv1 = (int *) malloc(nm * sizeof(int))) == (int *) NULL)
	    execerro("", "Memoria disponivel esgotada");

	if ((fv1 =
	     (double *) malloc(nm * sizeof(double))) == (double *) NULL)
	    execerro("", "Memoria disponivel esgotada");

	if ((z =
	     (double *) malloc(nm * nm * sizeof(double))) ==
	    (double *) NULL)
	    execerro("", "Memoria disponivel esgotada");

	if ((A =
	     (double *) malloc(nm * nm * sizeof(double))) ==
	    (double *) NULL)
	    execerro("", "Memoria disponivel esgotada");

	if ((wr =
	     (double *) malloc(nm * sizeof(double))) == (double *) NULL)
	    execerro("", "Memoria disponivel esgotada");

	if ((wi =
	     (double *) malloc(nm * sizeof(double))) == (double *) NULL)
	    execerro("", "Memoria disponivel esgotada");

	for (i = 0; i < NLINS((Simbolo_t *) d1); i++) {
	    iv1[i] = 0;
	    fv1[i] = 0.0;
	    for (j = 0; j < NLINS((Simbolo_t *) d1); j++) {
		A[i * NCOLS((Simbolo_t *) d1) + j] = (double)
		    DADOij((Simbolo_t *) d1, j, i);
		z[i * NCOLS((Simbolo_t *) d1) + j] = 0.0;
	    }
	}

	(void) rg_(nm, n, A, wr, wi, &matz, z, iv1, fv1, &ierr);


	for (i = 0; i < NLINS((Simbolo_t *) d1); i++) {
	    DADOij(md1, i, 0) = (double) wr[i];
	    DADOij(md1, i, 1) = (double) wi[i];
	}
	for (i = 0; i < NLINS(d1); i++) {
	    for (j = 0; j < NLINS(d1); j++) {
		DADOij(md2, j, i) = (double) z[i * NCOLS(d1) + j];
	    }
	}
	free((char *) z);
	free((char *) wr);
	free((char *) wi);
	free((char *) iv1);
	free((char *) fv1);
	free((char *) A);
    } else {
	meigen(d1, md1, md2);
    }
}


/*
** mod_mcata1() : representa a funcao CATA do CM
*/

Simbolo_t *mod_mcata1(d1, d2, d3, d4)
Simbolo_t *d1;
Simbolo_t *d2;
Simbolo_t *d3;
char *d4;
{
    Simbolo_t *sr;

    sr = faztemp((int) (mcata(d1, (Simbolo_t *) 0, d2, d3, 0, d4) + 1),
		 NCOLS(d1));

    mcata(d1, sr, d2, d3, 1, d4);

    return (sr);
}


/* 
** mod_gindex() : gera uma matriz linha temporaria cujos valores sao 
** 		  os numeros: n1, n1+1, ...., n2
*/

Simbolo_t *mod_gindex(n1, n2)
int n1, n2;
{
    Simbolo_t *s;
    int i;
    int j;

    s = faztemp(1, n2 - n1 + 1);	/* Cria s coma a dimensao 1x(n1-n2+1) */

    /* atribui valores `a matriz s */

    j = 0;
    for (i = n1; i <= n2; i++) {
	DADOij(s, 0, j) = (double) i;
	j++;
    }
    return (s);
}
