/***********************************************************************************
 MULTVAR - Módulo para análise de modelos lineares multivariados

 Direitos Autorais Reservados (c) 2004 Embrapa Informática Agropecuária

 Este programa é software livre; você pode redistribuí-lo e/ou modificá-lo sob os
 termos da Licença Pública Geral GNU conforme publicada pela Free Software
 Foundation; tanto a versão 2 da Licença, como (a seu critério) qualquer versão
 posterior.

 Este programa é distribuído na expectativa de que seja útil, porém, SEM NENHUMA
 GARANTIA; nem mesmo a garantia implícita de COMERCIABILIDADE OU ADEQUAÇÃO A UMA
 FINALIDADE ESPECÍFICA. Consulte a Licença Pública Geral do GNU para mais detalhes.

 Você deve ter recebido uma cópia da Licença Pública Geral do GNU junto com este
 programa; se não, escreva para a Free Software Foundation, Inc., no endereço 59
 Temple Street, Suite 330, Boston, MA 02111-1307 USA.

 **********************************************************************************

 Parte integrante do SOC - SOftware Científico

 Descrição:

	Rotina principal para controle de execucao do modulo de analise
	multivariada, feito de acordo com as macros MLMXLX.AJU, MLMXLX.DOC
	e MLMXLX.HIP, desenvolvidas com os comandos do modulo CM.

 Desenvolvido por:

	Maurício Noronha Festa
	Sônia Ternes 
	Sílvio Roberto Medeiros Evangelista

 Informações do CVS:
        $Source: /cvsroot/soc/soc/src/multvar/fontes/multvar.c,v $:
        $Revision: 1.1 $:
        $Date: 2005/03/24 16:08:44 $:

***********************************************************************************/
static char *sccsid = "@(#)multvar.c	3.7	(NTIA/modlin)	97/02/17";


#include "mod_incl.h"
#include "multvar.h"
#include "pcsgbd.h"

#include <string.h>
#include <stdlib.h>
#include <signal.h>

/*
** Variaveis globais usadas
*/
int flagA = 0;			/* execucao iterativa (fora do EXEC) */
int fl_X = 0;			/* permissao de execucao */
int fl_iter = 1;		/* execucao iterativa */
int fl_uni = 0;			/* realiza analise univariada no comando MEDREP */
int fl_uni_t = 0;		/* realiza analise univariada no comando TESTEM */
int fl_imp = 0;			/* redirecionamento da saida para a impressora */
int fl_analise;			/* tipo de analise a ser feita no MEDREP */
int imp_h = 0;			/* habilita impressao da matriz H */
int imp_e = 0;			/* habilita impressao da matriz E */
int imp_u = 1;			/* impressao da matriz U a primeira vez apenas */
int imp_er = 0;			/* habilita impressao da matriz E residual */
int imp_v = 0;			/* habilita impressao do vetor/raizes caracteristicas */
int ntit = 0;			/* controle de numero de titulos em titm */
int nrot = 0;			/* controle de numero de rotulos em rotm */

char titulo[96];		/* vetor para armazenar titulo (comando 'tlm') */
char rotulo[32];		/* vetor para armazenar rotulo (comando 'rotm') */
char string[9];			/* vetor para armazenar string do MEDREP */

char *nome_lin[MAXEFTS];	/* para impressao de nome de linhas */
char nome_col[TAM];		/* para impressao de nome de colunas */

#ifdef MSDOS
int nlinha = 25;		/* numero de linhas para montagem do relatorio */
#else
int nlinha = 24;
#endif

int ncoluna = 80;		/* numero de colunas para montagem do relatorio */

extern char exretstd[];		/* Contem o nome do arquivo exretstd */
extern int flagstd;

Efeitos efeitos;		/* efeitos especificados no comando mod */

/*
** Funcao Principal
*/
main(argc, argv)
int argc;
char *argv[];
{
    void controlC();

    int i;

    signal(SIGINT, controlC);

    /* Interpretacao de argv[] */
    /*
     ** O primeiro parametro de argv (argv[0]) ja eh parametro do programa.
     ** Portanto, este executavel so funciona se chamado a a partir de outro, mais 
     ** especificamente, do modulo modlin. 
     */

    while (argc--) {
	if (argv[0][0] == '-') {
	    for (i = 1; argv[0][i]; ++i) {

		switch (argv[0][i]) {

		case 'A':
		    flagA = 1;
#ifdef RS6000
		    flagstd = 1;
		    /*
		     *strchr(&argv[0][i+1], '|') = '\0';
		     */
		    if (*strchr(&argv[0][i + 1], '|'))
			*strchr(&argv[0][i + 1], '|') = '\0';

		    if (freopen(&argv[0][i + 1], "r", stdin) ==
			(FILE *) NULL) {

			i += (strlen(&argv[0][i + 1]) + 1);
			*strchr(&argv[0][i + 1], '|') = '\0';
			i += (strlen(&argv[0][i + 1]) + 1);
			strcpy(exretstd, &argv[0][i + 1]);

			fprintf(stderr, "Arquivo stdin nao abre.\n");
			setOSstdin();

#ifdef MEM_DEBUG
			if (Mem_Used() != 0) {
			    printf
				("*** Lista de memoria nao esta' vazia ***\n");
			    Mem_Store("multvar");
			    exit(2);
			}
#endif
			exit(5);
		    }
		    i += (strlen(&argv[0][i + 1]) + 1);

		    *strchr(&argv[0][i + 1], '|') = '\0';
		    fseek(stdin, atol(&argv[0][i + 1]), 0);

		    i += (strlen(&argv[0][i + 1]) + 1);
		    strcpy(exretstd, &argv[0][i + 1]);
		    i += strlen(&argv[0][i + 1]);

#endif
		    setbuf(stdin, (char *) NULL);
		    fl_iter = 0;

		    break;

		case 'X':

		    if (strcmp(&argv[0][i + 1], "ntia") == 0) {
			fl_X = 1;
			i += 4;
		    }

		    break;

		case 'l':

		    nlinha = atoi(&argv[0][i + 1]);
		    i += strlen(&argv[0][i + 1]);

		    break;

		case 'c':

		    ncoluna = atoi(&argv[0][i + 1]);
		    i += strlen(&argv[0][i + 1]);

		    break;

		case 'P':

		    fl_imp = 1;

		    break;
		}
	    }
	} else
	    break;
	++argv;
    }

    /* monta a estrutura Efeitos */
    mlmEfts();

    /* Chama a funcao para execucao do parser */
    mlmaju((char *) NULL);

    /* Teste para alocacao/liberacao de memoria */
    setOSstdin();
#ifdef MEM_DEBUG
    if (Mem_Used() != 0) {
	printf("*** Lista de memoria nao esta' vazia ***\n");
	Mem_Store("multvar");
	exit(2);
    }
#endif

    /* Finalizacao */
    exit(0);
}

/*
** Le o arquivo "xlxefts" e monta a estrutura "efeitos",
** inserindo o nome dos efeitos lidos.
*/

static char AREA[2048];		/* Area Reservada para armazenar nomes dos efeitos */

void mlmEfts()
{
    FILE *fp;
    int c, i, j;
    char buffer[256];
    static char *p2;

    p2 = AREA;

    fp = fopen("xlxefts", "r");
    if (fp == (FILE *) NULL) {
	fprintf(stderr, PegaMsg(5020), "xlxefts");
	setOSstdin();
#ifdef MEM_DEBUG
	if (Mem_Used() != 0) {
	    printf("*** Lista de memoria nao esta' vazia ***\n");
	    Mem_Store("multvar");
	    exit(2);
	}
#endif

	exit(2);
    }


    i = 0;
    j = 0;
    while ((c = fgetc(fp)) != EOF) {
	if (c == '\n' || c == '\r') {
	    buffer[j] = '\0';
	    if ((strlen(p2) + strlen(buffer) + 1) > 2048)
		mlmerro(17, 1);

	    strcpy(p2, buffer);
	    /*
	       efeitos.nome[i] = p2;
	     */
	    EFE_SetNome(efeitos, i, p2);

	    p2 += (strlen(p2) + 1);

	    i++;
	    j = 0;
	} else {
	    buffer[j] = c;
	    j++;
	}
    }
    EFE_SetNefe(efeitos, i);

    if (fclose(fp) == EOF) {
	fprintf(stderr, PegaMsg(5005), "xlxefts");
	setOSstdin();
#ifdef MEM_DEBUG
	if (Mem_Used() != 0) {
	    printf("*** Lista de memoria nao esta' vazia ***\n");
	    Mem_Store("multvar");
	    exit(2);
	}
#endif

	exit(2);
    }


    /*
     ** Apaga arquivo de entrada xlxefts com o nome dos efeitos
     */
    unlink("xlxefts");
}

/*
** Apaga arquivos intermediarios se interrompido pelo usuario
*/

void controlC(sig)
int sig;
{
    if (access("xlxefts", 0) != -1)
	unlink("xlxefts");

    if (access("xlxmat.dat", 0) != -1)
	unlink("xlxmat.dat");
    if (access("xlxmat.hdr", 0) != -1)
	unlink("xlxmat.hdr");

    if (access("xlxesp.dat", 0) != -1)
	unlink("xlxesp.dat");
    if (access("xlxesp.hdr", 0) != -1)
	unlink("xlxesp.hdr");

    if (access("xlxefe.dat", 0) != -1)
	unlink("xlxefe.dat");
    if (access("xlxefe.hdr", 0) != -1)
	unlink("xlxefe.hdr");

    if (flagA == 0)
	fprintf(stderr, "\n********  Interrompido pelo usuario\n");

    setOSstdin();
#ifdef MEM_DEBUG
    if (Mem_Used() != 0) {
	printf("***Lista de memoria nao esta vazia ***\n");
	Men_Store("multvar");
	exit(1);
    }
#endif

    exit(sig);
}
