/***********************************************************************************
 REGRESS - Anlise de Regresso Linear Univariada

 Direitos Autorais Reservados (c) 2004 Embrapa Informtica Agropecuria

 Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
 termos da Licena Pblica Geral GNU conforme publicada pela Free Software
 Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
 posterior.

 Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
 GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
 FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

 Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
 programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
 Temple Street, Suite 330, Boston, MA 02111-1307 USA.

 **********************************************************************************

 Parte integrante do SOC - SOftware Cientfico

 Descrio:

    O mdulo diagnostico calcula o diagnstico de regresso da observao lida
	segundo a teoria proposta por Belsley (1980), a partir das seguintes 
	variveis de entrada:   
		par : numero da var. dependente em analise
		nd1 : numero total de var. dependentes
		n   : numero de observacoes
		sd  : variancia do modelo completo 

 Desenvolvido por:

	  Dalton Francisco Andrade
	  Maurcio Noronha Festa
        Slvio Roberto Medeiros Evangelista
	  Snia Ternes Frassetto

 Informaes do CVS:
        $Source:
        $Revision:
        $Date:

***********************************************************************************/


#include "regress.h"
#include "diagnost.h"

#ifdef MEM_DEBUG
#include "mshell.h"
#endif

#include <stdlib.h>

static char *sccsid = "@(#)diagnost.c	3.4	(NTIA/regress)	95/09/15";



extern int fl_padrao;
extern double soma[MAXMEM];
extern double varia[MAXMEM];
extern char *obs1[];
extern int fl_padrao;
extern double soma[MAXMEM];
extern double varia[MAXMEM];
extern ARQ * fd;

static double saida1[MAXMEM];

/* ================== Variaveis globais usadas ============================ */

struct Diag  st_ci;	/* diagonal da matriz chapeu x'inv(X'X)x */
struct Diag  st_ri;	/* res. estudentizado internamente */
struct Diag  st_ti;	/* res. estudentizado externamente */
struct Diag  st_di;	/* estatistica D de Cook */
struct Diag  st_df;	/* estatistica DFFITs */

int pp;		/* numero de colunas da matriz X */

double ci;	/* diagonal da "matriz chapeu" */
double vci;	/* raiz quadrada de ci */

double sdi;	/* media quadratica dos residuos do modelo sem x[i,] */
double si;	/* raiz quadrada de sdi */

double s;	/* desvio padrao do modelo */

extern char *obs1[];		/* observacao lida */
extern char *efts[MAXMEM];

extern int nef;
extern int fl_int;
extern int fl_imp;
extern int dg1;
extern int dg2;
extern int dg;
extern int fl_dg;
extern int fl_dg1;

/* ====================================================================== */

double x_[MAXMEM];		/* linha lida da matriz X ou obs. lida */

void diagnostico(par,nd1,n,sd)
int	par;
int	nd1;
int	n;
double  sd;
{
	extern double confianca;
	extern char   obs2[MAXCLASS][80];
	unsigned int i=1;
	unsigned int n1;	/* numero de observacaoes */
	double res,ri;
	int flag=0;
	int flagcl, ix, j;
	
	rew_arq(fd);

	pp = nef + fl_int;	/* numero de colunas de X */
	n1 = n - 1;

	st_ci.pts = di_aloc();
	st_ci.vpt = dd_aloc();

	st_ri.pts = di_aloc();
	st_ri.vpt = dd_aloc();

	st_di.pts = di_aloc();
	st_di.vpt = dd_aloc();

	st_df.pts = di_aloc();
	st_df.vpt = dd_aloc();

	st_ti.pts = di_aloc();
	st_ti.vpt = dd_aloc();

	st_ci.ii  = 0;	st_ci.mem = TAM;
	st_di.ii  = 0;	st_di.mem = TAM;
	st_ri.ii  = 0;	st_ri.mem = TAM;
	st_df.ii  = 0;	st_df.mem = TAM;
	st_ti.ii  = 0;	st_ti.mem = TAM;

	st_ci.lim  = 2.0 * (double) pp / (double) n1;
	st_di.lim  = finv(1.0 - confianca/100.,(double) pp,(double) (n1-pp));
	st_ri.lim  = 2.0;
	st_df.lim  = 2.0 * sqrt((double) pp / (double) n1);
	st_ti.lim =  2.0;

	sd = sd / ((double) n1 - (double) pp);
	s  = sqrt(sd);

	if (fl_imp) {
		putchar(12);
	}
	else {
		printf("\n\n\n\n");
	}

	if (fl_dg1) {
		printf("\n                          Diagnostico  de  Regressao\n");
		printf("\n                        Variavel Dependente : %8s",efts[fl_int+par]);
		printf("\n\n                           Valores das Estatisticas");
		printf("\n\n\n  Obs             Ci             Ri             Ti         DCook        DFFits");
		printf("\n");
	}
	else {
		printf("\n	Diagnostico  de  Regressao\n");
		printf("\n	Variavel Dependente : %8s",efts[fl_int+par]);
	}


	saida[0] = 1.0;

	if (nby > 0)
		flagcl = 0;
	else
		flagcl = 1;

	i = 1;
	while (ler_obs(fd,obs1) !=  (long) EOF ) {
	    if (flagcl == 0) {
		for(j=0; j < nby; ++j) {
		    ix = ef_by[j];
		    if(strcmpi(obs1[ix],obs2[j])!=0)
			break;
		}
		if (j == nby)
		    flagcl = 1;
	    }

	    if (flagcl) {
		if (class() != 0)
			break;
		if (missing() == -1) {
		    expand_mat(nd1);

		    if (fl_dg1)
			printf("\n%5d",i);

		    x_calc(nd1);
		    res = residuo(nd1,par);

		    ci = x_ixlx_xl();
		    vci = 1.0 - ci;

		    sdi = (((double) n1 - (double) pp)*sd) /
			    (double) (n1-pp-1) - ((res*res) /
			    ((double) (n1-pp-1)*vci));
		    si = sqrt(sdi);

		    f_ci(i);
		    ri = f_ri(i,res);
		    f_ti(i,res);
		    f_dcook(i,res,ri);
		    f_dffits(i,res);
		    flag = 1;
		    ++i;
		}
	    }
	}
  
	if (flag) {
		if (fl_dg)
			imprime();
	}
}
	

void x_calc(nd1)
int nd1;
{
	int k,i;

	if (fl_int) {
		x_[0] = 1.0;
		k = 1;
	}
	else
		k = 0;

	for (i = fl_int+nd1 ; i <= nd1 +nef - !fl_int ; ++i) {
		x_[k++] = atof(obs1[ef[i][1]]);
	}
}


double residuo(nd1,par)	/* Calculo do residuo da obs. lida */
int nd1;
int par;
{
	int i;
	int k;
	double y,r;

	
	y = 0.0;

	for(i=0; i <= nd1+nef-!fl_int; ++i) {
		if ((ef[i][0] == -1) || (i == 0 && fl_int)) {
			k = sv[i];
			if (fl_padrao && i > 0) {
				if ((fabs(varia[i])) > 0.0) {
					saida1[k] = (saida[k] - (soma[i]/glt))/
						       sqrt(fabs(varia[i]));	
				}
				else {
					printf("\nO desvio padrao de uma das variaveis independentes e zero\n\n");
					setOSstdin();
#ifdef  MEM_DEBUG
					if (Mem_Used() != 0) {
						printf("*** Lista de memoria nao esta' vazia ***\n");
						Mem_Store("regress");
						exit(0);
					}
#endif
					exit(0);
				}
			} 
			else {
				saida1[k] = saida[k];
			}
		
			y += saida1[k]*xlx[idx(nxlx,k)];
		}
	}

	r = atof(obs1[ef[fl_int+par][1]]) - y;
	return(r);
}



double x_ixlx_xl()		/* Calculo de x*inv(X'X)*x' */
{
	double vx = 0.0;
	double x_ixlx;		/* resultado de x*inv(X'X) */
	int j,i;


	for (j=0; j < nxlx; ++j) {
		x_ixlx = 0.0;
		for (i=0; i < nxlx; ++i) {
			x_ixlx +=  x_[i] * xlx[idx(i,j)];
		}
		vx += x_ixlx * x_[j];
	}

	return(vx);
}

	
void imprime()
{

	extern int fl_int;
	extern int fl_imp;

	if (fl_dg1) {
		printf("\n\n\n");
	}
	else {
		printf("\n\n");
	}
		

	printf("\n	Pontos Influentes Segundo Estatistica :\n");

	printf("\n	Ci     : i-esimo elemento da diagonal da matriz 'chapeu'\n");
	imp_dg(st_ci);

	printf("\n	Ri     : residuo estudentizado internamente\n");
	imp_dg(st_ri);

	printf("\n	Ti     : residuo estudentizado externamente\n");
	imp_dg(st_ti);

	printf("\n	DCook  : D de Cook\n");
	imp_dg(st_di);

	printf("\n	DFFits : mudanca ocorrida no ajuste\n");
	imp_dg(st_df);

	printf("\n");
}

