
/***********************************************************************************
 STEPWISE - Anlise de Modelos de Regresso via mtodo stepwise

 Direitos Autorais Reservados (c) 2004 Embrapa Informtica Agropecuria

 Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
 termos da Licena Pblica Geral GNU conforme publicada pela Free Software
 Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
 posterior.

 Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
 GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
 FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

 Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
 programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
 Temple Street, Suite 330, Boston, MA 02111-1307 USA.

 **********************************************************************************

 Parte integrante do SOC - SOftware Cientfico

 Descrio:

 Desenvolvido por:

	Dalton Francisco Andrade
	Snia Ternes 
	Slvio Roberto Medeiros Evangelista

 Informaes do CVS:
        $Source: /cvsroot/soc/soc/src/stepwise/fontes/paramet.c,v $:
        $Revision: 1.1 $:
        $Date: 2005/03/22 19:17:18 $:

***********************************************************************************/


#include <stdio.h>
#include "stepwise.h"


static char *sccsid = "@(#)paramet.c	3.1	(NTIA/stepwise)	94/10/24";

/* ------------------------------------------------------------------------
** par_reg
**	Descricao      : calcula aos parametros referentes a regressao corrente
**	Rot. chamadas  : Nenhuma
**	Variaveis ent. : x1 : valor da soma de quadrados
**			 x2 : valor do quadrado medio
**			 valorf : esatistica F do paramentro
**			 a  : resultado da fprob
** ------------------------------------------------------------------------- */
void par_reg(x1, x2, valorf, a)
double *x1;
double *x2;
double *valorf;
double *a;
{
    extern double squad;

    int i;
    int cont = 0;

    for (i = 0; i < nxlx; ++i) {
	if (v[i] == -1) {
	    f_qm(i);
	    ++cont;
	}
	if (cont == nxlxs)
	    break;
    }

    sse[ultcol] = fabs(sse[ultcol]);

    qmr = (glr) ? sse[ultcol] / ((double) glr) : 0.0;	/* qm. do residuo */

    if (fl_int)
	(*x1) = (sse[0] - sse[ultcol]);
    else
	(*x1) = (squad - sse[ultcol]);

    (*x2) = (*x1) / ((double) glm);

    (*valorf) = (qmr) ? (*x2) / qmr : 9999.9999;
    if ((*valorf) > 10000.)
	(*valorf) = 9999.9999;

    (*a) = fprob((*valorf), (double) glm, (double) glr);
}

/* ------------------------------------------------------------------------
** poe_reg
**	Descricao      : Preenche as estruturas referentes a regressao
**			 com os valores calculados em par_reg
**	Variaveis ent. : x1 : valor da soma de quadrados
**			 x2 : valor do quadrado medio
**			 valorf : esatistica F do paramentro
**			 a  : resultado da fprob
** ------------------------------------------------------------------------- */

void poe_reg(x1, x2, valorf, a)
double x1;
double x2;
double valorf;
double a;
{
    extern double squad;

    reg.gl = glm;
    reg.squad = x1;
    reg.msquad = x2;
    reg.pF = a;
    reg.vF = valorf;
    res.gl = glr;
    res.squad = sse[ultcol];
    res.msquad = qmr;

    /* Coeficientes de determinacao */
    c_det = (x1 / ((fl_int) ? sse[0] : squad));
    c_aju = (1.0 - (1.0 - (x1 / ((fl_int) ? sse[0] : squad))) *
	     (glt - fl_int) / glr);
}


/* ------------------------------------------------------------------------
** poe_tabF
**	Descricao      : Preenche as estruturas referentes a tabela
**			 de F para sair das variaveis do modelo
**	Variaveis ent. :  Nenhuma
** ------------------------------------------------------------------- */

void poe_tabF()
{
    int j, t;
    int cont = 0;
    double x1, valorf, dp;
#ifdef EDIX
    double *p;
#endif

    t = 0;
    for (j = 0; j < nxlx; ++j) {
	if (v[j] == -1) {
	    x1 = xlx[idx(nxlx, j)];
	    valorf = obs[j] / qmr;
	    dp = sqrt(xlx[idx(j, j)] * qmr);

	    if ((j == 0) && (fl_int)) {
		Coef.vl = x1;
		Coef.dp = dp;
	    } else {
		Fsai[t].col = j;
#ifdef EDIX
		p = &(Fsai[t].vF);
		*p = valorf;
		p = &(Fsai[t].coef);
		*p = x1;
		p = &(Fsai[t].dp);
		*p = dp;
		x1 = fprob(valorf, 1.0, (double) glr);
		p = &(Fsai[t++].pF);
		*p = x1;
#else
		Fsai[t].vF = valorf;
		Fsai[t].coef = x1;
		Fsai[t].dp = dp;
		x1 = fprob(valorf, 1.0, (double) glr);
		Fsai[t++].pF = x1;
#endif
	    }
	    ++cont;
	}
	if (cont == nxlxs)
	    break;
    }
}

void f_qm(i)
int i;
{
    double x11, x22;

    x11 = xlx[idx(nxlx, i)];
    x22 = xlx[idx(i, i)];
    if (fabs(x22) < 1.0e-14)
	saida[i] = 0.0;
    else
	saida[i] = x11 * x11 / x22;

    obs[i] = saida[i];
}
