
/***********************************************************************************
 STEPWISE - Anlise de Modelos de Regresso via mtodo stepwise

 Direitos Autorais Reservados (c) 2004 Embrapa Informtica Agropecuria

 Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
 termos da Licena Pblica Geral GNU conforme publicada pela Free Software
 Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
 posterior.

 Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
 GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
 FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

 Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
 programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
 Temple Street, Suite 330, Boston, MA 02111-1307 USA.

 **********************************************************************************

 Parte integrante do SOC - SOftware Cientfico

 Descrio:

	Calcula os passos iterativos do stepwise, a partir da
	matriz xlx sweepada col. das var. do manter ou na
	coluna da varivel mais correlacionada com Y.

 Desenvolvido por:

	Dalton Francisco Andrade
	Snia Ternes 
	Slvio Roberto Medeiros Evangelista

 Informaes do CVS:
        $Source: /cvsroot/soc/soc/src/stepwise/fontes/stepwise.c,v $:
        $Revision: 1.1 $:
        $Date: 2005/03/22 19:17:18 $:

***********************************************************************************/

#include <stdio.h>
#include "stepwise.h"

static char *sccsid = "@(#)stepwise.c	3.1	(NTIA/stepwise)	94/10/24";

#ifdef MSDOS
int nlins = 25;
#else
int nlins = 24;
#endif

int ncols = 80;

int stepwise(m)
int m;				/* numero de observacoes utilizadas */
{
    int i, kk, k, s, fim, c;
    double Sq;			/* valor da soma de quad. do passo anterior */
    double maiorF;		/* valor do maior F para entrar */
    int iter;			/* no. de var. independentes a cada passo */
    int col;			/* coluna da var. que entrou no modelo */
    int modelo[MAXITER];	/* indica ituacao da var.no modelo :
				   -1 : ja esteve no modelo
				   1 : esta no modelo
				   0 : nao esta no modelo (candidata a entrar) */
    cop_mod(modelo);

    modelo[ultcol] = 1;
    iter = (nm) ? 1 : 2;
    nxlxs = fl_int + nm + iter;	/* dimensao de x'x */

    /* looping para os passos do stepwise */

    for (kk = 2; (kk < n_iter); ++kk) {
	if ((m - 1) < (nm + iter + 1)) {
	    printf
		("\n*** Numero de observacoes nao permite a inclusao de mais uma variavel. ***\n\n");
	    return 0;
	}

	/* looping para combinacoes de regs */

	fim = nxlx - fl_int - nm - iter;
	zera_F();
	maiorF = 0.0;
	col = 0;
	Sq = xlx[idx(nxlx, nxlx)];

	for (i = 0; i <= fim; ++i) {
	    k = 0;
	    if ((c = escolhe(i, modelo)) != -1)
		k = poe(c, modelo);

	    if (k) {		/* entrou variavel */
		ultcol = c;
		if (s = tab_F(c, Sq, &maiorF))
		    col = c;
	    }

	    if (c != -1) {
		/* retira ultima var independente do modelo */
		tira(c, modelo);
	    }

	}

	if (col) {		/* vai entrar uma variavel */
	    if (tenter != 0)
		ent_s();	/* F das var. fora do modelo */
	    putchr1();
	    ite_s(col, kk);	/* numero da iteracao */
	    anv_s();		/* quadro ANOVA */
	    sai_s();		/* F das var. dentro do modelo */

	    /* sweepa coluna da var. que entrou */
	    s = poe(col, modelo);
	    ultcol = col;

	    if (s = v_sai()) {	/* ha var para sair na 
				   presenca da que vai entrar */
		col = tira_var(s, &iter, modelo);
		putchr1();
		ites_s(++kk, col);
		anv_s();
		sai_s();
	    }

	    /* para proxima iteracao, dim. aumenta de 1 */
	    nxlxs++;
	    iter++;

	    cop_mod(modelo);	/* atualiza vetor modelo */
	} else {		/* nao ha variavel para entrar */
	    if (tenter != 0)
		ent_s();
	    break;
	}
    }				/* lopping de kk */
    return 1;
}				/* final da funcao */

int escolhe(i, modelo)
int i;
int *modelo;
{
    int j;
    int k = -1;

    for (j = fl_int + nm + i; j < np - 1; ++j) {
	if (modelo[j] == 0) {
	    k = j;
	    break;
	}
    }
    return (k);
}

void cop_mod(modelo)
int *modelo;
{
    int i;

    for (i = 0; i < nxlx; ++i) {
	if (v[i] == -1)
	    modelo[i] = 1;
	else
	    modelo[i] = 0;
    }
}
