
/***********************************************************************************
 STEPWISE - Anlise de Modelos de Regresso via mtodo stepwise

 Direitos Autorais Reservados (c) 2004 Embrapa Informtica Agropecuria

 Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
 termos da Licena Pblica Geral GNU conforme publicada pela Free Software
 Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
 posterior.

 Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
 GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
 FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

 Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
 programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
 Temple Street, Suite 330, Boston, MA 02111-1307 USA.

 **********************************************************************************

 Parte integrante do SOC - SOftware Cientfico

 Descrio:

 Desenvolvido por:

	Dalton Francisco Andrade
	Snia Ternes 
	Slvio Roberto Medeiros Evangelista

 Informaes do CVS:
        $Source: /cvsroot/soc/soc/src/stepwise/fontes/variav.c,v $:
        $Revision: 1.1 $:
        $Date: 2005/03/22 19:17:18 $:

***********************************************************************************/


#include <stdio.h>
#include "stepwise.h"

static char *sccsid = "@(#)variav.c	3.1	(NTIA/stepwise)	94/10/24";

/* ------------------------------------------------------------------------
** poe()
**	Descricao      : coloca var. no modelo
**	Rot. chamadas  : s_sweep()
**	Variaveis ent. : col : coluna da var. que entrara no modelo
**			 modelo : vetor com indices das var. no modelo
**	Variaveis sai. : resultado de s_sweep
**
 ------------------------------------------------------------------------ */

int poe(col, modelo)
int col;
int *modelo;
{
    modelo[col] = 1;
    return (s_sweep(col));
}

/* ------------------------------------------------------------------------
** tira()
**	Descricao      : retira var. no modelo
**	Rot. chamadas  : s_sweep()
**	Variaveis ent. : col : coluna da var. que saira do modelo
**			 modelo : vetor com indices das var. no modelo
 ------------------------------------------------------------------------ */

void tira(col, modelo)
int col;
int *modelo;
{
    modelo[col] = -1;
    s_sweep(col);
}

/* ------------------------------------------------------------------------
** v_ent
**	Descricao      : seleciona a var que entrara no modelo
**	Variaveis ent. : Fenter : estrutura referentes a var fora do modelo
**	Variaveis sai. : ent : posicao no arq_ntia da var a entrar
**			 -1  : se nenhuma var for entrar
** ------------------------------------------------------------------------- */
int v_ent()
{
    int i, ent;

    ent = -1;

    for (i = fl_int; i < tenter; ++i) {
	if (Fenter[i].pF > alfae) {
	    ent = Fenter[i].col;
	    break;
	}
    }
    return (ent);
}

/* ------------------------------------------------------------------------
** v_sai
**	Descricao      : verifica o numero de var que sairao do modelo
**	Rot. chamadas  : Nenhuma
**	Variaveis ent. : Fsai : estrutura referentes a var no modelo
**	Variaveis sai. : sai : numero de variaveis que sairao 
**			 0   : se nenhuma var for sair
** ------------------------------------------------------------------------- */
int v_sai()
{
    int i, sai;

    sai = 0;

    for (i = nm; i < (nxlxs - 1); ++i) {
	if (Fsai[i].pF > alfas) {
	    ++sai;
	}
    }
    return (sai);
}

/* ------------------------------------------------------------------------
** peg_pos
**	Descricao      : transforma a coluna de xlx na posicao
**			 correspondente nos vetores ef,efts
**	Variaveis ent. : var : posicao da variavel no arquivo_ntia
**	Variaveis sai. : j   : coluna da matriz xlx
** ------------------------------------------------------------------------- */
int peg_pos(col)
int col;
{
    if ((col == 0) && fl_int)
	return (0);
    else
	return (((col + 1) == ndim) ? fl_int : (col + 1));
}

/* -------------------------------------------------------------------------
** tira_var
**	Descricao      : "retira" as variaveis que sairao do modelo 
**			 sweepando as colunas correspondentes
**	Variaveis ent. : n : numero de variaveis que sirao do modelo
**			iter: numero de var. indep. no modelo
** ------------------------------------------------------------------------- */
int tira_var(n, iter, modelo)
int n;
int *iter;
int *modelo;
{
    int k;
    int nvez = 0;
    int col = -1;
    double menorf = 999.9;

    for (k = nm; k < (nxlxs - 1); ++k) {
	if (nvez < n) {
	    if (Fsai[k].pF > alfas) {
		if (Fsai[k].vF < menorf) {
		    menorf = Fsai[k].vF;
		    col = Fsai[k].col;
		}
		++nvez;
	    }
	} else {
	    break;
	}
    }
    /* sweepa col. da var. que vai sair */
    s_sweep(col);
    --nxlxs;
    --(*iter);
    ultcol = col;

    atu_Fsai();
    return (col);
}

void atu_Fsai()
{
    double x1, x2, valorf, a;

    glm = nxlxs - fl_int;
    glr = (glt - fl_int) - glm;

    /* inicializa estruturas */
    zera_F();

    /* calcula estat. da regressao */
    par_reg(&x1, &x2, &valorf, &a);

    /* guarda as estat. e calcula coef. de ajuste */
    poe_reg(x1, x2, valorf, a);

    /* preenche quadro de F para sair */
    poe_tabF();
}
